/*	RCT3 Tool
	Copyright 2005 Jonathan Wilson

	This file is part of the RCT3 Tool
	The RCT3 File Importer Program is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.

	Small modifications made by Klinn - May 2005 - search for 'by KLN'
	1) Added crude support for recolorable objects:
			- controls added to IDD_CREATESCENERY dialog box
			- 'Recolor' parameter added to SaveOVL function
		 Note that the object texture bitmap's colors will be replaced when recolored in-game:
			- color 0 = not known, avoid use
			- colors 1-85 = re-color #1, shades from light to dark
			- colors 86-170 = re-color #2, light to dark
			- colors 171-255 = re-color #3, light to dark
		 A suitable custom indexed color table should be created and used when converting
		 an RGB-format image to indexed 256-color mode, so that the results can be more
		 easily visualized. Frontier appears to have used shades of blue for 1-85, magenta
		 for 86-170, and yellow for 171-255. Suggest leaving color 0 = white.
	2) Changed Texture Style from an edit box to a combobox.
	3) Added confirmation MessageBox after theme is successfully installed.
	4) Revised the GUISkinItemPos struct, swapped the 'Left' and 'Top' members, and
	swapped the 'Right' and 'Bottom' members, so that game will pick up correct coords.

	Bug fixes by Belgabor:
    - Random install behaviour
    - Texture file handles left open.

*/

#define _WIN32_WINNT 0x0500

#pragma warning (disable: 4100)
#include "libOVL\StyleOVL.h"
#include "libOVL\OVL.h"
#include "libOVL\Common.h"
#include "libASE-1.0\libase.h"
//#include "png.h"
#include <IL/il.h>
#include <IL/ilu.h>
//#include "ati_compress.h"
#include <float.h>
#include <commdlg.h>
#include <shlobj.h>
#include <dirent.h>
#include <math.h>
#include "resource.h"

HINSTANCE hInst;
unsigned long scn_version = 2;
#define DEFAULT_SCN_FIXORIENT 0L
#define TITLE_VERSION "RCT3 Object Importer (v14, modified by DragonsIOA & Belgabor [v1.4.6])"

RGBQUAD recolpalette_bmy[256];
RGBQUAD recolpalette_rgb[256];

char *il_bgr = "BGR";
char *il_bgra = "BGRA";
char *il_colour_index = "Colour Index";
char *il_luminance = "Luminance";
char *il_luminance_alpha = "Luminance Alpha";
char *il_rgb = "RGB";
char *il_rgba = "RGBA";
char *il_byte = "Byte";
char *il_unsigned_byte = "Unsigned Byte";
char *il_short = "Short";
char *il_unsigned_short = "Unsigned Short";
char *il_int = "Int";
char *il_unsigned_int = "Unsigned Int";
char *il_float = "Float";
char *il_double = "Double";
char *il_pal_none = "None";
char *il_pal_rgb24 = "RGB24";
char *il_pal_rgb32 = "RGB32";
char *il_pal_rgba32 = "RGBA32";
char *il_pal_bgr24 = "BGR24";
char *il_pal_bgr32 = "BGR32";
char *il_pal_bgra32 = "BGRA32";
char *il_origin_lower_left = "Lower Left";
char *il_origin_upper_left = "Upper Left";
char *il_unknown = "Unknown";

char *getILString(ILenum x) {
    switch (x) {
    case IL_COLOUR_INDEX:
        return il_colour_index;
    case IL_RGB:
        return il_rgb;
    case IL_RGBA:
        return il_rgba;
    case IL_BGR:
        return il_bgr;
    case IL_BGRA:
        return il_bgra;
    case IL_LUMINANCE:
        return il_luminance;
    case IL_LUMINANCE_ALPHA:
        return il_luminance_alpha;
    case IL_BYTE:
        return il_byte;
    case IL_UNSIGNED_BYTE:
        return il_unsigned_byte;
    case IL_SHORT:
        return il_short;
    case IL_UNSIGNED_SHORT:
        return il_unsigned_short;
    case IL_INT:
        return il_int;
    case IL_UNSIGNED_INT:
        return il_unsigned_int;
    case IL_FLOAT:
        return il_float;
    case IL_DOUBLE:
        return il_double;
    case IL_PAL_NONE:
        return il_pal_none;
    case IL_PAL_RGB24:
        return il_pal_rgb24;
    case IL_PAL_RGB32:
        return il_pal_rgb32;
    case IL_PAL_RGBA32:
        return il_pal_rgba32;
    case IL_PAL_BGR24:
        return il_pal_bgr24;
    case IL_PAL_BGR32:
        return il_pal_bgr32;
    case IL_PAL_BGRA32:
        return il_pal_bgra32;
    case IL_ORIGIN_LOWER_LEFT:
        return il_origin_lower_left;
    case IL_ORIGIN_UPPER_LEFT:
        return il_origin_upper_left;
    default:
        return il_unknown;
    }
}
BOOL CALLBACK InfoDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG: {
        SetDlgItemText(hwnd, IDC_INFO, (char *) lParam);
        //SetDlgItemText(hwnd, IDC_INFO, "Test\r\nTest");
    }
    break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDOK: {
            EndDialog(hwnd, 1);
        }
        break;
        default:
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

void showBitmapInfo(HWND parent, char *filename) {
    ILuint bmp;
    ILenum Error;
    char res[4000];
    char tmp[256];

    snprintf(res, 4000, "File: %s\r\n", filename);

    ilGenImages(1, &bmp);
    ilBindImage(bmp);

    if (!ilLoadImage(filename)) {
        snprintf(res, 4000, "%sFile open error. devIL messages:\r\n", res);
        while ((Error = ilGetError())) {
            snprintf(res, 4000, "%s  %s\r\n", res, iluErrorString(Error));
        }
    } else {
        snprintf(res, 4000, "%sSize: %dx%d\r\n", res, ilGetInteger(IL_IMAGE_WIDTH),
                 ilGetInteger(IL_IMAGE_HEIGHT));
        snprintf(res, 4000, "%sOrigin: %s\r\n", res, getILString(ilGetInteger(IL_IMAGE_ORIGIN)));
        snprintf(res, 4000, "%sFormat: %s\r\n", res, getILString(ilGetInteger(IL_IMAGE_FORMAT)));
        snprintf(res, 4000, "%sType: %s\r\n", res, getILString(ilGetInteger(IL_IMAGE_TYPE)));
        snprintf(res, 4000, "%sBits per Pixel: %d\r\n", res, ilGetInteger(IL_IMAGE_BITS_PER_PIXEL));
        snprintf(res, 4000, "%sColours: %d\r\n", res, iluColoursUsed());
        snprintf(res, 4000, "%sPalette Type: %s\r\n", res,
                 getILString(ilGetInteger(IL_PALETTE_TYPE)));
        snprintf(res, 4000, "%sPalette Bytes per Pixel: %d\r\n", res, ilGetInteger(IL_PALETTE_BPP));
        snprintf(res, 4000, "%sPalette Colours: %d\r\n", res, ilGetInteger(IL_PALETTE_NUM_COLS));
    }
    DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_INFO), parent, InfoDlgProc, (LPARAM) & res);
    //MessageBox(parent, res, "Info", MB_OK | MB_ICONINFORMATION);
    ilDeleteImages(1, &bmp);
}

HWND SceneryDlgHwnd;
unsigned char *data, *data2;
bool save = false;
//BITMAPFILEHEADER bmf;
//BITMAPINFOHEADER bmi;
//BITMAPFILEHEADER bmf2;
//BITMAPINFOHEADER bmi2;
unsigned long ShapeSize;
long CurrentTextString = -1;
long CurrentIcon = -1;
long CurrentIconTexture = -1;
long CurrentScenery = -1;
long CurrentEffectPoint = -1;
struct FlexiTexture {
    char *Name;
    char *Texture;
    char *Alpha;
    unsigned long Recolorable;
};
struct MeshStruct {
    BOOL disabled;
    char *TXS;
    char *FTX;
    unsigned long place;
    unsigned long flags;
    unsigned long unknown;
    char *Name;
    BOOL effectpoint;
    ASE_Vector3D effectpoint_vert;
};
std::vector < EffectPoint * >::iterator EffectPointsIterator;
std::vector < Text * >::iterator TextStringsIterator;
std::vector < Icon * >::iterator IconsIterator;
std::vector < IconTexture * >::iterator IconTexturesIterator;
std::vector < Scenery * >::iterator SceneryItemsIterator;
std::vector < FlexiTexture * >FlexiTextures;
std::vector < FlexiTexture * >::iterator FlexiTexturesIterator;
std::vector < MeshStruct * >MeshStructs;
std::vector < MeshStruct * >::iterator MeshStructsIterator;
long
CurrentMeshStruct = -1;
long
CurrentFlexiTexture = -1;
long
CurrentReference = -1;
long
CurrentFixOrient = DEFAULT_SCN_FIXORIENT;
std::vector < char *>::iterator ReferencesIterator;
bool scenerysave = false;
char *stdstyle = "Style\\Themed\\";

unsigned long local_log2(unsigned long y);
bool InstallTheme(HWND hwnd);
bool ReadIconTexture(IconTexture * t);
char *LastCreatedOVL = NULL;
COLORREF g_colors[16];

void ConvertASEMesh(StaticShape1 * ss, StaticShape2 * ss2, ASE_Mesh * pase) {
    ss2->IndexCount = pase->faceCount * 3;
    unsigned long *Triangles = new unsigned long[ss2->IndexCount];
    ss2->Triangles = Triangles;
    // we don't know the vertex count yet

    std::vector < VERTEX > camVerts;

    VERTEX tv;
    long i, j;

    for (i = 0; i < pase->vertexCount; i++) {
        if (pase->vertices[i].x < ss->BoundingBox1.x) {
            ss->BoundingBox1.x = pase->vertices[i].x;
        }
        if (pase->vertices[i].y < ss->BoundingBox1.y) {
            ss->BoundingBox1.y = pase->vertices[i].y;
        }
        if (pase->vertices[i].z < ss->BoundingBox1.z) {
            ss->BoundingBox1.z = pase->vertices[i].z;
        }
        if (pase->vertices[i].x > ss->BoundingBox2.x) {
            ss->BoundingBox2.x = pase->vertices[i].x;
        }
        if (pase->vertices[i].y > ss->BoundingBox2.y) {
            ss->BoundingBox2.y = pase->vertices[i].y;
        }
        if (pase->vertices[i].z > ss->BoundingBox2.z) {
            ss->BoundingBox2.z = pase->vertices[i].z;
        }

    }

    /*  Vertexes[i].position.x = curobj.vertices[i].x;
       Vertexes[i].position.y = curobj.vertices[i].y;
       Vertexes[i].position.z = curobj.vertices[i].z;
       Vertexes[i].color = 0xFFFFFFFF;
       Vertexes[i].normal.x = curobj.vertex_normals[i].x;
       Vertexes[i].normal.y = curobj.vertex_normals[i].y;
       Vertexes[i].normal.z = curobj.vertex_normals[i].z;
       Vertexes[i].tu = 0;
       Vertexes[i].tv = 0;
       }
     */
    for (i = 0; i < pase->faceCount * 3; i += 3) {
        UINT index = i / 3;
        UINT tvs, tts;
        BOOL add;

        tvs = pase->faces[index].vertex[0];
        tts = pase->faces[index].texture_coordinates[0];

        tv.position.x = pase->vertices[tvs].x;
        tv.position.y = pase->vertices[tvs].y;
        tv.position.z = pase->vertices[tvs].z;
        tv.color = 0xffffffff;
        tv.normal.x = pase->vertex_normals[tvs].x;
        tv.normal.y = pase->vertex_normals[tvs].y;
        tv.normal.z = pase->vertex_normals[tvs].z;
        tv.tu = pase->texture_coordinates[tts].x;
        tv.tv = pase->texture_coordinates[tts].y;

        // now see if we have already added this point
        add = TRUE;
        for (j = 0; j < (long) camVerts.size(); ++j) {
            VERTEX *pv = &camVerts[j];
            if (memcmp(pv, &tv, sizeof(VERTEX)) == 0) {
                // we have a match so exit
                add = FALSE;
                break;
            }
        }
        if (add) {
            camVerts.push_back(tv);
        }
        // j should have our real index value now as well
        Triangles[i] = j;

        // now for the second one
        tvs = pase->faces[index].vertex[1];
        tts = pase->faces[index].texture_coordinates[1];

        tv.position.x = pase->vertices[tvs].x;
        tv.position.y = pase->vertices[tvs].y;
        tv.position.z = pase->vertices[tvs].z;
        tv.color = 0xffffffff;
        tv.normal.x = pase->vertex_normals[tvs].x;
        tv.normal.y = pase->vertex_normals[tvs].y;
        tv.normal.z = pase->vertex_normals[tvs].z;
        tv.tu = pase->texture_coordinates[tts].x;
        tv.tv = pase->texture_coordinates[tts].y;

        // now see if we have already added this point
        add = TRUE;
        for (j = 0; j < (long) camVerts.size(); ++j) {
            VERTEX *pv = &camVerts[j];
            if (memcmp(pv, &tv, sizeof(VERTEX)) == 0) {
                // we have a match so exit
                add = FALSE;
                break;
            }
        }
        if (add) {
            camVerts.push_back(tv);
        }
        // j should have our real index value now as well
        Triangles[i + 1] = j;

        // now for the second one
        tvs = pase->faces[index].vertex[2];
        tts = pase->faces[index].texture_coordinates[2];

        tv.position.x = pase->vertices[tvs].x;
        tv.position.y = pase->vertices[tvs].y;
        tv.position.z = pase->vertices[tvs].z;
        tv.color = 0xffffffff;
        tv.normal.x = pase->vertex_normals[tvs].x;
        tv.normal.y = pase->vertex_normals[tvs].y;
        tv.normal.z = pase->vertex_normals[tvs].z;
        tv.tu = pase->texture_coordinates[tts].x;
        tv.tv = pase->texture_coordinates[tts].y;

        // now see if we have already added this point
        add = TRUE;
        for (j = 0; j < (long) camVerts.size(); ++j) {
            VERTEX *pv = &camVerts[j];
            if (memcmp(pv, &tv, sizeof(VERTEX)) == 0) {
                // we have a match so exit
                add = FALSE;
                break;
            }
        }
        if (add) {
            camVerts.push_back(tv);
        }
        // j should have our real index value now as well
        Triangles[i + 2] = j;
    }

    // we now should have a full list of completed vertices
    UINT vertcnt = (UINT) camVerts.size();
    ss2->VertexCount = vertcnt;
    if (vertcnt) {
        VERTEX *Vertexes = new VERTEX[vertcnt];
        for (i = 0; i < (long) vertcnt; ++i) {
            VERTEX *pv = &camVerts[i];
            memcpy(&Vertexes[i], pv, sizeof(VERTEX));
        }
        ss2->Vertexes = Vertexes;
    }
    ss->TotalIndexCount += ss2->IndexCount;
    ss->TotalVertexCount += ss2->VertexCount;

    /*

       sh->TotalIndexCount += curobj.faceCount*3;
       sh->TotalVertexCount += curobj.vertexCount;
       CurrentMesh++;

       }
     */
}

BOOL CALLBACK EffectPointParticleDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG: {
        SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST, LB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST, LB_ADDSTRING, 0,
                           (LPARAM) "(Enter Manually)");
        HKEY key;
        LONG res = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\{907B4640-266B-4A21-92FB-CD1A86CD0F63}",
                                0, KEY_QUERY_VALUE, &key);
        char InstallLocation[MAX_PATH] = "";
        char Location[MAX_PATH] = "";
        unsigned long length = MAX_PATH;
        unsigned long i;
        res =
            RegQueryValueEx(key, "InstallLocation", 0, NULL, (LPBYTE) InstallLocation, &length);
        res = RegCloseKey(key);
        sprintf(Location, "%s\\ParticleEffects\\", InstallLocation);
        int n;
        DIR *d = opendir(Location);
        struct dirent *e;
        while (e = readdir(d)) {
            char t[MAX_PATH];
            char ex[MAX_PATH];
            _splitpath(e->d_name, NULL, NULL, t, ex);
            if (!stricmp(ex, ".frw"))
                SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST, LB_ADDSTRING, 0, (LPARAM) t);
        }
        closedir(d);
        SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST, LB_SETCURSEL, 0, 0);
    }
    break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDOK: {
            long sel = SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST,
                                          LB_GETCURSEL, 0, 0);
            char t[MAX_PATH + 18];
            strcpy(t, "particlestart01_");
            if (sel <= 0)
                strcat(t, "enternameofparticleeffecthere");
            else {
                char t2[MAX_PATH];
                SendDlgItemMessage(hwnd, IDC_EFFECT_PARTICLE_LIST, LB_GETTEXT, sel,
                                   (LPARAM) t2);
                for (int i = 0; i < strlen(t2); i++)
                    t2[i] = tolower(t2[i]);
                strcat(t, t2);
            }
            SetWindowText(GetDlgItem(GetParent(hwnd), IDC_NAME), t);
            EndDialog(hwnd, 1);
        }
        break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        default:
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

void MakeLight(HWND hdlg, HWND htarget) {
    char temp[256];
    int f = 0;
    long sel = SendDlgItemMessage(hdlg, IDC_LIGHT_TYPE, CB_GETCURSEL, 0, 0);
    switch (sel) {
    case 1:
        strcpy(temp, "simplelightstart01");
        f = 16;
        break;
    case 2:
        strcpy(temp, "nblightstart01");
        f = 12;
        break;
    default:
        strcpy(temp, "lightstart01");
        f = 10;
        break;
    }
    sel = SendDlgItemMessage(hdlg, IDC_LIGHT_OPTION, CB_GETCURSEL, 0, 0);
    switch (sel) {
    case 1:
        strcat(temp, "_lamp");
        break;
    case 2:
        strcat(temp, "_torch");
        break;
    case 3:
        strcat(temp, "_flexi0");
        break;
    case 4:
        strcat(temp, "_flexi1");
        break;
    case 5:
        strcat(temp, "_flexi2");
        break;
    default:
        sel = 0;
        break;
    }
    int l = 0;
    char ptemp[5];
    if (sel < 3) {
        GetDlgItemText(hdlg, IDC_LIGHT_COLOR_R, ptemp, 4);
        strcat(temp, "_r");
        if (!strcmp(ptemp, ""))
            strncat(temp, "0", 255);
        else
            strncat(temp, ptemp, 255);
        strncat(temp, "g", 255);
        GetDlgItemText(hdlg, IDC_LIGHT_COLOR_G, ptemp, 4);
        if (!strcmp(ptemp, ""))
            strncat(temp, "0", 255);
        else
            strncat(temp, ptemp, 255);
        strncat(temp, "b", 255);
        GetDlgItemText(hdlg, IDC_LIGHT_COLOR_B, ptemp, 4);
        if (!strcmp(ptemp, ""))
            strncat(temp, "0", 255);
        else
            strncat(temp, ptemp, 255);
    }
    strncat(temp, "_radius", 255);
    GetDlgItemText(hdlg, IDC_LIGHT_COLOR_RADIUS, ptemp, 4);
    if (!strcmp(ptemp, ""))
        strncat(temp, "0", 255);
    else
        strncat(temp, ptemp, 255);
    if (SendDlgItemMessage(hdlg, IDC_LIGHT_COLOR_CM, BM_GETCHECK, 0, 0))
        strncat(temp, "cm", 255);
    SetWindowText(htarget, temp);
    SendMessage(htarget, EM_SETSEL, f, f + 2);
}

BOOL CALLBACK EffectPointLightDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG: {
        SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_ADDSTRING, 0, (LPARAM) "Normal");
        SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_ADDSTRING, 0, (LPARAM) "Simple");
        SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_ADDSTRING, 0, (LPARAM) "No Bulb");
        SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_SETCURSEL, 0, 0);
        CheckDlgButton(hwnd, IDC_LIGHT_SHINE, true);
        CheckDlgButton(hwnd, IDC_LIGHT_BULB, true);
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "None");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "Lamp");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "Torch");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "Custom Color 1");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "Custom Color 2");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_ADDSTRING, 0, (LPARAM) "Custom Color 3");
        SendDlgItemMessage(hwnd, IDC_LIGHT_OPTION, CB_SETCURSEL, 0, 0);
        SetDlgItemText(hwnd, IDC_LIGHT_COLOR_R, "255");
        SetDlgItemText(hwnd, IDC_LIGHT_COLOR_G, "255");
        SetDlgItemText(hwnd, IDC_LIGHT_COLOR_B, "255");
        SetDlgItemText(hwnd, IDC_LIGHT_COLOR_RADIUS, "10");
        CheckDlgButton(hwnd, IDC_LIGHT_COLOR_CM, false);
        MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
    }
    break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_LIGHT_TYPE: {
            long sel = SendDlgItemMessage(hwnd, IDC_LIGHT_TYPE, CB_GETCURSEL,
                                          0, 0);
            CheckDlgButton(hwnd, IDC_LIGHT_SHINE, (sel == 0)
                           || (sel == 2));
            CheckDlgButton(hwnd, IDC_LIGHT_BULB, (sel == 0)
                           || (sel == 1));

            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
        }
        break;
        case IDC_LIGHT_SHINE:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_BULB:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_OPTION:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_COLOR: {
            CHOOSECOLOR cc;
            char ptemp[5];
            GetDlgItemText(hwnd, IDC_LIGHT_COLOR_R, ptemp, 4);
            int r = atoi(ptemp);
            GetDlgItemText(hwnd, IDC_LIGHT_COLOR_G, ptemp, 4);
            int g = atoi(ptemp);
            GetDlgItemText(hwnd, IDC_LIGHT_COLOR_B, ptemp, 4);
            int b = atoi(ptemp);
            ZeroMemory(&cc, sizeof(cc));
            cc.lStructSize = sizeof(cc);
            cc.hwndOwner = hwnd;
            cc.rgbResult = RGB(r, g, b);
            cc.lpCustColors = (COLORREF *) & g_colors;
            cc.Flags = CC_ANYCOLOR | CC_FULLOPEN | CC_RGBINIT;
            if (ChooseColor(&cc)) {
                snprintf(ptemp, 4, "%d", GetRValue(cc.rgbResult));
                SetDlgItemText(hwnd, IDC_LIGHT_COLOR_R, ptemp);
                snprintf(ptemp, 4, "%d", GetGValue(cc.rgbResult));
                SetDlgItemText(hwnd, IDC_LIGHT_COLOR_G, ptemp);
                snprintf(ptemp, 4, "%d", GetBValue(cc.rgbResult));
                SetDlgItemText(hwnd, IDC_LIGHT_COLOR_B, ptemp);
                MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            }
        }
        break;
        case IDC_LIGHT_COLOR_R:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_COLOR_G:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_COLOR_B:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_COLOR_RADIUS:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDC_LIGHT_COLOR_CM:
            MakeLight(hwnd, GetDlgItem(hwnd, IDC_LIGHT_PREVIEW));
            break;
        case IDOK:
            MakeLight(hwnd, GetDlgItem(GetParent(hwnd), IDC_NAME));
            EndDialog(hwnd, 1);
            break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        default:
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

void do_rot_x(HWND hwnd, double angle) {
    SetDlgItemText(hwnd, IDC_M11, "1");
    SetDlgItemText(hwnd, IDC_M12, "0");
    SetDlgItemText(hwnd, IDC_M13, "0");
    SetDlgItemText(hwnd, IDC_M21, "0");
    SetDlgItemText(hwnd, IDC_M31, "0");
    char temp[100];

    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M22, temp);
    sprintf(temp, "%f", -sin(angle));
    SetDlgItemText(hwnd, IDC_M23, temp);
    sprintf(temp, "%f", sin(angle));
    SetDlgItemText(hwnd, IDC_M32, temp);
    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M33, temp);
}

void do_rot_y(HWND hwnd, double angle) {
    SetDlgItemText(hwnd, IDC_M22, "1");
    SetDlgItemText(hwnd, IDC_M21, "0");
    SetDlgItemText(hwnd, IDC_M23, "0");
    SetDlgItemText(hwnd, IDC_M12, "0");
    SetDlgItemText(hwnd, IDC_M32, "0");
    char temp[100];

    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M11, temp);
    sprintf(temp, "%f", sin(angle));
    SetDlgItemText(hwnd, IDC_M13, temp);
    sprintf(temp, "%f", -sin(angle));
    SetDlgItemText(hwnd, IDC_M31, temp);
    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M33, temp);
}

void do_rot_z(HWND hwnd, double angle) {
    SetDlgItemText(hwnd, IDC_M33, "1");
    SetDlgItemText(hwnd, IDC_M13, "0");
    SetDlgItemText(hwnd, IDC_M23, "0");
    SetDlgItemText(hwnd, IDC_M31, "0");
    SetDlgItemText(hwnd, IDC_M32, "0");
    char temp[100];

    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M11, temp);
    sprintf(temp, "%f", -sin(angle));
    SetDlgItemText(hwnd, IDC_M12, temp);
    sprintf(temp, "%f", sin(angle));
    SetDlgItemText(hwnd, IDC_M21, temp);
    sprintf(temp, "%f", cos(angle));
    SetDlgItemText(hwnd, IDC_M22, temp);
}

BOOL CALLBACK EffectPointDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG: {
        if (CurrentEffectPoint != -1) {
            EffectPoint *e = EffectPoints[CurrentEffectPoint];
            char temp[100];
            sprintf(temp, "%f", e->Transform._11);
            SetDlgItemText(hwnd, IDC_M11, temp);
            sprintf(temp, "%f", e->Transform._12);
            SetDlgItemText(hwnd, IDC_M12, temp);
            sprintf(temp, "%f", e->Transform._13);
            SetDlgItemText(hwnd, IDC_M13, temp);
            sprintf(temp, "%f", e->Transform._14);
            SetDlgItemText(hwnd, IDC_M14, temp);
            sprintf(temp, "%f", e->Transform._21);
            SetDlgItemText(hwnd, IDC_M21, temp);
            sprintf(temp, "%f", e->Transform._22);
            SetDlgItemText(hwnd, IDC_M22, temp);
            sprintf(temp, "%f", e->Transform._23);
            SetDlgItemText(hwnd, IDC_M23, temp);
            sprintf(temp, "%f", e->Transform._24);
            SetDlgItemText(hwnd, IDC_M24, temp);
            sprintf(temp, "%f", e->Transform._31);
            SetDlgItemText(hwnd, IDC_M31, temp);
            sprintf(temp, "%f", e->Transform._32);
            SetDlgItemText(hwnd, IDC_M32, temp);
            sprintf(temp, "%f", e->Transform._33);
            SetDlgItemText(hwnd, IDC_M33, temp);
            sprintf(temp, "%f", e->Transform._34);
            SetDlgItemText(hwnd, IDC_M34, temp);
            sprintf(temp, "%f", e->Transform._41);
            SetDlgItemText(hwnd, IDC_M41, temp);
            sprintf(temp, "%f", e->Transform._42);
            SetDlgItemText(hwnd, IDC_M42, temp);
            sprintf(temp, "%f", e->Transform._43);
            SetDlgItemText(hwnd, IDC_M43, temp);
            sprintf(temp, "%f", e->Transform._44);
            SetDlgItemText(hwnd, IDC_M44, temp);
            SetDlgItemText(hwnd, IDC_NAME, e->Name);
        } else {
            SetDlgItemText(hwnd, IDC_M11, "1");
            SetDlgItemText(hwnd, IDC_M12, "0");
            SetDlgItemText(hwnd, IDC_M13, "0");
            SetDlgItemText(hwnd, IDC_M14, "0");
            SetDlgItemText(hwnd, IDC_M21, "0");
            SetDlgItemText(hwnd, IDC_M22, "1");
            SetDlgItemText(hwnd, IDC_M23, "0");
            SetDlgItemText(hwnd, IDC_M24, "0");
            SetDlgItemText(hwnd, IDC_M31, "0");
            SetDlgItemText(hwnd, IDC_M32, "0");
            SetDlgItemText(hwnd, IDC_M33, "1");
            SetDlgItemText(hwnd, IDC_M34, "0");
            SetDlgItemText(hwnd, IDC_M41, "0");
            SetDlgItemText(hwnd, IDC_M42, "0");
            SetDlgItemText(hwnd, IDC_M43, "0");
            SetDlgItemText(hwnd, IDC_M44, "1");
        }
        BOOL haspoints = false;
        SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB, CB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB, CB_ADDSTRING, 0,
                           (LPARAM) "Reset Defaults");
        for (MeshStructsIterator = MeshStructs.begin();
                MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
            if ((*MeshStructsIterator)->effectpoint) {
                SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB, CB_ADDSTRING, 0,
                                   (LPARAM) (*MeshStructsIterator)->Name);
                haspoints = true;
            }
        }
        SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB, CB_SETCURSEL, 0, 0);
        EnableWindow(GetDlgItem(hwnd, IDC_EFFECT_POINTS_CB), haspoints);

    }
    break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        case IDC_EFFECT_POINTS_AUTO: {
            long sel = SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB,
                                          CB_GETCURSEL, 0,
                                          0);
            if (sel == 0) {
                SetDlgItemText(hwnd, IDC_M11, "1");
                SetDlgItemText(hwnd, IDC_M12, "0");
                SetDlgItemText(hwnd, IDC_M13, "0");
                SetDlgItemText(hwnd, IDC_M14, "0");
                SetDlgItemText(hwnd, IDC_M21, "0");
                SetDlgItemText(hwnd, IDC_M22, "1");
                SetDlgItemText(hwnd, IDC_M23, "0");
                SetDlgItemText(hwnd, IDC_M24, "0");
                SetDlgItemText(hwnd, IDC_M31, "0");
                SetDlgItemText(hwnd, IDC_M32, "0");
                SetDlgItemText(hwnd, IDC_M33, "1");
                SetDlgItemText(hwnd, IDC_M34, "0");
                SetDlgItemText(hwnd, IDC_M41, "0");
                SetDlgItemText(hwnd, IDC_M42, "0");
                SetDlgItemText(hwnd, IDC_M43, "0");
                SetDlgItemText(hwnd, IDC_M44, "1");
            } else if (sel > 0) {
                char temp[100];
                long i = 0;
                for (MeshStructsIterator = MeshStructs.begin();
                        MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                    if ((*MeshStructsIterator)->effectpoint) {
                        i++;
                        if (i == sel) {
                            sprintf(temp, "%f", (*MeshStructsIterator)->effectpoint_vert.x);
                            SetDlgItemText(hwnd, IDC_M41, temp);
                            sprintf(temp, "%f", (*MeshStructsIterator)->effectpoint_vert.y);
//                              sprintf(temp, "%f", (CurrentFixOrient)
//                                      ? (*MeshStructsIterator)->effectpoint_vert.
//                                      z : (*MeshStructsIterator)->effectpoint_vert.y);
                            SetDlgItemText(hwnd, IDC_M42, temp);
                            sprintf(temp, "%f", (*MeshStructsIterator)->effectpoint_vert.z);
//                              sprintf(temp, "%f", (CurrentFixOrient)
//                                      ? (*MeshStructsIterator)->effectpoint_vert.
//                                      y : (*MeshStructsIterator)->effectpoint_vert.z);
                            SetDlgItemText(hwnd, IDC_M43, temp);
                        }
                    }
                }
            }
        }
        break;
        case IDC_EFFECT_POINTS_CREATE: {
            HMENU pu = LoadMenu(hInst, MAKEINTRESOURCE(IDR_EFFECTPOINT));
            POINT pos;
            GetCursorPos(&pos);
            TrackPopupMenuEx(GetSubMenu(pu, 0), TPM_LEFTALIGN, pos.x, pos.y, hwnd, NULL);
        }
        break;
        case IDM_EFFECT_LIGHT_START:
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_EFFECT_LIGHT), hwnd, EffectPointLightDlgProc)
                    == 1)
                SetFocus(GetDlgItem(hwnd, IDC_NAME));
            break;
        case IDM_EFFECT_PARTICLE_START:
            if (DialogBox
                    (hInst, MAKEINTRESOURCE(IDD_EFFECT_PARTICLE), hwnd,
                     EffectPointParticleDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_NAME, EM_SETSEL, 13, 15);
                SetFocus(GetDlgItem(hwnd, IDC_NAME));
            }
            break;
        case IDM_EFFECT_PARTICLE_END:
            SetDlgItemText(hwnd, IDC_NAME, "particleend01");
            SendDlgItemMessage(hwnd, IDC_NAME, EM_SETSEL, 11, 13);
            SetFocus(GetDlgItem(hwnd, IDC_NAME));
            break;
        case IDM_EFFECT_LAUNCHER_START:
            SetDlgItemText(hwnd, IDC_NAME, "fireworkstart01");
            SendDlgItemMessage(hwnd, IDC_NAME, EM_SETSEL, 13, 15);
            SetFocus(GetDlgItem(hwnd, IDC_NAME));
            break;
        case IDM_EFFECT_LAUNCHER_END:
            SetDlgItemText(hwnd, IDC_NAME, "fireworkend01");
            SendDlgItemMessage(hwnd, IDC_NAME, EM_SETSEL, 11, 13);
            SetFocus(GetDlgItem(hwnd, IDC_NAME));
            break;
        case IDM_EFFECT_PEEP_1:
            SetDlgItemText(hwnd, IDC_NAME, "peep01");
            SendDlgItemMessage(hwnd, IDC_NAME, EM_SETSEL, 4, 6);
            SetFocus(GetDlgItem(hwnd, IDC_NAME));
            break;
        case IDM_EFFECT_PEEP_VENDOR:
            SetDlgItemText(hwnd, IDC_NAME, "vendor");
            break;
        case IDM_EFFECT_FOUNTAIN_NOZBL:
            SetDlgItemText(hwnd, IDC_NAME, "nozbl");
            break;
        case IDM_EFFECT_FOUNTAIN_NOZBR:
            SetDlgItemText(hwnd, IDC_NAME, "nozbr");
            break;
        case IDM_EFFECT_FOUNTAIN_NOZTL:
            SetDlgItemText(hwnd, IDC_NAME, "noztl");
            break;
        case IDM_EFFECT_FOUNTAIN_NOZTR:
            SetDlgItemText(hwnd, IDC_NAME, "noztr");
            break;
        case IDC_EFFECT_POINTS_ROT_VERT: {
            long sel = SendDlgItemMessage(hwnd, IDC_EFFECT_POINTS_CB,
                                          CB_GETCURSEL, 0,
                                          0);
            char temp[100];
            double x, y, z, xt, yt, zt;
            if (sel <= 0) {
                MessageBox(hwnd, "No vertex selected to point at.", "Error",
                           MB_OK | MB_ICONSTOP);
                break;
            } else if (sel > 0) {
                long i = 0;
                for (MeshStructsIterator = MeshStructs.begin();
                        MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                    if ((*MeshStructsIterator)->effectpoint) {
                        i++;
                        if (i == sel) {
                            xt = (*MeshStructsIterator)->effectpoint_vert.x;
                            yt = (*MeshStructsIterator)->effectpoint_vert.y;
                            zt = (*MeshStructsIterator)->effectpoint_vert.z;
//                              yt = (CurrentFixOrient)
//                                  ? (*MeshStructsIterator)->effectpoint_vert.
//                                  z : (*MeshStructsIterator)->effectpoint_vert.y;
//                              zt = (CurrentFixOrient)
//                                  ? (*MeshStructsIterator)->effectpoint_vert.
//                                  y : (*MeshStructsIterator)->effectpoint_vert.z;
                        }
                    }
                }
            }
            GetDlgItemText(hwnd, IDC_M41, temp, 99);
            x = atof(temp);
            GetDlgItemText(hwnd, IDC_M42, temp, 99);
            y = atof(temp);
            GetDlgItemText(hwnd, IDC_M43, temp, 99);
            z = atof(temp);
            if (x == x) {
                double angle = atan((yt - y) / (zt - z));
                do_rot_x(hwnd, angle);
            } else if (y == y) {
                double angle = atan((zt - z) / (xt - x));
                do_rot_y(hwnd, angle);
            } else if (z == z) {
                double angle = atan((xt - x) / (yt - y));
                do_rot_z(hwnd, angle);
            } else {
                MessageBox(hwnd,
                           "Currently only rotation around one axis supported. That means one of the coordinates of the vertices must be the same value.",
                           "Error", MB_OK | MB_ICONSTOP);
            }
        }
        break;
        case IDC_EFFECT_POINTS_ROT_X: {
            char temp[100];
            GetDlgItemText(hwnd, IDC_EFFECT_POINTS_ROT_DEG, temp, 99);
            double angle = atof(temp);
            angle = angle * M_PI / 180.0;
            do_rot_x(hwnd, angle);
        }
        break;
        case IDC_EFFECT_POINTS_ROT_Y: {
            char temp[100];
            GetDlgItemText(hwnd, IDC_EFFECT_POINTS_ROT_DEG, temp, 99);
            double angle = atof(temp);
            angle = angle * M_PI / 180.0;
//              if (CurrentFixOrient)
//                  do_rot_z(hwnd, angle);
//              else
            do_rot_y(hwnd, angle);
        }
        break;
        case IDC_EFFECT_POINTS_ROT_Z: {
            char temp[100];
            GetDlgItemText(hwnd, IDC_EFFECT_POINTS_ROT_DEG, temp, 99);
            double angle = atof(temp);
            angle = angle * M_PI / 180.0;
//              if (CurrentFixOrient)
//                  do_rot_y(hwnd, angle);
//              else
            do_rot_z(hwnd, angle);
        }
        break;
        case IDC_EFFECT_POINTS_ROT_CLEAR: {
            SetDlgItemText(hwnd, IDC_M11, "1");
            SetDlgItemText(hwnd, IDC_M12, "0");
            SetDlgItemText(hwnd, IDC_M13, "0");
            SetDlgItemText(hwnd, IDC_M21, "0");
            SetDlgItemText(hwnd, IDC_M22, "1");
            SetDlgItemText(hwnd, IDC_M23, "0");
            SetDlgItemText(hwnd, IDC_M31, "0");
            SetDlgItemText(hwnd, IDC_M32, "0");
            SetDlgItemText(hwnd, IDC_M33, "1");
        }
        break;
        case IDC_EFFECT_POINTS_ROT_HELP:
            MessageBox(hwnd, "There might be some help here some time in the future =)",
                       "Rotation Help", MB_OK | MB_ICONINFORMATION);
            break;
        case IDOK:
            unsigned long m11len, m12len, m13len, m14len, m21len, m22len, m23len, m24len, m31len,
            m32len, m33len, m34len, m41len, m42len, m43len, m44len, namelen;
            char *temp;
            namelen = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            m11len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M11));
            m12len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M12));
            m13len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M13));
            m14len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M14));
            m21len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M21));
            m22len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M22));
            m23len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M23));
            m24len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M24));
            m31len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M31));
            m32len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M32));
            m33len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M33));
            m34len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M34));
            m41len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M41));
            m42len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M42));
            m43len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M43));
            m44len = GetWindowTextLength(GetDlgItem(hwnd, IDC_M44));
            if ((namelen != 0) && (m11len != 0) && (m12len != 0)
                    && (m13len != 0) && (m14len != 0) && (m21len != 0)
                    && (m22len != 0) && (m23len != 0) && (m24len != 0)
                    && (m31len != 0) && (m32len != 0) && (m33len != 0)
                    && (m34len != 0) && (m41len != 0) && (m42len != 0)
                    && (m43len != 0) && (m44len != 0)) {
                if (CurrentEffectPoint != -1) {
                    delete EffectPoints[CurrentEffectPoint]->Name;
                } else {
                    EffectPoint *t = new EffectPoint;
                    EffectPoints.push_back(t);
                    CurrentEffectPoint = EffectPoints.size() - 1;
                }
                EffectPoint *e = EffectPoints[CurrentEffectPoint];
                e->Name = new char[namelen + 1];
                GetDlgItemText(hwnd, IDC_NAME, e->Name, namelen + 1);
                temp = new char[m11len + 1];
                GetDlgItemText(hwnd, IDC_M11, temp, m11len + 1);
                e->Transform._11 = (float) atof(temp);
                delete temp;
                temp = new char[m12len + 1];
                GetDlgItemText(hwnd, IDC_M12, temp, m12len + 1);
                e->Transform._12 = (float) atof(temp);
                delete temp;
                temp = new char[m13len + 1];
                GetDlgItemText(hwnd, IDC_M13, temp, m13len + 1);
                e->Transform._13 = (float) atof(temp);
                delete temp;
                temp = new char[m14len + 1];
                GetDlgItemText(hwnd, IDC_M14, temp, m14len + 1);
                e->Transform._14 = (float) atof(temp);
                delete temp;
                temp = new char[m21len + 1];
                GetDlgItemText(hwnd, IDC_M21, temp, m21len + 1);
                e->Transform._21 = (float) atof(temp);
                delete temp;
                temp = new char[m22len + 1];
                GetDlgItemText(hwnd, IDC_M22, temp, m22len + 1);
                e->Transform._22 = (float) atof(temp);
                delete temp;
                temp = new char[m23len + 1];
                GetDlgItemText(hwnd, IDC_M23, temp, m23len + 1);
                e->Transform._23 = (float) atof(temp);
                delete temp;
                temp = new char[m24len + 1];
                GetDlgItemText(hwnd, IDC_M24, temp, m24len + 1);
                e->Transform._24 = (float) atof(temp);
                delete temp;
                temp = new char[m31len + 1];
                GetDlgItemText(hwnd, IDC_M31, temp, m31len + 1);
                e->Transform._31 = (float) atof(temp);
                delete temp;
                temp = new char[m32len + 1];
                GetDlgItemText(hwnd, IDC_M32, temp, m32len + 1);
                e->Transform._32 = (float) atof(temp);
                delete temp;
                temp = new char[m33len + 1];
                GetDlgItemText(hwnd, IDC_M33, temp, m33len + 1);
                e->Transform._33 = (float) atof(temp);
                delete temp;
                temp = new char[m34len + 1];
                GetDlgItemText(hwnd, IDC_M34, temp, m34len + 1);
                e->Transform._34 = (float) atof(temp);
                delete temp;
                temp = new char[m41len + 1];
                GetDlgItemText(hwnd, IDC_M41, temp, m41len + 1);
                e->Transform._41 = (float) atof(temp);
                delete temp;
                temp = new char[m42len + 1];
                GetDlgItemText(hwnd, IDC_M42, temp, m42len + 1);
                e->Transform._42 = (float) atof(temp);
                delete temp;
                temp = new char[m43len + 1];
                GetDlgItemText(hwnd, IDC_M43, temp, m43len + 1);
                e->Transform._43 = (float) atof(temp);
                delete temp;
                temp = new char[m44len + 1];
                GetDlgItemText(hwnd, IDC_M44, temp, m44len + 1);
                e->Transform._44 = (float) atof(temp);
                delete temp;
                EndDialog(hwnd, 1);
            }
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK TextureDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    OPENFILENAME ofn;
    char sfile[MAX_PATH] = "";
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_ADDSTRING, 0, (LPARAM) "No Colors");
        SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_ADDSTRING, 0, (LPARAM) "One Color Set");
        SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_ADDSTRING, 0, (LPARAM) "Two Color Sets");
        SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_ADDSTRING, 0, (LPARAM) "Three Color Sets");
        SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_SETCURSEL, 0, 0);
        if (CurrentFlexiTexture != -1) {
            SetDlgItemText(hwnd, IDC_NAME, FlexiTextures[CurrentFlexiTexture]->Name);
            SetDlgItemText(hwnd, IDC_TEXTURE, FlexiTextures[CurrentFlexiTexture]->Texture);
            SetDlgItemText(hwnd, IDC_ALPHA, FlexiTextures[CurrentFlexiTexture]->Alpha);
            long Recolor = FlexiTextures[CurrentFlexiTexture]->Recolorable;
            switch (Recolor) {
            case 3:
                Recolor = 2;
                break;
            case 7:
                Recolor = 3;
                break;
            }
            SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_SETCURSEL, Recolor, 0);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        case IDC_OPENALPHA:
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter =
                "Textures (*.bmp;*.png;*.jpg;*.tga)\0*.png;*.jpg;*.jpeg;*.bmp;*.tga\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Alpha Texture";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".bmp";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SetDlgItemText(hwnd, IDC_ALPHA, sfile);
            }
            break;
        case IDC_ALPHA_INFO: {
            char tmp[MAX_PATH];
            GetDlgItemText(hwnd, IDC_ALPHA, tmp, MAX_PATH - 1);
            if (strlen(tmp))
                showBitmapInfo(hwnd, tmp);
            else
                MessageBox(hwnd,
                           "Please select an alpha texture before requesting info on it ;-)",
                           "Error", MB_OK | MB_ICONSTOP);
        }
        break;
        case IDC_OPENTEXTURE:
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter =
                "Textures (*.bmp;*.png;*.jpg;*.tga)\0*.png;*.jpg;*.jpeg;*.bmp;*.tga\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Texture";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".bmp";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SetDlgItemText(hwnd, IDC_TEXTURE, sfile);
                unsigned long NameLen = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
                if (!NameLen) {
                    char temp[MAX_PATH];
                    _splitpath(sfile, NULL, NULL, temp, NULL);
                    SetDlgItemText(hwnd, IDC_NAME, temp);
                }
            }
            break;
        case IDC_TEXTURE_INFO: {
            char tmp[MAX_PATH];
            GetDlgItemText(hwnd, IDC_TEXTURE, tmp, MAX_PATH - 1);
            if (strlen(tmp))
                showBitmapInfo(hwnd, tmp);
            else
                MessageBox(hwnd, "Please select a texture before requesting info on it ;-)",
                           "Error", MB_OK | MB_ICONSTOP);
        }
        break;
        case IDOK:
            unsigned long TextureLen = GetWindowTextLength(GetDlgItem(hwnd, IDC_TEXTURE));
            unsigned long AlphaLen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ALPHA));
            unsigned long NameLen = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            if ((TextureLen != 0) && (NameLen != 0)) {
                TextureLen++;
                AlphaLen++;
                NameLen++;
                char *Name = new char[NameLen];
                char *Texture = new char[TextureLen];
                char *Alpha = new char[AlphaLen];
                GetDlgItemText(hwnd, IDC_TEXTURE, Texture, TextureLen);
                GetDlgItemText(hwnd, IDC_ALPHA, Alpha, AlphaLen);
                GetDlgItemText(hwnd, IDC_NAME, Name, NameLen);
                long Recolor = SendDlgItemMessage(hwnd, IDC_RECOLOR, CB_GETCURSEL, 0,
                                                  0);
                switch (Recolor) {
                case 2:
                    Recolor = 3;
                    break;
                case 3:
                    Recolor = 7;
                    break;
                }
                if (CurrentFlexiTexture != -1) {
                    delete FlexiTextures[CurrentFlexiTexture]->Alpha;
                    delete FlexiTextures[CurrentFlexiTexture]->Name;
                    delete FlexiTextures[CurrentFlexiTexture]->Texture;
                } else {
                    FlexiTexture *ft = new FlexiTexture;
                    FlexiTextures.push_back(ft);
                    CurrentFlexiTexture = FlexiTextures.size() - 1;
                }
                FlexiTextures[CurrentFlexiTexture]->Alpha = Alpha;
                FlexiTextures[CurrentFlexiTexture]->Name = Name;
                FlexiTextures[CurrentFlexiTexture]->Recolorable = Recolor;
                FlexiTextures[CurrentFlexiTexture]->Texture = Texture;
                EndDialog(hwnd, 1);
            } else {
                MessageBox(hwnd, "Nothing to save", "Error", MB_OK);
            }
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK MeshDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaque");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueChrome");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueChromeModulate");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueGlossReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueGlossSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueGlossSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIOpaqueSpecular100Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueSpecular50Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueText");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueUnlit");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIGlass");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlpha");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaChrome");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMask");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskChrome");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskLow");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskLowSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIAlphaMaskLowSpecular100Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskLowSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIAlphaMaskLowSpecular50Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIAlphaMaskSpecular100Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIAlphaMaskSpecular50Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskUnlit");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskUnlitLow");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaSpecular100Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaSpecular50Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaText");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaUnlit");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIFillZ");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskCaustics");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskLowCaustics");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIEmissiveMask");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIEmissiveMaskReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIEmissiveMaskSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIEmissiveMaskSpecular100Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIEmissiveMaskSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIEmissiveMaskSpecular50Reflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueCaustics");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0,
                           (LPARAM) "SIOpaqueCausticsGlossSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueCausticsReflection");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueCausticsSpecular100");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueCausticsSpecular50");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIWater");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIOpaqueDS");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaDS");
        SendDlgItemMessage(hwnd, IDC_TXS, CB_ADDSTRING, 0, (LPARAM) "SIAlphaMaskDS");
        SendDlgItemMessage(hwnd, IDC_PLACE, CB_ADDSTRING, 0,
                           (LPARAM) "Do Not Show Texture When Placing");
        SendDlgItemMessage(hwnd, IDC_PLACE, CB_ADDSTRING, 0, (LPARAM) "Show Texture When Placing");
        SendDlgItemMessage(hwnd, IDC_PLACE, CB_ADDSTRING, 0, (LPARAM) "Unknown");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "None");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Scrolling Sign");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Terrain Texture");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Cliffs Texture");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Water Texture");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Water Mask Texture");
        SendDlgItemMessage(hwnd, IDC_FLAGS, CB_ADDSTRING, 0, (LPARAM) "Advertising Sign Texture");
        SendDlgItemMessage(hwnd, IDC_UNKNOWN, CB_ADDSTRING, 0, (LPARAM) "Unknown 1");
        SendDlgItemMessage(hwnd, IDC_UNKNOWN, CB_ADDSTRING, 0, (LPARAM) "Unknown 2");
        for (FlexiTexturesIterator = FlexiTextures.begin();
                FlexiTexturesIterator != FlexiTextures.end(); FlexiTexturesIterator++) {
            SendDlgItemMessage(hwnd, IDC_FTX, CB_ADDSTRING, 0,
                               (LPARAM) (*FlexiTexturesIterator)->Name);
        }
        CheckDlgButton(hwnd, IDC_DISABLE, MeshStructs[CurrentMeshStruct]->disabled);
        if (MeshStructs[CurrentMeshStruct]->disabled) {
            EnableWindow(GetDlgItem(hwnd, IDC_PLACE), false);
            EnableWindow(GetDlgItem(hwnd, IDC_FLAGS), false);
            EnableWindow(GetDlgItem(hwnd, IDC_UNKNOWN), false);
            EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
            EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
        }
        SendDlgItemMessage(hwnd, IDC_PLACE, CB_SETCURSEL, MeshStructs[CurrentMeshStruct]->place, 0);
        if (MeshStructs[CurrentMeshStruct]->flags == 0) {
            SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 0, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->flags == 12) {
            SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 1, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->flags == 12288) {
            EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
            EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
            SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 2, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->flags == 20480) {
            EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
            EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
            SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 3, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->flags == 32788) {
            EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
            EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
            SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 6, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->FTX != 0) {
            if (stricmp(MeshStructs[CurrentMeshStruct]->FTX, "siwater") == 0) {
                EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 4, 0);
            }
            if (stricmp(MeshStructs[CurrentMeshStruct]->FTX, "watermask")
                    == 0) {
                EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                SendDlgItemMessage(hwnd, IDC_FLAGS, CB_SETCURSEL, 5, 0);
            }
        }
        if (MeshStructs[CurrentMeshStruct]->unknown == 1) {
            SendDlgItemMessage(hwnd, IDC_UNKNOWN, CB_SETCURSEL, 0, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->unknown == 3) {
            SendDlgItemMessage(hwnd, IDC_UNKNOWN, CB_SETCURSEL, 1, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->TXS != 0) {
            SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                               (LPARAM) MeshStructs[CurrentMeshStruct]->TXS);
            SetDlgItemText(hwnd, IDC_TXS, MeshStructs[CurrentMeshStruct]->TXS);
        } else {
            SendDlgItemMessage(hwnd, IDC_TXS, CB_SETCURSEL, (WPARAM) - 1, 0);
        }
        if (MeshStructs[CurrentMeshStruct]->FTX != 0) {
            SendDlgItemMessage(hwnd, IDC_FTX, CB_SELECTSTRING, (WPARAM) - 1,
                               (LPARAM) MeshStructs[CurrentMeshStruct]->FTX);
            SetDlgItemText(hwnd, IDC_FTX, MeshStructs[CurrentMeshStruct]->FTX);
        } else {
            SendDlgItemMessage(hwnd, IDC_FTX, CB_SETCURSEL, (WPARAM) - 1, 0);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        case IDC_DISABLE:
            if (IsDlgButtonChecked(hwnd, IDC_DISABLE)) {
                EnableWindow(GetDlgItem(hwnd, IDC_PLACE), false);
                EnableWindow(GetDlgItem(hwnd, IDC_FLAGS), false);
                EnableWindow(GetDlgItem(hwnd, IDC_UNKNOWN), false);
                EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
            } else {
                EnableWindow(GetDlgItem(hwnd, IDC_PLACE), true);
                EnableWindow(GetDlgItem(hwnd, IDC_FLAGS), true);
                EnableWindow(GetDlgItem(hwnd, IDC_UNKNOWN), true);
                EnableWindow(GetDlgItem(hwnd, IDC_FTX), true);
                EnableWindow(GetDlgItem(hwnd, IDC_TXS), true);
            }
            break;
        case IDC_FLAGS:
            if (HIWORD(wParam) == CBN_SELCHANGE) {
                unsigned long flags = SendDlgItemMessage(hwnd, IDC_FLAGS, CB_GETCURSEL, 0,
                                      0);
                switch (flags) {
                case 0:
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SETCURSEL, (WPARAM) - 1, 0);
                    SendDlgItemMessage(hwnd, IDC_FTX, CB_SETCURSEL, (WPARAM) - 1, 0);
                    SetDlgItemText(hwnd, IDC_FTX, NULL);
                    SetDlgItemText(hwnd, IDC_TXS, NULL);
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), true);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), true);
                    break;
                case 1:
                    SendDlgItemMessage(hwnd, IDC_FTX, CB_SETCURSEL, (WPARAM) - 1, 0);
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIOpaqueUnlit");
                    SetDlgItemText(hwnd, IDC_FTX, NULL);
                    SetDlgItemText(hwnd, IDC_TXS, "SIOpaqueUnlit");
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), true);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), true);
                    break;
                case 2:
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                    SetDlgItemText(hwnd, IDC_FTX, "useterraintexture");
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIOpaque");
                    SetDlgItemText(hwnd, IDC_TXS, "SIOpaque");
                    break;
                case 3:
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                    SetDlgItemText(hwnd, IDC_FTX, "useclifftexture");
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIOpaque");
                    SetDlgItemText(hwnd, IDC_TXS, "SIOpaque");
                    break;
                case 4:
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                    SetDlgItemText(hwnd, IDC_FTX, "siwater");
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIWater");
                    SetDlgItemText(hwnd, IDC_TXS, "SIWater");
                    break;
                case 5:
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                    SetDlgItemText(hwnd, IDC_FTX, "watermask");
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIFillZ");
                    SetDlgItemText(hwnd, IDC_TXS, "SIFillZ");
                    break;
                case 6:
                    EnableWindow(GetDlgItem(hwnd, IDC_FTX), false);
                    EnableWindow(GetDlgItem(hwnd, IDC_TXS), false);
                    SetDlgItemText(hwnd, IDC_FTX, "UseAdTexture");
                    SendDlgItemMessage(hwnd, IDC_TXS, CB_SELECTSTRING, (WPARAM) - 1,
                                       (LPARAM) "SIOpaque");
                    SetDlgItemText(hwnd, IDC_TXS, "SIOpaque");
                    break;
                }
            }
            break;
        case IDOK:
            unsigned long place = SendDlgItemMessage(hwnd, IDC_PLACE, CB_GETCURSEL, 0, 0);
            unsigned long flags = SendDlgItemMessage(hwnd, IDC_FLAGS, CB_GETCURSEL, 0, 0);
            unsigned long unknown = SendDlgItemMessage(hwnd, IDC_UNKNOWN, CB_GETCURSEL, 0, 0);
            unsigned long ftxlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_FTX));
            unsigned long txslen = GetWindowTextLength(GetDlgItem(hwnd, IDC_TXS));
            unsigned long doall = SendDlgItemMessage(hwnd, IDC_DOALL, BM_GETCHECK, 0, 0);
            if ((flags != -1) || (place != -1) && (unknown != -1)
                    && (ftxlen != 0) && (txslen != 0)) {
                ftxlen++;
                txslen++;
                char *ftx = new char[ftxlen];
                char *txs = new char[txslen];
                GetDlgItemText(hwnd, IDC_FTX, ftx, ftxlen);
                GetDlgItemText(hwnd, IDC_TXS, txs, txslen);
                if (doall != BST_CHECKED) {
                    MeshStructs[CurrentMeshStruct]->disabled =
                        IsDlgButtonChecked(hwnd, IDC_DISABLE);
                    if ((flags == 0) || (flags == 4) || (flags == 5)) {
                        MeshStructs[CurrentMeshStruct]->flags = 0;
                    }
                    if (flags == 1) {
                        MeshStructs[CurrentMeshStruct]->flags = 12;
                    }
                    if (flags == 2) {
                        MeshStructs[CurrentMeshStruct]->flags = 12288;
                    }
                    if (flags == 3) {
                        MeshStructs[CurrentMeshStruct]->flags = 20480;
                    }
                    if (flags == 6) {
                        MeshStructs[CurrentMeshStruct]->flags = 32788;
                    }
                    MeshStructs[CurrentMeshStruct]->place = place;
                    MeshStructs[CurrentMeshStruct]->FTX = ftx;
                    MeshStructs[CurrentMeshStruct]->TXS = txs;
                    if (unknown == 0) {
                        MeshStructs[CurrentMeshStruct]->unknown = 1;
                    }
                    if (unknown == 1) {
                        MeshStructs[CurrentMeshStruct]->unknown = 3;
                    }
                } else {
                    for (UINT ttt = 0; ttt < (UINT) MeshStructs.size(); ++ttt) {
                        MeshStructs[ttt]->disabled = IsDlgButtonChecked(hwnd, IDC_DISABLE);
                        if ((flags == 0) || (flags == 4) || (flags == 5)) {
                            MeshStructs[ttt]->flags = 0;
                        }
                        if (flags == 1) {
                            MeshStructs[ttt]->flags = 12;
                        }
                        if (flags == 2) {
                            MeshStructs[ttt]->flags = 12288;
                        }
                        if (flags == 3) {
                            MeshStructs[ttt]->flags = 20480;
                        }
                        if (flags == 6) {
                            MeshStructs[ttt]->flags = 32788;
                        }
                        MeshStructs[ttt]->place = place;
                        MeshStructs[ttt]->FTX = ftx;
                        MeshStructs[ttt]->TXS = txs;
                        if (unknown == 0) {
                            MeshStructs[ttt]->unknown = 1;
                        }
                        if (unknown == 1) {
                            MeshStructs[ttt]->unknown = 3;
                        }

                    }
                }
//                                              delete[] ftx;
//                                              delete[] txs;

                EndDialog(hwnd, 1);
            }
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

char *asename;
void CleanUpMeshStructs() {
    for (unsigned long i = 0; i < MeshStructs.size(); i++) {
        if (MeshStructs[i]->FTX != 0) {
            delete MeshStructs[i]->FTX;
        }
        if (MeshStructs[i]->TXS != 0) {
            delete MeshStructs[i]->TXS;
        }
        if (MeshStructs[i]->Name != 0) {
            delete MeshStructs[i]->Name;
        }
        delete MeshStructs[i];
    }
    MeshStructs.clear();
    CurrentMeshStruct = -1;
}

void CleanUpScenery() {
    unsigned long i;
    for (i = 0; i < FlexiTextures.size(); i++) {
        delete FlexiTextures[i]->Alpha;
        delete FlexiTextures[i]->Name;
        delete FlexiTextures[i]->Texture;
        delete FlexiTextures[i];
    }
    FlexiTextures.clear();
    CurrentFlexiTexture = -1;
    for (i = 0; i < References.size(); i++) {
        delete References[i];
    }
    References.clear();
    CurrentReference = -1;
    for (i = 0; i < EffectPoints.size(); i++) {
        if (EffectPoints[i]->Name != 0)
            delete EffectPoints[i]->Name;
        delete EffectPoints[i];
    }
    EffectPoints.clear();
    CurrentEffectPoint = -1;
    CleanUpMeshStructs();
    if (asename != 0) {
        delete asename;
    }
    asename = new char[1];
    asename[0] = 0;
    CurrentFixOrient = DEFAULT_SCN_FIXORIENT;
}

BOOL CALLBACK SceneryDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    int reflen;
    OPENFILENAME ofn;
    char sfile[MAX_PATH] = "";
    SceneryDlgHwnd = hwnd;
    unsigned long aselen;
    unsigned long i;
    switch (Message) {
    case WM_INITDIALOG:
        SetMenu(hwnd, LoadMenu(hInst, MAKEINTRESOURCE(IDR_SCENERY)));
        SendDlgItemMessage(hwnd, IDC_MESHES, LB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
        SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
        CheckDlgButton(hwnd, IDC_FIXORIENT, DEFAULT_SCN_FIXORIENT);
        CleanUpScenery();
        SetDlgItemText(hwnd, IDC_ASE, NULL);
        SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_ADDSTRING, 0, (LPARAM) "None");
        SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_ADDSTRING, 0, (LPARAM) "Soaked!");
        SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_ADDSTRING, 0, (LPARAM) "Wild!");
        SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
        SetDlgItemText(hwnd, IDC_SWAY, "0.0");
        SetDlgItemText(hwnd, IDC_BRIGHTNESS, "1.0");
        SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
        SetDlgItemText(hwnd, IDC_SCALE, "0.0");
        scenerysave = false;
        break;
    case WM_CLOSE:
        if (scenerysave == true) {
            if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO) == IDNO) {
                break;
            }
        }
        EndDialog(hwnd, 0);
        break;
    case WM_MEASUREITEM: {
        LPMEASUREITEMSTRUCT lpmis = (LPMEASUREITEMSTRUCT) lParam;
        lpmis->itemHeight = 16;
    }
    return TRUE;
    case WM_DRAWITEM: {
        LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT) lParam;

        if (lpdis->itemID == -1)
            break;

        switch (lpdis->CtlID) {
        case IDC_MESHES:
            if (lpdis->itemID >= MeshStructs.size())
                break;
            switch (lpdis->itemAction) {
            case ODA_FOCUS:
            case ODA_SELECT:
            case ODA_DRAWENTIRE:

                // Display the text associated with the item.

                //SendMessage(lpdis->hwndItem, LB_GETTEXT, lpdis->itemID, (LPARAM) tchBuffer);
                TEXTMETRIC tm;
                if (lpdis->itemState & ODS_SELECTED) {
                    FillRect(lpdis->hDC, &lpdis->rcItem, (HBRUSH) GetStockObject(BLACK_BRUSH));
                    SetBkColor(lpdis->hDC, RGB(0, 0, 0));
                } else {
                    FillRect(lpdis->hDC, &lpdis->rcItem, (HBRUSH) GetStockObject(WHITE_BRUSH));
                    SetBkColor(lpdis->hDC, RGB(255, 255, 255));
                }

                if (MeshStructs[lpdis->itemID]->effectpoint)
                    SetTextColor(lpdis->hDC, RGB(160, 210, 160));
                else if (MeshStructs[lpdis->itemID]->disabled)
                    SetTextColor(lpdis->hDC, RGB(128, 128, 128));
                else if ((MeshStructs[lpdis->itemID]->FTX == 0)
                         || (MeshStructs[lpdis->itemID]->TXS == 0)
                         || (strlen(MeshStructs[lpdis->itemID]->FTX) == 0)
                         || (strlen(MeshStructs[lpdis->itemID]->TXS) == 0))
                    SetTextColor(lpdis->hDC, RGB(255, 60, 60));
                else if (lpdis->itemState & ODS_SELECTED)
                    SetTextColor(lpdis->hDC, RGB(255, 255, 255));
                else
                    SetTextColor(lpdis->hDC, RGB(0, 0, 0));


                GetTextMetrics(lpdis->hDC, &tm);

                int y = (lpdis->rcItem.bottom + lpdis->rcItem.top - tm.tmHeight) / 2;

                TextOut(lpdis->hDC, 6, y, MeshStructs[lpdis->itemID]->Name,
                        strlen(MeshStructs[lpdis->itemID]->Name));

                if (lpdis->itemState & ODS_FOCUS) {
                    DrawFocusRect(lpdis->hDC, &lpdis->rcItem);
                }
                break;
            }
        }
    }
    break;

    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_EXPANSIONFLAG:
        case IDC_SWAY:
        case IDC_BRIGHTNESS:
        case IDC_UNKNOWNFACTOR:
        case IDC_SCALE:
        case IDC_GREENERY:
        case IDC_FLOWERS:
        case IDC_VARYROTATION:
        case IDC_UNKNOWNBITS:
            scenerysave = true;
            break;
        case IDC_NORMALSCENERY: {
            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
            SetDlgItemText(hwnd, IDC_SWAY, "0.0");
            SetDlgItemText(hwnd, IDC_BRIGHTNESS, "1.0");
            SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
            SetDlgItemText(hwnd, IDC_SCALE, "0.0");
            CheckDlgButton(hwnd, IDC_GREENERY, false);
            CheckDlgButton(hwnd, IDC_FLOWERS, false);
            CheckDlgButton(hwnd, IDC_VARYROTATION, false);
            CheckDlgButton(hwnd, IDC_UNKNOWNBITS, false);
            scenerysave = true;
        }
        break;
        case IDC_SWAYINGTREE: {
            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
            SetDlgItemText(hwnd, IDC_SWAY, "0.2");
            SetDlgItemText(hwnd, IDC_BRIGHTNESS, "0.8");
            SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
            SetDlgItemText(hwnd, IDC_SCALE, "0.4");
            CheckDlgButton(hwnd, IDC_GREENERY, true);
            CheckDlgButton(hwnd, IDC_FLOWERS, false);
            CheckDlgButton(hwnd, IDC_VARYROTATION, true);
            CheckDlgButton(hwnd, IDC_UNKNOWNBITS, false);
            scenerysave = true;
        }
        break;
        case IDC_NOTES: {
            MessageBox(GetActiveWindow(),
                       "- Expansion Flag: Set on objects of the respective expansion. No known in-game effect.\n"
                       "- Greenery: Set on trees, shrubs and prehistoric fern. No known in-game effect.\n"
                       "- Flowers: Set on flowers. Effect untested.\n"
                       "- Vary Rotation: Random object rotation on placement. Set on trees & shrubs.\n"
                       "- Unknown Bits: Set on several objects. Effect untested.\n"
                       "- Swaying Factor: Amount of swaying. 0.2 on almost all trees & shrubs. 0.0 for safari trees.\n"
                       "- Brightness Variation: If < 1.0, object brightness varies on placement.\n"
                       "- Unknown Factor: 1.0 in all checked original objects. Also seems to affect brightness.\n"
                       "- Scale Variation: If > 0.0, object size varies on placement.\n"
                       "These settings only affect the model, not the effect points. Shadows are also unaffected.\n\n"
                       "All in-game trees & shrubs (with the noted Safari exception) have the same values for Swaying,\n"
                       "Brightness and Scale. They also have Greenery and Rotation set.\n"
                       "Some in-game trees are normal scenery in regard to these settings (e.g. Sci-Fi).",
                       "Visual Options Notes", MB_OK | MB_ICONINFORMATION);
        }
        break;
        case IDC_ASE:
            if (HIWORD(wParam) == EN_KILLFOCUS) {
                aselen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ASE));
                if (aselen != 0) {
                    char *ase = new char[aselen + 1];
                    GetDlgItemText(hwnd, IDC_ASE, ase, aselen + 1);
                    if (stricmp(ase, asename) != 0) {
                        delete asename;
                        asename = ase;
                        FILE *f = fopen(ase, "rt");
                        if (f == 0) {
                            MessageBox(hwnd, "Can't Open ASE File", "Error", MB_OK);
                            SetDlgItemText(hwnd, IDC_ASE, NULL);
                            break;
                        }
                        fclose(f);
                        SendDlgItemMessage(hwnd, IDC_MESHES, LB_RESETCONTENT, 0, 0);
                        CleanUpMeshStructs();
                        ASE_Scene *scene = ASE_loadFilename(ase);
                        for (long j = 0; j < scene->objectCount; j++) {
                            SendDlgItemMessage(hwnd, IDC_MESHES, LB_ADDSTRING, 0,
                                               (LPARAM) scene->objs[j].name);
                            MeshStruct *ms = new MeshStruct;
                            if ((strcmp(scene->objs[j].name, "root") == 0)
                                    || (scene->objs[j].mesh.vertexCount < 3)) {
                                ms->disabled = true;
                            } else {
                                ms->disabled = false;
                            }
                            if (scene->objs[j].mesh.vertexCount == 1) {
                                ms->effectpoint = true;
                                ms->effectpoint_vert = scene->objs[j].mesh.vertices[0];
                            } else {
                                ms->effectpoint = false;
                            }
                            ms->Name = new char[strlen(scene->objs[j].name) + 1];
                            strcpy(ms->Name, scene->objs[j].name);
                            ms->flags = 0;
                            ms->place = 0;
                            ms->unknown = 3;
                            ms->FTX = 0;
                            ms->TXS = 0;
                            MeshStructs.push_back(ms);
                        }
                    }
                }
            }
            break;
        case IDCANCEL:
            if (scenerysave == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            EndDialog(hwnd, 0);
            break;
        case ID_FILE_SAVESCENERY:
            if (asename[0] == 0) {
                MessageBox(hwnd, "Can't Save Scenery File due to missing data", "Error", MB_OK);
                break;
            }
            {
                bool fail = false;
                for (MeshStructsIterator = MeshStructs.begin();
                        MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                    if ((*MeshStructsIterator)->disabled == false) {
                        if (((*MeshStructsIterator)->FTX == 0)
                                || ((*MeshStructsIterator)->TXS == 0)) {
                            char err[256];
                            snprintf(err, 255,
                                     "Can't Save Scenery File due to missing data in mesh \"%s\"",
                                     (*MeshStructsIterator)->Name);
                            MessageBox(hwnd, err, "Error", MB_OK);
                            fail = true;
                        }
                    }
                }
                if (fail == true) {
                    break;
                }
            }
            if (!strcmp(sfile, "")) {
                char fname[_MAX_FNAME];
                _splitpath(asename, NULL, NULL, fname, NULL);
                strncpy(sfile, fname, MAX_PATH);
            }
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Scenery Files (*.scn)\0*.scn\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Save Scenery file";
            ofn.lpstrDefExt = ".scn";
            ofn.nMaxFile = MAX_PATH;
            ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
            if (GetSaveFileName(&ofn)) {
                FILE *f = fopen(sfile, "wb");
                unsigned long aselen = strlen(asename) + 1;
                fwrite(&aselen, sizeof(aselen), 1, f);
                fwrite(asename, aselen, 1, f);
                unsigned long size = MeshStructs.size();
                fwrite(&size, sizeof(size), 1, f);
                for (i = 0; i < size; i++) {
                    fwrite(&MeshStructs[i]->disabled, sizeof(MeshStructs[i]->disabled), 1, f);
                    if (MeshStructs[i]->disabled == false) {
                        fwrite(&MeshStructs[i]->flags, sizeof(MeshStructs[i]->flags), 1, f);
                        fwrite(&MeshStructs[i]->place, sizeof(MeshStructs[i]->place), 1, f);
                        fwrite(&MeshStructs[i]->unknown, sizeof(MeshStructs[i]->unknown), 1, f);
                        unsigned long ftxlen = strlen(MeshStructs[i]->FTX) + 1;
                        unsigned long txslen = strlen(MeshStructs[i]->TXS) + 1;
                        fwrite(&ftxlen, sizeof(ftxlen), 1, f);
                        fwrite(MeshStructs[i]->FTX, ftxlen, 1, f);
                        fwrite(&txslen, sizeof(txslen), 1, f);
                        fwrite(MeshStructs[i]->TXS, txslen, 1, f);
                    }
                }
                size = FlexiTextures.size();
                fwrite(&size, sizeof(size), 1, f);
                for (i = 0; i < size; i++) {
                    fwrite(&FlexiTextures[i]->Recolorable,
                           sizeof(FlexiTextures[i]->Recolorable), 1, f);
                    unsigned long namelen = strlen(FlexiTextures[i]->Name) + 1;
                    fwrite(&namelen, sizeof(namelen), 1, f);
                    fwrite(FlexiTextures[i]->Name, namelen, 1, f);
                    unsigned long texturelen = strlen(FlexiTextures[i]->Texture) + 1;
                    fwrite(&texturelen, sizeof(texturelen), 1, f);
                    fwrite(FlexiTextures[i]->Texture, texturelen, 1, f);
                    unsigned long alphalen = strlen(FlexiTextures[i]->Alpha) + 1;
                    fwrite(&alphalen, sizeof(alphalen), 1, f);
                    fwrite(FlexiTextures[i]->Alpha, alphalen, 1, f);
                }
                size = References.size();
                fwrite(&size, sizeof(size), 1, f);
                for (i = 0; i < size; i++) {
                    unsigned long reflen = strlen(References[i]) + 1;
                    fwrite(&reflen, sizeof(reflen), 1, f);
                    fwrite(References[i], reflen, 1, f);
                }
                size = EffectPoints.size();
                fwrite(&size, sizeof(size), 1, f);
                for (i = 0; i < size; i++) {
                    unsigned long namelen = strlen(EffectPoints[i]->Name) + 1;
                    fwrite(&namelen, sizeof(namelen), 1, f);
                    fwrite(EffectPoints[i]->Name, namelen, 1, f);
                    fwrite(&EffectPoints[i]->Transform._11,
                           sizeof(EffectPoints[i]->Transform._11), 1, f);
                    fwrite(&EffectPoints[i]->Transform._12,
                           sizeof(EffectPoints[i]->Transform._12), 1, f);
                    fwrite(&EffectPoints[i]->Transform._13,
                           sizeof(EffectPoints[i]->Transform._13), 1, f);
                    fwrite(&EffectPoints[i]->Transform._14,
                           sizeof(EffectPoints[i]->Transform._14), 1, f);
                    fwrite(&EffectPoints[i]->Transform._21,
                           sizeof(EffectPoints[i]->Transform._21), 1, f);
                    fwrite(&EffectPoints[i]->Transform._22,
                           sizeof(EffectPoints[i]->Transform._22), 1, f);
                    fwrite(&EffectPoints[i]->Transform._23,
                           sizeof(EffectPoints[i]->Transform._23), 1, f);
                    fwrite(&EffectPoints[i]->Transform._24,
                           sizeof(EffectPoints[i]->Transform._24), 1, f);
                    fwrite(&EffectPoints[i]->Transform._31,
                           sizeof(EffectPoints[i]->Transform._31), 1, f);
                    fwrite(&EffectPoints[i]->Transform._32,
                           sizeof(EffectPoints[i]->Transform._32), 1, f);
                    fwrite(&EffectPoints[i]->Transform._33,
                           sizeof(EffectPoints[i]->Transform._33), 1, f);
                    fwrite(&EffectPoints[i]->Transform._34,
                           sizeof(EffectPoints[i]->Transform._34), 1, f);
                    fwrite(&EffectPoints[i]->Transform._41,
                           sizeof(EffectPoints[i]->Transform._41), 1, f);
                    fwrite(&EffectPoints[i]->Transform._42,
                           sizeof(EffectPoints[i]->Transform._42), 1, f);
                    fwrite(&EffectPoints[i]->Transform._43,
                           sizeof(EffectPoints[i]->Transform._43), 1, f);
                    fwrite(&EffectPoints[i]->Transform._44,
                           sizeof(EffectPoints[i]->Transform._44), 1, f);
                }
                // Write a version number first
                fwrite(&scn_version, sizeof(long), 1, f);
                // SCN Version 1 Stuff
                unsigned long fixorient = SendDlgItemMessage(hwnd, IDC_FIXORIENT, BM_GETCHECK, 0,
                                          0);
                fwrite(&fixorient, sizeof(long), 1, f);

                // SCN Version 2 Stuff
                unsigned int sivflags = 0;
                if (SendDlgItemMessage(hwnd, IDC_GREENERY, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_GREENERY;
                if (SendDlgItemMessage(hwnd, IDC_FLOWERS, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_FLOWER;
                if (SendDlgItemMessage(hwnd, IDC_VARYROTATION, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_ROTATION;
                if (SendDlgItemMessage(hwnd, IDC_UNKNOWNBITS, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_UNKNOWN;
                unsigned long exp = SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_GETCURSEL, 0, 0);
                if (exp == 1)
                    sivflags |= OVL_SIV_SOAKED;
                else if (exp == 2)
                    sivflags |= OVL_SIV_WILD;
                fwrite(&sivflags, sizeof(unsigned int), 1, f);

                int len = GetWindowTextLength(GetDlgItem(hwnd, IDC_SWAY));
                char *temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_SWAY, temp, len + 1);
                float fl = (float) atof(temp);
                delete temp;
                fwrite(&fl, sizeof(float), 1, f);

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_BRIGHTNESS));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_BRIGHTNESS, temp, len + 1);
                fl = (float) atof(temp);
                delete temp;
                fwrite(&fl, sizeof(float), 1, f);

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_UNKNOWNFACTOR));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, temp, len + 1);
                fl = (float) atof(temp);
                delete temp;
                fwrite(&fl, sizeof(float), 1, f);

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_SCALE));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_SCALE, temp, len + 1);
                fl = (float) atof(temp);
                delete temp;
                fwrite(&fl, sizeof(float), 1, f);

                scenerysave = false;
                fclose(f);
            }
            break;
        case ID_FILE_OPENSCENERY:
            if (scenerysave == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }

            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Scenery Files (*.scn)\0*.scn\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Scenery File";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".scn";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SendDlgItemMessage(hwnd, IDC_MESHES, LB_RESETCONTENT, 0, 0);
                SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_RESETCONTENT, 0, 0);
                SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);

                CleanUpScenery();

                SetDlgItemText(hwnd, IDC_ASE, NULL);
                FILE *f = fopen(sfile, "rb");
                unsigned long aselen;
                fread(&aselen, sizeof(aselen), 1, f);
                char *ase = new char[MAX_PATH];
                fread(ase, aselen, 1, f);
                long meshcount;
                fread(&meshcount, sizeof(meshcount), 1, f);
                FILE *f2 = fopen(ase, "rt");
                if (f2 == 0) {
                    MessageBox(hwnd, "Can't Open ASE File", "Error", MB_OK);
                    ZeroMemory(&ofn, sizeof(ofn));
                    ofn.lStructSize = sizeof(ofn);
                    ofn.hwndOwner = hwnd;
                    ofn.lpstrFilter = "3D Studio ASCII (*.ase)\0*.ase\0";
                    ofn.lpstrFile = ase;
                    ofn.lpstrTitle = "Select ASE file";
                    ofn.nMaxFile = MAX_PATH;
                    ofn.lpstrDefExt = ".ase";
                    ofn.Flags =
                        OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                        OFN_NOCHANGEDIR;
                    if (!GetOpenFileName(&ofn))
                        break;
                } else
                    fclose(f2);
                ASE_Scene *scene = ASE_loadFilename(ase);
                if (scene->objectCount != meshcount) {
                    MessageBox(hwnd,
                               "ASE File mesh count different from SCN file mesh count, can't load SCN file",
                               "Error", MB_OK);
                    break;
                }
                SetDlgItemText(hwnd, IDC_ASE, ase);
                asename = ase;
                for (long j = 0; j < scene->objectCount; j++) {
                    SendDlgItemMessage(hwnd, IDC_MESHES, LB_ADDSTRING, 0,
                                       (LPARAM) scene->objs[j].name);
                    MeshStruct *ms = new MeshStruct;
                    ms->disabled = false;
                    ms->flags = 0;
                    ms->place = 0;
                    ms->unknown = 3;
                    ms->FTX = 0;
                    ms->TXS = 0;
                    ms->Name = new char[strlen(scene->objs[j].name) + 1];
                    strcpy(ms->Name, scene->objs[j].name);
                    if (scene->objs[j].mesh.vertexCount == 1) {
                        ms->effectpoint = true;
                        ms->effectpoint_vert = scene->objs[j].mesh.vertices[0];
                    } else {
                        ms->effectpoint = false;
                    }
                    MeshStructs.push_back(ms);
                }
                for (i = 0; i < meshcount; i++) {
                    fread(&MeshStructs[i]->disabled, sizeof(MeshStructs[i]->disabled), 1, f);
                    if (MeshStructs[i]->disabled == false) {
                        fread(&MeshStructs[i]->flags, sizeof(MeshStructs[i]->flags), 1, f);
                        fread(&MeshStructs[i]->place, sizeof(MeshStructs[i]->place), 1, f);
                        fread(&MeshStructs[i]->unknown, sizeof(MeshStructs[i]->unknown), 1, f);
                        unsigned long ftxlen;
                        unsigned long txslen;
                        fread(&ftxlen, sizeof(ftxlen), 1, f);
                        MeshStructs[i]->FTX = new char[ftxlen];
                        fread(MeshStructs[i]->FTX, ftxlen, 1, f);
                        fread(&txslen, sizeof(txslen), 1, f);
                        MeshStructs[i]->TXS = new char[txslen];
                        fread(MeshStructs[i]->TXS, txslen, 1, f);
                    }
                }
                unsigned long texcount;
                fread(&texcount, sizeof(texcount), 1, f);
                for (i = 0; i < texcount; i++) {
                    FlexiTexture *ft = new FlexiTexture;
                    fread(&ft->Recolorable, sizeof(ft->Recolorable), 1, f);
                    unsigned long namelen;
                    fread(&namelen, sizeof(namelen), 1, f);
                    ft->Name = new char[namelen];
                    fread(ft->Name, namelen, 1, f);
                    unsigned long texturelen;
                    fread(&texturelen, sizeof(texturelen), 1, f);
                    ft->Texture = new char[texturelen];
                    fread(ft->Texture, texturelen, 1, f);
                    unsigned long alphalen;
                    fread(&alphalen, sizeof(alphalen), 1, f);
                    ft->Alpha = new char[alphalen];
                    fread(ft->Alpha, alphalen, 1, f);
                    FlexiTextures.push_back(ft);
                }
                SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
                for (FlexiTexturesIterator = FlexiTextures.begin();
                        FlexiTexturesIterator != FlexiTextures.end(); FlexiTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_ADDSTRING, 0,
                                       (LPARAM) (*FlexiTexturesIterator)->Name);
                }
                unsigned long refcount;
                fread(&refcount, sizeof(refcount), 1, f);
                for (i = 0; i < refcount; i++) {
                    unsigned long reflen;
                    fread(&reflen, sizeof(reflen), 1, f);
                    char *ref = new char[reflen];
                    fread(ref, reflen, 1, f);
                    References.push_back(ref);
                }
                SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_RESETCONTENT, 0, 0);
                for (ReferencesIterator = References.begin();
                        ReferencesIterator != References.end(); ReferencesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_ADDSTRING, 0,
                                       (LPARAM) (*ReferencesIterator));
                }
                unsigned long ptcount;
                fread(&ptcount, sizeof(ptcount), 1, f);
                if (!feof(f)) {
                    for (i = 0; i < ptcount; i++) {
                        EffectPoint *e = new EffectPoint;
                        unsigned long namelen;
                        fread(&namelen, sizeof(namelen), 1, f);
                        e->Name = new char[namelen];
                        fread(e->Name, namelen, 1, f);
                        fread(&e->Transform._11, sizeof(e->Transform._11), 1, f);
                        fread(&e->Transform._12, sizeof(e->Transform._12), 1, f);
                        fread(&e->Transform._13, sizeof(e->Transform._13), 1, f);
                        fread(&e->Transform._14, sizeof(e->Transform._14), 1, f);
                        fread(&e->Transform._21, sizeof(e->Transform._21), 1, f);
                        fread(&e->Transform._22, sizeof(e->Transform._22), 1, f);
                        fread(&e->Transform._23, sizeof(e->Transform._23), 1, f);
                        fread(&e->Transform._24, sizeof(e->Transform._24), 1, f);
                        fread(&e->Transform._31, sizeof(e->Transform._31), 1, f);
                        fread(&e->Transform._32, sizeof(e->Transform._32), 1, f);
                        fread(&e->Transform._33, sizeof(e->Transform._33), 1, f);
                        fread(&e->Transform._34, sizeof(e->Transform._34), 1, f);
                        fread(&e->Transform._41, sizeof(e->Transform._41), 1, f);
                        fread(&e->Transform._42, sizeof(e->Transform._42), 1, f);
                        fread(&e->Transform._43, sizeof(e->Transform._43), 1, f);
                        fread(&e->Transform._44, sizeof(e->Transform._44), 1, f);
                        EffectPoints.push_back(e);
                    }
                }
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
                for (EffectPointsIterator = EffectPoints.begin();
                        EffectPointsIterator != EffectPoints.end(); EffectPointsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_ADDSTRING, 0,
                                       (LPARAM) (*EffectPointsIterator)->Name);
                }
                if (!feof(f)) {
                    long version = 0;
                    fread(&version, sizeof(long), 1, f);

                    if (version >= 1) {
                        long fixorient = DEFAULT_SCN_FIXORIENT;
                        fread(&fixorient, sizeof(long), 1, f);
                        CheckDlgButton(hwnd, IDC_FIXORIENT, fixorient);
                        CurrentFixOrient = fixorient;
                    } else {
                        CheckDlgButton(hwnd, IDC_FIXORIENT, false);
                    }
                    // SCN Version 2 Stuff
                    if (version >= 2) {
                        unsigned int sivflags = 0;
                        fread(&sivflags, sizeof(unsigned int), 1, f);
                        CheckDlgButton(hwnd, IDC_GREENERY, (sivflags & OVL_SIV_GREENERY));
                        CheckDlgButton(hwnd, IDC_FLOWERS, (sivflags & OVL_SIV_FLOWER));
                        CheckDlgButton(hwnd, IDC_VARYROTATION, (sivflags & OVL_SIV_ROTATION));
                        CheckDlgButton(hwnd, IDC_UNKNOWNBITS, (sivflags & OVL_SIV_UNKNOWN));
                        if (sivflags & OVL_SIV_SOAKED)
                            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 1, 0);
                        else if (sivflags & OVL_SIV_WILD)
                            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 2, 0);
                        else
                            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);

                        char temp[256];
                        float fl;
                        fread(&fl, sizeof(float), 1, f);
                        snprintf(temp, 255, "%f", fl);
                        SetDlgItemText(hwnd, IDC_SWAY, temp);

                        fread(&fl, sizeof(float), 1, f);
                        snprintf(temp, 255, "%f", fl);
                        SetDlgItemText(hwnd, IDC_BRIGHTNESS, temp);

                        fread(&fl, sizeof(float), 1, f);
                        snprintf(temp, 255, "%f", fl);
                        SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, temp);

                        fread(&fl, sizeof(float), 1, f);
                        snprintf(temp, 255, "%f", fl);
                        SetDlgItemText(hwnd, IDC_SCALE, temp);

                    } else {
                        SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
                        SetDlgItemText(hwnd, IDC_SWAY, "0.0");
                        SetDlgItemText(hwnd, IDC_BRIGHTNESS, "1.0");
                        SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
                        SetDlgItemText(hwnd, IDC_SCALE, "0.0");
                        CheckDlgButton(hwnd, IDC_GREENERY, false);
                        CheckDlgButton(hwnd, IDC_FLOWERS, false);
                        CheckDlgButton(hwnd, IDC_VARYROTATION, false);
                        CheckDlgButton(hwnd, IDC_UNKNOWNBITS, false);
                    }
                } else {
                    CheckDlgButton(hwnd, IDC_FIXORIENT, false);

                    SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
                    SetDlgItemText(hwnd, IDC_SWAY, "0.0");
                    SetDlgItemText(hwnd, IDC_BRIGHTNESS, "1.0");
                    SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
                    SetDlgItemText(hwnd, IDC_SCALE, "0.0");
                    CheckDlgButton(hwnd, IDC_GREENERY, false);
                    CheckDlgButton(hwnd, IDC_FLOWERS, false);
                    CheckDlgButton(hwnd, IDC_VARYROTATION, false);
                    CheckDlgButton(hwnd, IDC_UNKNOWNBITS, false);
                }
                fclose(f);
                scenerysave = false;
            }
            break;
        case ID_FILE_NEWSCENERY:
            if (scenerysave == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            SendDlgItemMessage(hwnd, IDC_MESHES, LB_RESETCONTENT, 0, 0);
            SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_RESETCONTENT, 0, 0);
            SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
            SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
            CheckDlgButton(hwnd, IDC_FIXORIENT, DEFAULT_SCN_FIXORIENT);
            SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_SETCURSEL, 0, 0);
            SetDlgItemText(hwnd, IDC_SWAY, "0.0");
            SetDlgItemText(hwnd, IDC_BRIGHTNESS, "1.0");
            SetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, "1.0");
            SetDlgItemText(hwnd, IDC_SCALE, "0.0");
            CheckDlgButton(hwnd, IDC_GREENERY, false);
            CheckDlgButton(hwnd, IDC_FLOWERS, false);
            CheckDlgButton(hwnd, IDC_VARYROTATION, false);
            CheckDlgButton(hwnd, IDC_UNKNOWNBITS, false);
            CleanUpScenery();
            SetDlgItemText(hwnd, IDC_ASE, NULL);
            scenerysave = false;
            break;
        case ID_FILE_SAVEOVL: {
            bool textureovl = false;
            if (asename[0] == 0) {
                if (MessageBox(hwnd,
                           "You did not select an ASE file. Do you want to create a texture ovl?",
                           "Query", MB_YESNO | MB_ICONQUESTION) == IDNO)
                    break;
                textureovl = true;
            }
            unsigned long objectCount = 0;
            BOOL failed = false;
            for (MeshStructsIterator = MeshStructs.begin();
                    MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                if ((*MeshStructsIterator)->disabled == false) {
                    objectCount++;
                    if (((*MeshStructsIterator)->FTX == 0)
                            || ((*MeshStructsIterator)->TXS == 0)
                            || (strlen((*MeshStructsIterator)->FTX) == 0)
                            || (strlen((*MeshStructsIterator)->TXS) == 0)) {
                        char err[256];
                        snprintf(err, 255,
                                 "Can't Save Scenery Ovl File due to missing data in mesh \"%s\"",
                                 (*MeshStructsIterator)->Name);
                        MessageBox(hwnd, err, "Error", MB_OK);
                        failed = true;
                        break;
                    }
                }
            }
            if (failed)
                break;
            if ((!textureovl) && (objectCount == 0)) {
                MessageBox(hwnd, "No meshes to export", "Error", MB_OK);
                break;
            }
            if (!strcmp(sfile, "")) {
                char fname[_MAX_FNAME];
                _splitpath(asename, NULL, NULL, fname, NULL);
                GetDlgItemText(GetParent(hwnd), IDC_THEMEPREFIX, sfile, MAX_PATH);
                strncat(sfile, fname, MAX_PATH);
            }
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Overlay Files (*.common.ovl)\0*.common.ovl\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Save Scenery OVL file";
            ofn.lpstrDefExt = ".common.ovl";
            ofn.nMaxFile = MAX_PATH;
            ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
            if (GetSaveFileName(&ofn)) {
                StaticShape1 *sh = new StaticShape1;
                sh->EffectCount = EffectPoints.size();
                sh->MeshCount = objectCount;
                sh->MeshCount2 = objectCount;
                sh->TotalIndexCount = 0;
                sh->TotalVertexCount = 0;
                sh->BoundingBox2.x = FLT_MIN;
                sh->BoundingBox2.y = FLT_MIN;
                sh->BoundingBox2.z = FLT_MIN;
                sh->BoundingBox1.x = FLT_MAX;
                sh->BoundingBox1.y = FLT_MAX;
                sh->BoundingBox1.z = FLT_MAX;
                sh->sh = new StaticShape2 *[objectCount];
                unsigned long fixorient = SendDlgItemMessage(hwnd, IDC_FIXORIENT,
                                          BM_GETCHECK, 0, 0);
                if (EffectPoints.size() != 0) {
                    unsigned int e;
                    sh->EffectName = new char *[EffectPoints.size()];
                    sh->EffectPosition = new D3DMATRIX[EffectPoints.size()];
                    for (e = 0; e < EffectPoints.size(); e++) {
                        sh->EffectName[e] = EffectPoints[e]->Name;
                        if (fixorient) {
                            sh->EffectPosition[e]._11 = EffectPoints[e]->Transform._22;
                            sh->EffectPosition[e]._12 = EffectPoints[e]->Transform._23;
                            sh->EffectPosition[e]._13 = -EffectPoints[e]->Transform._21;
                            sh->EffectPosition[e]._14 = EffectPoints[e]->Transform._24;
                            sh->EffectPosition[e]._21 = EffectPoints[e]->Transform._32;
                            sh->EffectPosition[e]._22 = EffectPoints[e]->Transform._33;
                            sh->EffectPosition[e]._23 = -EffectPoints[e]->Transform._31;
                            sh->EffectPosition[e]._24 = EffectPoints[e]->Transform._34;
                            sh->EffectPosition[e]._31 = -EffectPoints[e]->Transform._12;
                            sh->EffectPosition[e]._32 = -EffectPoints[e]->Transform._13;
                            sh->EffectPosition[e]._33 = EffectPoints[e]->Transform._11;
                            sh->EffectPosition[e]._34 = -EffectPoints[e]->Transform._14;
                            sh->EffectPosition[e]._41 = EffectPoints[e]->Transform._42;
                            sh->EffectPosition[e]._42 = EffectPoints[e]->Transform._43;
                            sh->EffectPosition[e]._43 = -EffectPoints[e]->Transform._41;
                            sh->EffectPosition[e]._44 = EffectPoints[e]->Transform._44;
                        } else
                            sh->EffectPosition[e] = EffectPoints[e]->Transform;
                    }
                } else {
                    sh->EffectName = 0;
                    sh->EffectPosition = 0;
                }
                ASE_Scene *scene = ASE_loadFilename(asename);
                unsigned long CurrentMesh = 0;
                unsigned long CurrentObj = 0;
                for (MeshStructsIterator = MeshStructs.begin();
                        MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                    if ((*MeshStructsIterator)->disabled == false) {
                        StaticShape2 *sh2 = new StaticShape2;
                        sh->sh[CurrentMesh] = sh2;
                        sh2->fts = 0;
                        sh2->PlaceTexturing = (*MeshStructsIterator)->place;
                        sh2->TextureData = 0;
                        sh2->textureflags = (*MeshStructsIterator)->flags;
                        sh2->unk1 = 0xFFFFFFFF;
                        sh2->unk4 = (*MeshStructsIterator)->unknown;
                        ASE_Mesh curobj = scene->objs[CurrentObj].mesh;

                        if (fixorient) {
                            // Let's fix the orientation
                            for (i = 0; i < curobj.vertexCount; i++) {
                                float temp;
                                /* Old Version
                                   temp = curobj.vertices[i].y;
                                   curobj.vertices[i].y = curobj.vertices[i].z;
                                   curobj.vertices[i].z = temp;
                                   temp = curobj.vertex_normals[i].y;
                                   curobj.vertex_normals[i].y = curobj.vertex_normals[i].z;
                                   curobj.vertex_normals[i].z = temp;
                                 */
                                temp = curobj.vertices[i].y;
                                curobj.vertices[i].y = curobj.vertices[i].z;
                                curobj.vertices[i].z = -curobj.vertices[i].x;
                                curobj.vertices[i].x = temp;
                                temp = curobj.vertex_normals[i].y;
                                curobj.vertex_normals[i].y = curobj.vertex_normals[i].z;
                                curobj.vertex_normals[i].z = -curobj.vertex_normals[i].x;
                                curobj.vertex_normals[i].x = temp;
                            }
                            for (i = 0; i < curobj.faceCount; i++) {
                                float temp;
                                // I don't know if this is necessary, but it doesn't hurt =)
                                temp = curobj.faces[i].face_normal.y;
                                curobj.faces[i].face_normal.y = curobj.faces[i].face_normal.z;
                                curobj.faces[i].face_normal.z = -curobj.faces[i].face_normal.x;
                                curobj.faces[i].face_normal.x = temp;
                                int temp2;
                                // This IS necessary. It reverses the sense order of the vertices
                                // which is used to put the texture on the right side of the face
                                temp2 = curobj.faces[i].vertex[0];
                                curobj.faces[i].vertex[0] = curobj.faces[i].vertex[1];
                                curobj.faces[i].vertex[1] = temp2;
                                temp2 = curobj.faces[i].texture_coordinates[0];
                                curobj.faces[i].texture_coordinates[0] =
                                    curobj.faces[i].texture_coordinates[1];
                                curobj.faces[i].texture_coordinates[1] = temp2;
                            }
                        }
                        // code added 7-28-2006 by DragonsIOA
                        ConvertASEMesh(sh, sh2, &curobj);

                        CurrentMesh++;
                    }
                    CurrentObj++;
                }

                char **ftx = new char *[objectCount];
                char **txs = new char *[objectCount];
                char fname[_MAX_FNAME];
                _splitpath(asename, NULL, NULL, fname, NULL);
                char *name = new char[strlen(fname) + 1];
                strcpy(name, fname);
                CurrentObj = 0;
                for (MeshStructsIterator = MeshStructs.begin();
                        MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                    if ((*MeshStructsIterator)->disabled == false) {
                        char *x = (*MeshStructsIterator)->FTX;
                        ftx[CurrentObj] = new char[strlen(x) + 1];
                        strcpy(ftx[CurrentObj], x);
                        x = (*MeshStructsIterator)->TXS;
                        txs[CurrentObj] = new char[strlen(x) + 1];
                        strcpy(txs[CurrentObj], x);
                        CurrentObj++;
                    }
                }
                CreateSceneryOvl(sfile);
                if (objectCount)
                    AddStaticShape(name, ftx, txs, sh);
                else
                    delete sh;
                char *ovlname;
                _splitpath(sfile, NULL, NULL, fname, NULL);
                strchr(fname, '.')[0] = 0;
                ovlname = new char[strlen(fname) + 1];
                strcpy(ovlname, fname);
                unsigned char *svd;
                SceneryItemVisual *sv;
                SceneryItemVisualLOD **svlod;
                SceneryItemVisualLOD *svlod1;
                SceneryItemVisualLOD *svlod2;
                SceneryItemVisualLOD *svlod3;
                unsigned long scenerysize =
                    sizeof(SceneryItemVisual) + (sizeof(SceneryItemVisualLOD) * 3) + 12;
                svd = new unsigned char[scenerysize];
                memset(svd, 0, scenerysize);
                sv = (SceneryItemVisual *) svd;
                svlod = (SceneryItemVisualLOD **) (svd + sizeof(SceneryItemVisual));
                svlod1 = (SceneryItemVisualLOD *) (svd + sizeof(SceneryItemVisual)
                                                   + 12);
                svlod2 = (SceneryItemVisualLOD *) (svd + sizeof(SceneryItemVisual)
                                                   + 12 + sizeof(SceneryItemVisualLOD));
                svlod3 = (SceneryItemVisualLOD *) (svd + sizeof(SceneryItemVisual)
                                                   + 12 + (sizeof(SceneryItemVisualLOD) * 2));
                /*		    sv->sivfalgs = 0;
                		    sv->sway = 0.0;
                		    sv->brightness = 1.0;
                		    sv->unk4 = 1.0;
                		    sv->scale = 0.0; */
                unsigned int sivflags = 0;
                if (SendDlgItemMessage(hwnd, IDC_GREENERY, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_GREENERY;
                if (SendDlgItemMessage(hwnd, IDC_FLOWERS, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_FLOWER;
                if (SendDlgItemMessage(hwnd, IDC_VARYROTATION, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_ROTATION;
                if (SendDlgItemMessage(hwnd, IDC_UNKNOWNBITS, BM_GETCHECK, 0, 0))
                    sivflags |= OVL_SIV_UNKNOWN;
                unsigned long exp =
                    SendDlgItemMessage(hwnd, IDC_EXPANSIONFLAG, CB_GETCURSEL, 0, 0);
                if (exp == 1)
                    sivflags |= OVL_SIV_SOAKED;
                else if (exp == 2)
                    sivflags |= OVL_SIV_WILD;
                sv->sivflags = sivflags;

                int len = GetWindowTextLength(GetDlgItem(hwnd, IDC_SWAY));
                char *temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_SWAY, temp, len + 1);
                sv->sway = (float) atof(temp);
                delete temp;

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_BRIGHTNESS));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_BRIGHTNESS, temp, len + 1);
                sv->brightness = (float) atof(temp);
                delete temp;

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_UNKNOWNFACTOR));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_UNKNOWNFACTOR, temp, len + 1);
                sv->unk4 = (float) atof(temp);
                delete temp;

                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_SCALE));
                temp = new char[len + 1];
                GetDlgItemText(hwnd, IDC_SCALE, temp, len + 1);
                sv->scale = (float) atof(temp);
                delete temp;

                sv->LODCount = 3;
                sv->LODMeshes = svlod;
                sv->unk6 = 0;
                sv->unk7 = 0;
                sv->unk8 = 0;
                sv->unk9 = 0;
                sv->unk10 = 0;
                sv->unk11 = 0;
                svlod[0] = svlod1;
                svlod[1] = svlod2;
                svlod[2] = svlod3;
                svlod1->MeshType = 0;
                svlod1->LODName = ovlname;
                svlod1->StaticShape = 0;
                svlod1->unk2 = 0;
                svlod1->BoneShape = 0;
                svlod1->unk4 = 0;
                svlod1->unk5 = 0;
                svlod1->unk6 = 0;
                svlod1->unk7 = 1.0;
                svlod1->unk8 = 1.0;
                svlod1->unk9 = 0;
                svlod1->unk10 = 1.0;
                svlod1->unk11 = 0;
                svlod1->unk12 = 1.0;
                svlod1->distance = 40;
                svlod1->AnimationCount = 0;
                svlod1->unk14 = 0;
                svlod1->AnimationArray = 0;
                svlod2->MeshType = 0;
                svlod2->LODName = ovlname;
                svlod2->StaticShape = 0;
                svlod2->unk2 = 0;
                svlod2->BoneShape = 0;
                svlod2->unk4 = 0;
                svlod2->unk5 = 0;
                svlod2->unk6 = 0;
                svlod2->unk7 = 1.0;
                svlod2->unk8 = 1.0;
                svlod2->unk9 = 0;
                svlod2->unk10 = 1.0;
                svlod2->unk11 = 0;
                svlod2->unk12 = 1.0;
                svlod2->distance = 100;
                svlod2->AnimationCount = 0;
                svlod2->unk14 = 0;
                svlod2->AnimationArray = 0;
                svlod3->MeshType = 0;
                svlod3->LODName = ovlname;
                svlod3->StaticShape = 0;
                svlod3->unk2 = 0;
                svlod3->BoneShape = 0;
                svlod3->unk4 = 0;
                svlod3->unk5 = 0;
                svlod3->unk6 = 0;
                svlod3->unk7 = 1.0;
                svlod3->unk8 = 1.0;
                svlod3->unk9 = 0;
                svlod3->unk10 = 1.0;
                svlod3->unk11 = 0;
                svlod3->unk12 = 1.0;
                svlod3->distance = 4000;
                svlod3->AnimationCount = 0;
                svlod3->unk14 = 0;
                svlod3->AnimationArray = 0;
                char *Names[3];
                Names[0] = name;
                Names[1] = name;
                Names[2] = name;
                if (objectCount)
                    AddSceneryItemVisual(ovlname, Names, sv);
                else {
                    delete ovlname;
                    delete svd;
                }
                bool error = false;
//                                              FILE *tex = 0;
//                                              FILE *alpha = 0;
                ILuint tex[2];
                ILenum Error;
                char tmperror[256];

                ilGenImages(2, (ILuint *) & tex);
                for (i = 0; i < FlexiTextures.size(); i++) {
                    //tex = fopen(FlexiTextures[i]->Texture,"rb");
                    ilBindImage(tex[0]);

                    if (!ilLoadImage(FlexiTextures[i]->Texture)) {
                        snprintf(tmperror, 255, "Can't open texture '%s'.",
                                 FlexiTextures[i]->Texture);
                        MessageBox(hwnd, tmperror, "Error", MB_OK);
                        while ((Error = ilGetError())) {
                            MessageBox(hwnd, iluErrorString(Error), "Error", MB_OK);
                        }
                        error = true;
                        break;
                    }
                    int width = ilGetInteger(IL_IMAGE_WIDTH);
                    int height = ilGetInteger(IL_IMAGE_HEIGHT);
                    if (width != height) {
                        snprintf(tmperror, 255, "Texture '%s' is not square.",
                                 FlexiTextures[i]->Texture);
                        MessageBox(hwnd, tmperror, "Error", MB_OK);
                        error = true;
                        break;
                    }
                    if ((1 << local_log2(width)) != width) {
                        snprintf(tmperror, 255,
                                 "The width/height of texture '%s' is not a power of 2.",
                                 FlexiTextures[i]->Texture);
                        MessageBox(hwnd, tmperror, "Error", MB_OK);
                        error = true;
                        break;
                    }
                    if (ilGetInteger(IL_IMAGE_FORMAT) != IL_COLOUR_INDEX) {
                        if (FlexiTextures[i]->Recolorable) {
                            if (ilGetInteger(IL_IMAGE_FORMAT) != IL_RGB)
                                ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
                            ILubyte *tmpdata = new ILubyte[width * height];
                            ILubyte *picdata = ilGetData();
                            unsigned int paldist /*[256] */ , palmax;
                            int r, d1, d2, d3;
                            int p;
                            for (r = 0; r < height * width * 3; r += 3) {
                                palmax = UINT_MAX;
                                for (p = 1; p < 256; p++) {	// Start with 1, do not assign white
                                    d1 = (int) picdata[r] - (int) recolpalette_rgb[p].rgbRed;
                                    d2 = (int) picdata[r + 1] -
                                         (int) recolpalette_rgb[p].rgbGreen;
                                    d3 = (int) picdata[r + 2] -
                                         (int) recolpalette_rgb[p].rgbBlue;
//                                      paldist[p] = d1 * d1 + d2 * d2 + d3 * d3;
                                    paldist = d1 * d1 + d2 * d2 + d3 * d3;
                                    if (paldist < palmax) {
                                        palmax = paldist;
                                        tmpdata[r / 3] = p;
                                    }
                                }
//                                  palmax = UINT_MAX;
//                                  for (p = 1; p < 256; p++) {
//                                      if (paldist[p] < palmax) {
//                                          palmax = paldist[p];
//                                          tmpdata[r / 3] = p;
//                                      }
//                                  }
                            }

                            ILuint temppic = ilGenImage();
                            ilBindImage(temppic);
                            ilCopyImage(tex[0]);
                            if (iluColorsUsed() > 256)
                                ilSetInteger(IL_QUANTIZATION_MODE, IL_NEU_QUANT);
                            else
                                ilSetInteger(IL_QUANTIZATION_MODE, IL_WU_QUANT);
                            ilConvertImage(IL_COLOUR_INDEX, IL_UNSIGNED_BYTE);
                            ilSetData(tmpdata);
                            ilConvertPal(IL_PAL_BGR32);
                            memcpy(ilGetPalette(), &recolpalette_bmy, 256 * 4);
                            ilBindImage(tex[0]);
                            ilCopyImage(temppic);
                            ilDeleteImage(temppic);
                            delete tmpdata;
                        } else {
                            if (iluColorsUsed() > 256)
                                ilSetInteger(IL_QUANTIZATION_MODE, IL_NEU_QUANT);
                            else
                                ilSetInteger(IL_QUANTIZATION_MODE, IL_WU_QUANT);
                            if (!ilConvertImage(IL_COLOUR_INDEX, IL_UNSIGNED_BYTE)) {
                                snprintf(tmperror, 255,
                                         "Couldn't convert texture '%s' to indexed colour.",
                                         FlexiTextures[i]->Texture);
                                MessageBox(hwnd, tmperror, "Error", MB_OK);
                                error = true;
                                break;
                            }
                        }
                    }
                    if (ilGetInteger(IL_PALETTE_TYPE) != IL_PAL_BGR32)
                        if (!ilConvertPal(IL_PAL_BGR32)) {
                            snprintf(tmperror, 255,
                                     "Couldn't convert palette of texture '%s'.",
                                     FlexiTextures[i]->Texture);
                            MessageBox(hwnd, tmperror, "Error", MB_OK);
                            error = true;
                            break;
                        }
                    if (ilGetInteger(IL_IMAGE_ORIGIN) == IL_ORIGIN_UPPER_LEFT)
                        if (!iluFlipImage()) {
                            snprintf(tmperror, 255, "Couldn't flip texture '%s'.",
                                     FlexiTextures[i]->Texture);
                            MessageBox(hwnd, tmperror, "Error", MB_OK);
                            error = true;
                            break;
                        }
#if 0
                    if (!ilSaveImage("F:/test.bmp")) {
                        snprintf(tmperror, 255, "Can't save texture '%s'.",
                                 FlexiTextures[i]->Texture);
                        MessageBox(hwnd, tmperror, "Error", MB_OK);
                        while ((Error = ilGetError())) {
                            MessageBox(hwnd, iluErrorString(Error), "Error", MB_OK);
                        }
                    }
#endif
                    bool AlphaPresent = false;
                    if (FlexiTextures[i]->Alpha[0] != 0) {
                        AlphaPresent = true;
                    }
                    if (AlphaPresent == true) {
                        ilBindImage(tex[1]);

                        if (!ilLoadImage(FlexiTextures[i]->Alpha)) {
                            snprintf(tmperror, 255, "Can't open alpha texture '%s'.",
                                     FlexiTextures[i]->Alpha);
                            MessageBox(hwnd, tmperror, "Error", MB_OK);
                            while ((Error = ilGetError())) {
                                MessageBox(hwnd, iluErrorString(Error), "Error", MB_OK);
                            }
                            error = true;
                            break;
                        }
                        int alwidth = ilGetInteger(IL_IMAGE_WIDTH);
                        int alheight = ilGetInteger(IL_IMAGE_HEIGHT);
                        if ((width != alwidth) || (height != alheight)) {
                            snprintf(tmperror, 255,
                                     "Size of alpha texture '%s' does not match with '%s'.",
                                     FlexiTextures[i]->Alpha, FlexiTextures[i]->Texture);
                            MessageBox(hwnd, tmperror, "Error", MB_OK);
                            error = true;
                            break;
                        }
                        if (ilGetInteger(IL_IMAGE_FORMAT) != IL_LUMINANCE)
                            if (!ilConvertImage(IL_LUMINANCE, IL_UNSIGNED_BYTE)) {
                                snprintf(tmperror, 255,
                                         "Couldn't convert alpha texture '%s' to grayscale.",
                                         FlexiTextures[i]->Alpha);
                                MessageBox(hwnd, tmperror, "Error", MB_OK);
                                error = true;
                                break;
                            }
#if 0
                        if (!iluFlipImage()) {
                            snprintf(tmperror, 255, "Couldn't flip alpha texture '%s'.",
                                     FlexiTextures[i]->Alpha);
                            MessageBox(hwnd, tmperror, "Error", MB_OK);
                            error = true;
                            break;
                        }
#endif
                    }


                    /*
                       if (tex == 0)
                       {
                       MessageBox(hwnd,"Can't open texture","Error",MB_OK);
                       error = true;
                       break;
                       }
                       bool AlphaPresent = false;
                       if (FlexiTextures[i]->Alpha[0] != 0)
                       {
                       AlphaPresent = true;
                       }
                       if (AlphaPresent == true)
                       {
                       alpha = fopen(FlexiTextures[i]->Alpha,"rb");
                       if (alpha == 0)
                       {
                       MessageBox(hwnd,"Can't open alpha texture","Error",MB_OK);
                       error = true;
                       break;
                       }
                       }
                       fread(&bmf,sizeof(bmf),1,tex);
                       fread(&bmi,sizeof(bmi),1,tex);
                       if ((bmi.biWidth != bmi.biHeight) || (bmi.biBitCount != 8) || (bmi.biWidth % 2))
                       {
                       MessageBox(hwnd,"Texture not square 8bpp image with height/width a power of 2","Error",MB_OK);
                       error = true;
                       break;
                       }
                       if (AlphaPresent == true)
                       {
                       fread(&bmf2,sizeof(bmf2),1,alpha);
                       fread(&bmi2,sizeof(bmi2),1,alpha);
                       if ((bmi2.biWidth != bmi2.biHeight) || (bmi2.biBitCount != 8) || (bmi2.biWidth % 2))
                       {
                       MessageBox(hwnd,"Alpha texture not square 8bpp image with height/width a power of 2","Error",MB_OK);
                       error = true;
                       break;
                       }
                       if (bmi2.biWidth != bmi.biWidth)
                       {
                       MessageBox(hwnd,"Alpha texture not same size as texture","Error",MB_OK);
                       error = true;
                       break;
                       }
                       }
                     */
                    RGBQUAD *colors;
                    colors = new RGBQUAD[256];
                    memset(colors, 0, 256 * sizeof(RGBQUAD));
//                      if (AlphaPresent == true) {
//                          if (bmi2.biClrUsed == 0) {
//                              bmi2.biClrUsed = 256;
//                          }
//                          fread(colors, bmi2.biClrUsed * sizeof(RGBQUAD), 1, alpha);
//                          fseek(alpha, bmf2.bfOffBits, SEEK_SET);
//                      }
//                      if (bmi.biClrUsed == 0) {
//                          bmi.biClrUsed = 256;
//                      }
//                      fread(colors, bmi.biClrUsed * sizeof(RGBQUAD), 1, tex);
//                      fseek(tex, bmf.bfOffBits, SEEK_SET);
                    unsigned char *texture = 0;
                    unsigned char *alphadata = 0;
//                      texture = new unsigned char[bmi.biWidth * bmi.biHeight];
                    texture = new unsigned char[width * height];
                    ilBindImage(tex[0]);
                    memcpy(texture, ilGetData(), width * height);
                    memcpy(colors, ilGetPalette(), ilGetInteger(IL_PALETTE_NUM_COLS) * 4);
//                      fread(texture, bmi.biWidth * bmi.biHeight, 1, tex);
                    if (AlphaPresent == true) {
//                          alphadata = new unsigned char[bmi.biWidth * bmi.biHeight];
//                          fread(alphadata, bmi.biWidth * bmi.biHeight, 1, alpha);
                        alphadata = new unsigned char[width * height];
                        ilBindImage(tex[1]);
                        memcpy(alphadata, ilGetData(), width * height);
                    }
                    FlexiTextureStruct *fts = new FlexiTextureStruct;
                    FlexiTextureInfoStruct *fti = new FlexiTextureInfoStruct;
                    char *FlexiTextureName = new char[strlen(FlexiTextures[i]->Name) + 1];
                    strcpy(FlexiTextureName, FlexiTextures[i]->Name);
//                      fti->scale = local_log2(bmi.biWidth);
//                      fti->height = bmi.biHeight;
//                      fti->width = bmi.biWidth;
                    fti->scale = local_log2(width);
                    fti->height = height;
                    fti->width = width;
                    fti->unk1 = 0;
                    fti->Recolorable = FlexiTextures[i]->Recolorable;
                    fti->unk3 = 1;
                    fti->offset1 = 0;
                    fti->unk4 = 1;
                    fti->fts2 = fts;
                    fts->scale = fti->scale;
                    fts->width = fti->width;
                    fts->height = fti->height;
                    fts->Recolorable = fti->Recolorable;
                    fts->palette = (unsigned char *) colors;
                    fts->texture = texture;
                    fts->alpha = alphadata;
                    AddFlexiTexture(FlexiTextureName, fti);
//                                                      fclose(tex);
//                                                      fclose(alpha);
                }
                ilDeleteImages(2, (ILuint *) & tex);
                if (error == true) {
                    break;
                }
                for (i = 0; i < References.size(); i++) {
                    AddReference(References[i]);
                }
                SaveSceneryOvl();
            }
        }
        break;
        case ID_FILE_EXIT:
            if (scenerysave == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            EndDialog(hwnd, 0);
            break;
        case IDC_MESHES:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_MESHFLAGS:
            CurrentMeshStruct = SendDlgItemMessage(hwnd, IDC_MESHES, LB_GETCURSEL, 0, 0);
            if (CurrentMeshStruct != -1) {
                if (DialogBox(hInst, MAKEINTRESOURCE(IDD_MESHFLAGS), hwnd, MeshDlgProc) == 1) {
                    scenerysave = true;
                }
            }
            break;
        case IDC_FIXORIENT:
            scenerysave = true;
            CurrentFixOrient = SendDlgItemMessage(hwnd, IDC_FIXORIENT, BM_GETCHECK, 0, 0);
            break;
        case IDC_OPENASE:
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "ASE Files (*.ase)\0*.ase\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open ASE File";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".ase";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SetDlgItemText(hwnd, IDC_ASE, sfile);
            }
            aselen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ASE));
            if (aselen != 0) {
                char *ase = new char[aselen + 1];
                GetDlgItemText(hwnd, IDC_ASE, ase, aselen + 1);
                if (stricmp(ase, asename) != 0) {
                    delete asename;
                    asename = ase;
                    FILE *f = fopen(ase, "rt");
                    if (f == 0) {
                        MessageBox(hwnd, "Can't Open ASE File", "Error", MB_OK);
                        SetDlgItemText(hwnd, IDC_ASE, NULL);
                        break;
                    }
                    fclose(f);
                    SendDlgItemMessage(hwnd, IDC_MESHES, LB_RESETCONTENT, 0, 0);
                    CleanUpMeshStructs();
                    ASE_Scene *scene = ASE_loadFilename(ase);
                    for (long j = 0; j < scene->objectCount; j++) {
                        SendDlgItemMessage(hwnd, IDC_MESHES, LB_ADDSTRING, 0,
                                           (LPARAM) scene->objs[j].name);
                        MeshStruct *ms = new MeshStruct;
                        if ((strcmp(scene->objs[j].name, "root") == 0)
                                || (scene->objs[j].mesh.vertexCount < 3)) {
                            ms->disabled = true;
                        } else {
                            ms->disabled = false;
                        }
                        ms->Name = new char[strlen(scene->objs[j].name) + 1];
                        strcpy(ms->Name, scene->objs[j].name);
                        if (scene->objs[j].mesh.vertexCount == 1) {
                            ms->effectpoint = true;
                            ms->effectpoint_vert = scene->objs[j].mesh.vertices[0];
                        } else {
                            ms->effectpoint = false;
                        }
                        ms->flags = 0;
                        ms->place = 0;
                        ms->unknown = 3;
                        ms->FTX = 0;
                        ms->TXS = 0;
                        MeshStructs.push_back(ms);
                    }
                }
            }
            break;
        case IDC_ADDTEX:
            CurrentFlexiTexture = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTURE), hwnd, TextureDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
                for (FlexiTexturesIterator = FlexiTextures.begin();
                        FlexiTexturesIterator != FlexiTextures.end(); FlexiTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_ADDSTRING, 0,
                                       (LPARAM) (*FlexiTexturesIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_TEXTURES:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDITTEX:
            CurrentFlexiTexture = SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTURE), hwnd, TextureDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
                for (FlexiTexturesIterator = FlexiTextures.begin();
                        FlexiTexturesIterator != FlexiTextures.end(); FlexiTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_ADDSTRING, 0,
                                       (LPARAM) (*FlexiTexturesIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_DELTEX:
            CurrentFlexiTexture = SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_GETCURSEL, 0, 0);
            if (CurrentFlexiTexture != -1) {
                delete FlexiTextures[CurrentFlexiTexture]->Alpha;
                delete FlexiTextures[CurrentFlexiTexture]->Name;
                delete FlexiTextures[CurrentFlexiTexture]->Texture;
                FlexiTextures.erase(FlexiTextures.begin() + CurrentFlexiTexture);
                SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_RESETCONTENT, 0, 0);
                for (FlexiTexturesIterator = FlexiTextures.begin();
                        FlexiTexturesIterator != FlexiTextures.end(); FlexiTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_TEXTURES, LB_ADDSTRING, 0,
                                       (LPARAM) (*FlexiTexturesIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_ADDPT:
            CurrentEffectPoint = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_EFFECTPT), hwnd, EffectPointDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
                for (EffectPointsIterator = EffectPoints.begin();
                        EffectPointsIterator != EffectPoints.end(); EffectPointsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_ADDSTRING, 0,
                                       (LPARAM) (*EffectPointsIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_EFFECTPT:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDITPT:
            CurrentEffectPoint = SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_EFFECTPT), hwnd, EffectPointDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
                for (EffectPointsIterator = EffectPoints.begin();
                        EffectPointsIterator != EffectPoints.end(); EffectPointsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_ADDSTRING, 0,
                                       (LPARAM) (*EffectPointsIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_DELPT:
            CurrentEffectPoint = SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_GETCURSEL, 0, 0);
            if (CurrentEffectPoint != -1) {
                delete EffectPoints[CurrentEffectPoint]->Name;
                EffectPoints.erase(EffectPoints.begin() + CurrentEffectPoint);
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
                for (EffectPointsIterator = EffectPoints.begin();
                        EffectPointsIterator != EffectPoints.end(); EffectPointsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_ADDSTRING, 0,
                                       (LPARAM) (*EffectPointsIterator)->Name);
                }
                scenerysave = true;
            }
            break;
        case IDC_AUTOPT: {
            BOOL haspoints = false;
            for (MeshStructsIterator = MeshStructs.begin();
                    MeshStructsIterator != MeshStructs.end(); MeshStructsIterator++) {
                if ((*MeshStructsIterator)->effectpoint) {
                    haspoints = true;
                    EffectPoint *t = new EffectPoint;
                    EffectPoints.push_back(t);
                    CurrentEffectPoint = EffectPoints.size() - 1;

                    EffectPoint *e = EffectPoints[CurrentEffectPoint];
                    e->Name = new char[strlen((*MeshStructsIterator)->Name) + 1];
                    strcpy(e->Name, (*MeshStructsIterator)->Name);
                    e->Transform._11 = 1.0;
                    e->Transform._12 = 0.0;
                    e->Transform._13 = 0.0;
                    e->Transform._14 = 0.0;
                    e->Transform._21 = 0.0;
                    e->Transform._22 = 1.0;
                    e->Transform._23 = 0.0;
                    e->Transform._24 = 0.0;
                    e->Transform._31 = 0.0;
                    e->Transform._32 = 0.0;
                    e->Transform._33 = 1.0;
                    e->Transform._34 = 0.0;
                    e->Transform._41 = (*MeshStructsIterator)->effectpoint_vert.x;
                    e->Transform._42 =
                        (CurrentFixOrient) ? (*MeshStructsIterator)->effectpoint_vert.
                        z : (*MeshStructsIterator)->effectpoint_vert.y;
                    e->Transform._43 =
                        (CurrentFixOrient) ? (*MeshStructsIterator)->effectpoint_vert.
                        y : (*MeshStructsIterator)->effectpoint_vert.z;
                    e->Transform._44 = 1.0;
                }
            }
            if (haspoints) {
                SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_RESETCONTENT, 0, 0);
                for (EffectPointsIterator = EffectPoints.begin();
                        EffectPointsIterator != EffectPoints.end(); EffectPointsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_EFFECTPT, LB_ADDSTRING, 0, (LPARAM)
                                       (*EffectPointsIterator)->Name);
                }
                scenerysave = true;
            } else
                MessageBox(hwnd,
                           "No meshes with only one vertex found. These are necessary for automatic effect point creation.",
                           "Error", MB_OK | MB_ICONWARNING);
        }
        break;
        case IDC_DELREF:
            CurrentReference = SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_GETCURSEL, 0, 0);
            if (CurrentReference != -1) {
                delete References[CurrentReference];
                References.erase(References.begin() + CurrentReference);
                SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_RESETCONTENT, 0, 0);
                for (ReferencesIterator = References.begin();
                        ReferencesIterator != References.end(); ReferencesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_ADDSTRING, 0,
                                       (LPARAM) (*ReferencesIterator));
                }
                scenerysave = true;
            }
            break;
        case IDC_ADDREF:
            reflen = GetWindowTextLength(GetDlgItem(hwnd, IDC_REFERENCE));
            if (reflen != 0) {
                char *reference = new char[reflen + 1];
                GetDlgItemText(hwnd, IDC_REFERENCE, reference, reflen + 1);
                SetDlgItemText(hwnd, IDC_REFERENCE, NULL);
                References.push_back(reference);
                CurrentReference = References.size() - 1;
                SendDlgItemMessage(hwnd, IDC_REFERENCES, LB_ADDSTRING, 0, (LPARAM) reference);
                scenerysave = true;
            }
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK TextEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        if (CurrentTextString != -1) {
            SetDlgItemText(hwnd, IDC_NAME, TextStrings[CurrentTextString]->name);
            SetDlgItemText(hwnd, IDC_TEXT, TextStrings[CurrentTextString]->text);
        } else
            EnableWindow(GetDlgItem(hwnd, IDOK), false);
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_COPY_TEXT: {
            unsigned long len, len2;
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            len2 = GetWindowTextLength(GetDlgItem(hwnd, IDC_TEXT));
            if (len == 0) {
                if (len2 == 0) {
                    MessageBox(hwnd, "Nothing to copy =)", "Error", MB_OK | MB_ICONSTOP);
                } else {
                    char *tmp = new char[len2 + 1];
                    GetDlgItemText(hwnd, IDC_TEXT, tmp, len2 + 1);
                    SetDlgItemText(hwnd, IDC_NAME, tmp);
                    delete tmp;
                }
            } else {
                if (len2 == 0) {
                    char *tmp = new char[len + 1];
                    GetDlgItemText(hwnd, IDC_NAME, tmp, len + 1);
                    SetDlgItemText(hwnd, IDC_TEXT, tmp);
                    delete tmp;
                } else {
                    MessageBox(hwnd, "Um, I'm confused.\nEither Name or Text should be empty.",
                               "Error", MB_OK | MB_ICONSTOP);
                }
            }
        }
        break;
        case IDC_NAME:
        case IDC_TEXT: {
            unsigned long len, len2;
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            len2 = GetWindowTextLength(GetDlgItem(hwnd, IDC_TEXT));
            EnableWindow(GetDlgItem(hwnd, IDOK), !((len == 0) && (len2 == 0)));
        }
        break;
        case IDOK:
            unsigned long len, len2;
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            len2 = GetWindowTextLength(GetDlgItem(hwnd, IDC_TEXT));
            if ((len == 0) && (len2 == 0)) {
                MessageBox(hwnd, "You entered nothing. Use \'Cancel\' to end this dialog!", "Error",
                           MB_OK | MB_ICONSTOP);
            } else {
                save = true;
                if (CurrentTextString != -1) {
                    delete TextStrings[CurrentTextString]->name;
                    delete TextStrings[CurrentTextString]->text;
                } else {
                    Text *t = new Text;
                    TextStrings.push_back(t);
                    CurrentTextString = TextStrings.size() - 1;
                }
                int id = IDC_NAME;
                if (len == 0) {
                    id = IDC_TEXT;
                    len = len2;
                }
                TextStrings[CurrentTextString]->name = new char[len + 1];
                GetDlgItemText(hwnd, id, TextStrings[CurrentTextString]->name, len + 1);
                TextStrings[CurrentTextString]->text = new char[len2 + 1];
                GetDlgItemText(hwnd, IDC_TEXT, TextStrings[CurrentTextString]->text, len2 + 1);
                EndDialog(hwnd, 1);
            }
            break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

bool enabled;
BOOL CALLBACK GSIEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
        for (IconTexturesIterator = IconTextures.begin();
                IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
            SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                               (LPARAM) (*IconTexturesIterator)->name);
        }
        if (CurrentIcon != -1) {
            SetDlgItemText(hwnd, IDC_NAME, Icons[CurrentIcon]->name);
            SetDlgItemInt(hwnd, IDC_TOP, Icons[CurrentIcon]->top, FALSE);
            SetDlgItemInt(hwnd, IDC_LEFT, Icons[CurrentIcon]->left, FALSE);
            SetDlgItemInt(hwnd, IDC_BOTTOM, Icons[CurrentIcon]->bottom, FALSE);
            SetDlgItemInt(hwnd, IDC_RIGHT, Icons[CurrentIcon]->right, FALSE);
            unsigned long pos = SendDlgItemMessage(hwnd, IDC_LIST, LB_FINDSTRING,
                                                   (WPARAM) - 1,
                                                   (WPARAM) Icons[CurrentIcon]->texture);
            SendDlgItemMessage(hwnd, IDC_LIST, LB_SETCURSEL, pos, 0);
        }
        EnableWindow(GetDlgItem(hwnd, IDOK),
                     SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0) >= 0);
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_LIST:
            EnableWindow(GetDlgItem(hwnd, IDOK),
                         SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0) >= 0);
            break;
        case IDOK:
            unsigned long len;
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            BOOL fail1, fail2, fail3, fail4;
            unsigned long top, left, bottom, right, texture;
            top = GetDlgItemInt(hwnd, IDC_TOP, &fail1, FALSE);
            left = GetDlgItemInt(hwnd, IDC_LEFT, &fail2, FALSE);
            bottom = GetDlgItemInt(hwnd, IDC_BOTTOM, &fail3, FALSE);
            right = GetDlgItemInt(hwnd, IDC_RIGHT, &fail4, FALSE);
            texture = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
            if ((len != 0) && (fail1 == TRUE) && (fail2 == TRUE)
                    && (fail3 == TRUE) && (fail4 == TRUE) && (texture != -1)) {
                save = true;
                if (CurrentIcon != -1) {
                    delete Icons[CurrentIcon]->name;
                    delete Icons[CurrentIcon]->texture;
                } else {
                    Icon *t = new Icon;
                    Icons.push_back(t);
                    CurrentIcon = Icons.size() - 1;
                }
                Icons[CurrentIcon]->name = new char[len + 1];
                GetDlgItemText(hwnd, IDC_NAME, Icons[CurrentIcon]->name, len + 1);
                Icons[CurrentIcon]->top = top;
                Icons[CurrentIcon]->left = left;
                Icons[CurrentIcon]->bottom = bottom;
                Icons[CurrentIcon]->right = right;
                Icons[CurrentIcon]->texture = new char[strlen(IconTextures[texture]->name) + 1];
                strcpy(Icons[CurrentIcon]->texture, IconTextures[texture]->name);
            }
            EndDialog(hwnd, 1);
            break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        case IDC_CALCULATE:
            BOOL fail;
            unsigned long iconnum = GetDlgItemInt(hwnd, IDC_ICONNUM, &fail, FALSE);
            if (fail == TRUE) {
                int x = iconnum / 8;
                int y = iconnum % 8;
                SetDlgItemInt(hwnd, IDC_TOP, y * 64, FALSE);
                SetDlgItemInt(hwnd, IDC_BOTTOM, y * 64 + 63, FALSE);
                SetDlgItemInt(hwnd, IDC_LEFT, x * 64, FALSE);
                SetDlgItemInt(hwnd, IDC_RIGHT, x * 64 + 63, FALSE);
            }
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK TextureEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    char sfile[MAX_PATH] = "";
    switch (Message) {
    case WM_INITDIALOG:
        if (CurrentIconTexture != -1) {
            SetDlgItemText(hwnd, IDC_NAME, IconTextures[CurrentIconTexture]->name);
            SetDlgItemText(hwnd, IDC_FILENAME, IconTextures[CurrentIconTexture]->filename);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_TEXTURE_INFO: {
            char tmp[MAX_PATH];
            GetDlgItemText(hwnd, IDC_FILENAME, tmp, MAX_PATH - 1);
            if (strlen(tmp))
                showBitmapInfo(hwnd, tmp);
            else
                MessageBox(hwnd, "Please select a texture before requesting info on it ;-)",
                           "Error", MB_OK | MB_ICONSTOP);
        }
        break;
        case IDC_LOAD:
            OPENFILENAME ofn;
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter =
                "Textures (*.png;*.bmp;*.jpg;*.tga)\0*.png;*.jpg;*.jpeg;*.bmp;*.tga\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Icon Texture";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".png";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SetDlgItemText(hwnd, IDC_FILENAME, sfile);
                unsigned long NameLen = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
                if (!NameLen) {
                    char temp[MAX_PATH];
                    _splitpath(sfile, NULL, NULL, temp, NULL);
                    SetDlgItemText(hwnd, IDC_NAME, temp);
                }
            }
            break;
        case IDOK:
            unsigned long len, len2;
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_NAME));
            len2 = GetWindowTextLength(GetDlgItem(hwnd, IDC_FILENAME));
            if ((len != 0) && (len2 != 0)) {
                save = true;
                if (CurrentIconTexture != -1) {
                    delete IconTextures[CurrentIconTexture]->name;
                    delete IconTextures[CurrentIconTexture]->filename;
                } else {
                    IconTexture *t = new IconTexture;
                    IconTextures.push_back(t);
                    CurrentIconTexture = IconTextures.size() - 1;
                }
                IconTextures[CurrentIconTexture]->name = new char[len + 1];
                GetDlgItemText(hwnd, IDC_NAME, IconTextures[CurrentIconTexture]->name, len + 1);
                IconTextures[CurrentIconTexture]->filename = new char[len2 + 1];
                GetDlgItemText(hwnd, IDC_FILENAME, IconTextures[CurrentIconTexture]->filename,
                               len2 + 1);

                ReadIconTexture(IconTextures[CurrentIconTexture]);

            }
            EndDialog(hwnd, 1);
            break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK SIDEditDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    char sfile[MAX_PATH] = "";
    switch (Message) {
    case WM_INITDIALOG:
        // Circumvent Bug in ResEdit
        SetWindowLong(GetDlgItem(hwnd, IDC_SID_SHOW_PATH), GWL_STYLE,
                      GetWindowLong(GetDlgItem(hwnd, IDC_SID_SHOW_PATH),
                                    GWL_STYLE) | SS_PATHELLIPSIS);

        SendDlgItemMessage(hwnd, IDC_SCENICON, CB_RESETCONTENT, 0, 0);
        for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
            SendDlgItemMessage(hwnd, IDC_SCENICON, CB_ADDSTRING, 0,
                               (LPARAM) (*IconsIterator)->name);
        }
        SendDlgItemMessage(hwnd, IDC_WALLICON, CB_RESETCONTENT, 0, 0);
        for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
            SendDlgItemMessage(hwnd, IDC_WALLICON, CB_ADDSTRING, 0,
                               (LPARAM) (*IconsIterator)->name);
        }
        SendDlgItemMessage(hwnd, IDC_WALLNAME, CB_RESETCONTENT, 0, 0);
        for (TextStringsIterator = TextStrings.begin();
                TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
            SendDlgItemMessage(hwnd, IDC_WALLNAME, CB_ADDSTRING, 0,
                               (LPARAM) (*TextStringsIterator)->name);
        }
        SendDlgItemMessage(hwnd, IDC_SCENNAME, CB_RESETCONTENT, 0, 0);
        for (TextStringsIterator = TextStrings.begin();
                TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
            SendDlgItemMessage(hwnd, IDC_SCENNAME, CB_ADDSTRING, 0,
                               (LPARAM) (*TextStringsIterator)->name);
        }
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Tree");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Plant");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Shrub");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Flowers");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Fence");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Wall Misc");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Path Lamp");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Small Scenery");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Medium Scenery");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Large Scenery");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Anamatronic Scenery");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Misc Scenery");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Middle");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Top");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Bottom");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Bottom Extra");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Girder");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Support Cap");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Ride Track");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Path");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Park Entrance");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Litter");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Guest Injection");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Ride");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Ride Entrance");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Ride Exit");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Keep Clear Fence");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Stall");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Ride Event");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Firework");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Litter Bin");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Bench");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Sign");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Photo Point");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Wall Straight");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Wall Roof");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Wall Corner");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Water Cannon (not used)");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Pool Piece");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Pool Extra");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Changing Room");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Laser Dome");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Water Jet");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Terrain Piece");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Particle Effect");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Animal Fence");
        SendDlgItemMessage(hwnd, IDC_TYPE, CB_ADDSTRING, 0, (WPARAM) "Animal Misc");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "None");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Adventure");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Crazy Golf");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Crazy Golf Holes");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Girder");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Ornate");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Asphalt");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Crazy");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Dirt");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Leafy");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Marble");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Tarmac");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Queue Set 1");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Roman Pillar");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Scaffolding");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Space");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Spooky");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Steel");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS1");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS2");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS3");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS4");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS5");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Wild West");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Aquarium");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Aquarium Corner");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Pool");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Pool Path");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Pool Small");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Sand");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "Underwater");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS1A");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS6");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS7");
        SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_ADDSTRING, 0, (WPARAM) "TS8");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 1");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 2");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 3");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 4");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 5");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 6");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 7");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 8");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 9");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 10");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 11");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 12");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 13");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 14");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 15");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 16");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 17");
        SendDlgItemMessage(hwnd, IDC_UNK4, CB_ADDSTRING, 0, (WPARAM) "unknown 18");
        SendDlgItemMessage(hwnd, IDC_UNK17, CB_ADDSTRING, 0, (WPARAM) "unknown 1");
        SendDlgItemMessage(hwnd, IDC_UNK17, CB_ADDSTRING, 0, (WPARAM) "unknown 2");
        SendDlgItemMessage(hwnd, IDC_UNK17, CB_ADDSTRING, 0, (WPARAM) "unknown 3");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Full Tile");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Path Edge (Inner)");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Path Edge (Outer)");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Wall");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "1/4 Tile");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "1/2 Tile");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Path Center");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Corner");
        SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_ADDSTRING, 0, (WPARAM) "Path Edge (Join)");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "None");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Guest Injection Point");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Litter Bin");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Bench");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Park Entrance");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Queue Line Entertainer");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Ride Event");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Sign");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Visa Point");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Waterable");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Path Fountain");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Loudspeaker");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Passport Post");
        SendDlgItemMessage(hwnd, IDC_PARAMS, CB_ADDSTRING, 0, (WPARAM) "Water Cannon");
        if (SceneryItems.size() > 0) {
            for (SceneryItemsIterator = SceneryItems.begin();
                    SceneryItemsIterator != SceneryItems.end(); SceneryItemsIterator++) {
                if (SendDlgItemMessage(hwnd, IDC_LOCATION, CB_FINDSTRING, 0,
                                       (LPARAM) (*SceneryItemsIterator)->location) == CB_ERR)
                    SendDlgItemMessage(hwnd, IDC_LOCATION, CB_ADDSTRING, 0,
                                       (LPARAM) (*SceneryItemsIterator)->location);
            }
        }

        if (CurrentScenery != -1) {
            SetDlgItemText(hwnd, IDC_OVL, SceneryItems[CurrentScenery]->ovl);
            SetDlgItemText(hwnd, IDC_LOCATION, SceneryItems[CurrentScenery]->location);
            SetDlgItemInt(hwnd, IDC_COST, SceneryItems[CurrentScenery]->cost, TRUE);
            SetDlgItemInt(hwnd, IDC_REFUND, SceneryItems[CurrentScenery]->refund, TRUE);
            SetDlgItemInt(hwnd, IDC_XSQUARES, SceneryItems[CurrentScenery]->sizex, FALSE);
            SetDlgItemInt(hwnd, IDC_YSQUARES, SceneryItems[CurrentScenery]->sizey, FALSE);
            SetDlgItemInt(hwnd, IDC_UNK31, SceneryItems[CurrentScenery]->unk31, FALSE);
            SetDlgItemInt(hwnd, IDC_UNK32, SceneryItems[CurrentScenery]->unk32, FALSE);
            SetDlgItemInt(hwnd, IDC_UNK33, SceneryItems[CurrentScenery]->unk33, FALSE);
            CheckDlgButton(hwnd, IDC_FLAG33, SceneryItems[CurrentScenery]->flag33);
            CheckDlgButton(hwnd, IDC_FLAG34, SceneryItems[CurrentScenery]->flag34);
            CheckDlgButton(hwnd, IDC_FLAG35, SceneryItems[CurrentScenery]->flag35);
            CheckDlgButton(hwnd, IDC_FLAG36, SceneryItems[CurrentScenery]->flag36);
            CheckDlgButton(hwnd, IDC_FLAG37, SceneryItems[CurrentScenery]->flag37);
            CheckDlgButton(hwnd, IDC_FLAG38, SceneryItems[CurrentScenery]->flag38);
            CheckDlgButton(hwnd, IDC_FLAG39, SceneryItems[CurrentScenery]->flag39);
            CheckDlgButton(hwnd, IDC_FLAG40, SceneryItems[CurrentScenery]->flag40);
            CheckDlgButton(hwnd, IDC_FLAG41, SceneryItems[CurrentScenery]->flag41);
            CheckDlgButton(hwnd, IDC_FLAG42, SceneryItems[CurrentScenery]->flag42);
            CheckDlgButton(hwnd, IDC_FLAG43, SceneryItems[CurrentScenery]->flag43);
            CheckDlgButton(hwnd, IDC_FLAG44, SceneryItems[CurrentScenery]->flag44);
            CheckDlgButton(hwnd, IDC_FLAG45, SceneryItems[CurrentScenery]->flag45);
            CheckDlgButton(hwnd, IDC_FLAG46, SceneryItems[CurrentScenery]->flag46);
            CheckDlgButton(hwnd, IDC_FLAG47, SceneryItems[CurrentScenery]->flag47);
            CheckDlgButton(hwnd, IDC_FLAG48, SceneryItems[CurrentScenery]->flag48);
            CheckDlgButton(hwnd, IDC_FLAG49, SceneryItems[CurrentScenery]->flag49);
            CheckDlgButton(hwnd, IDC_FLAG50, SceneryItems[CurrentScenery]->flag50);
            CheckDlgButton(hwnd, IDC_FLAG51, SceneryItems[CurrentScenery]->flag51);
            CheckDlgButton(hwnd, IDC_FLAG52, SceneryItems[CurrentScenery]->flag52);
            CheckDlgButton(hwnd, IDC_FLAG53, SceneryItems[CurrentScenery]->flag53);
            CheckDlgButton(hwnd, IDC_FLAG54, SceneryItems[CurrentScenery]->flag54);
            CheckDlgButton(hwnd, IDC_FLAG55, SceneryItems[CurrentScenery]->flag55);
            CheckDlgButton(hwnd, IDC_FLAG56, SceneryItems[CurrentScenery]->flag56);
            CheckDlgButton(hwnd, IDC_FLAG57, SceneryItems[CurrentScenery]->flag57);
            CheckDlgButton(hwnd, IDC_FLAG58, SceneryItems[CurrentScenery]->flag58);
            CheckDlgButton(hwnd, IDC_FLAG59, SceneryItems[CurrentScenery]->flag59);
            CheckDlgButton(hwnd, IDC_FLAG60, SceneryItems[CurrentScenery]->flag60);
            CheckDlgButton(hwnd, IDC_FLAG61, SceneryItems[CurrentScenery]->flag61);
            CheckDlgButton(hwnd, IDC_FLAG62, SceneryItems[CurrentScenery]->flag62);
            CheckDlgButton(hwnd, IDC_FLAG63, SceneryItems[CurrentScenery]->flag63);
            CheckDlgButton(hwnd, IDC_FLAG64, SceneryItems[CurrentScenery]->flag64);
            SetDlgItemInt(hwnd, IDC_DUNK2, SceneryItems[CurrentScenery]->dunk2, TRUE);
            SetDlgItemInt(hwnd, IDC_DUNK3, SceneryItems[CurrentScenery]->dunk3, FALSE);
            SetDlgItemInt(hwnd, IDC_DUNK4, SceneryItems[CurrentScenery]->dunk4, TRUE);
            if (SceneryItems[CurrentScenery]->dunk4 == 0xFFFFFFFF) {
                CheckDlgButton(hwnd, IDC_DDUNK4, true);
                SetDlgItemInt(hwnd, IDC_DUNK4, 0, true);
                EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), false);
            } else {
                CheckDlgButton(hwnd, IDC_DDUNK4, false);
                EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), true);
            }
            SetDlgItemInt(hwnd, IDC_DUNK5, SceneryItems[CurrentScenery]->dunk5, FALSE);
            CheckDlgButton(hwnd, IDC_FLAG1, SceneryItems[CurrentScenery]->flag1);
            CheckDlgButton(hwnd, IDC_FLAG2, SceneryItems[CurrentScenery]->flag2);
            CheckDlgButton(hwnd, IDC_FLAG3, SceneryItems[CurrentScenery]->flag3);
            CheckDlgButton(hwnd, IDC_FLAG4, SceneryItems[CurrentScenery]->flag4);
            CheckDlgButton(hwnd, IDC_FLAG5, SceneryItems[CurrentScenery]->flag5);
            CheckDlgButton(hwnd, IDC_FLAG6, SceneryItems[CurrentScenery]->flag6);
            CheckDlgButton(hwnd, IDC_FLAG7, SceneryItems[CurrentScenery]->flag7);
            CheckDlgButton(hwnd, IDC_FLAG8, SceneryItems[CurrentScenery]->flag8);
            CheckDlgButton(hwnd, IDC_FLAG9, SceneryItems[CurrentScenery]->flag9);
            CheckDlgButton(hwnd, IDC_FLAG10, SceneryItems[CurrentScenery]->flag10);
            CheckDlgButton(hwnd, IDC_FLAG11, SceneryItems[CurrentScenery]->flag11);
            CheckDlgButton(hwnd, IDC_FLAG12, SceneryItems[CurrentScenery]->flag12);
            CheckDlgButton(hwnd, IDC_FLAG13, SceneryItems[CurrentScenery]->flag13);
            CheckDlgButton(hwnd, IDC_FLAG14, SceneryItems[CurrentScenery]->flag14);
            CheckDlgButton(hwnd, IDC_FLAG15, SceneryItems[CurrentScenery]->flag15);
            CheckDlgButton(hwnd, IDC_FLAG16, SceneryItems[CurrentScenery]->flag16);
            CheckDlgButton(hwnd, IDC_FLAG17, SceneryItems[CurrentScenery]->flag17);
            CheckDlgButton(hwnd, IDC_FLAG18, SceneryItems[CurrentScenery]->flag18);
            CheckDlgButton(hwnd, IDC_FLAG19, SceneryItems[CurrentScenery]->flag19);
            CheckDlgButton(hwnd, IDC_FLAG20, SceneryItems[CurrentScenery]->flag20);
            CheckDlgButton(hwnd, IDC_FLAG21, SceneryItems[CurrentScenery]->flag21);
            CheckDlgButton(hwnd, IDC_FLAG22, SceneryItems[CurrentScenery]->flag22);
            CheckDlgButton(hwnd, IDC_FLAG23, SceneryItems[CurrentScenery]->flag23);
            CheckDlgButton(hwnd, IDC_FLAG24, SceneryItems[CurrentScenery]->flag24);
            CheckDlgButton(hwnd, IDC_FLAG25, SceneryItems[CurrentScenery]->flag25);
            CheckDlgButton(hwnd, IDC_FLAG26, SceneryItems[CurrentScenery]->flag26);
            CheckDlgButton(hwnd, IDC_FLAG27, SceneryItems[CurrentScenery]->flag27);
            CheckDlgButton(hwnd, IDC_FLAG28, SceneryItems[CurrentScenery]->flag28);
            CheckDlgButton(hwnd, IDC_FLAG29, SceneryItems[CurrentScenery]->flag29);
            CheckDlgButton(hwnd, IDC_FLAG30, SceneryItems[CurrentScenery]->flag30);
            CheckDlgButton(hwnd, IDC_FLAG31, SceneryItems[CurrentScenery]->flag31);
            CheckDlgButton(hwnd, IDC_FLAG32, SceneryItems[CurrentScenery]->flag32);
            SendDlgItemMessage(hwnd, IDC_TYPE, CB_SETCURSEL, SceneryItems[CurrentScenery]->type, 0);
            SendDlgItemMessage(hwnd, IDC_UNK4, CB_SETCURSEL, SceneryItems[CurrentScenery]->unk4, 0);
            SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_SETCURSEL,
                               SceneryItems[CurrentScenery]->supportstype, 0);
            SendDlgItemMessage(hwnd, IDC_UNK17, CB_SETCURSEL,
                               SceneryItems[CurrentScenery]->unk17, 0);
            SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_SETCURSEL,
                               SceneryItems[CurrentScenery]->sizeflag, 0);
            SendDlgItemMessage(hwnd, IDC_PARAMS, CB_SETCURSEL,
                               SceneryItems[CurrentScenery]->extra, 0);
            unsigned long pos = SendDlgItemMessage(hwnd, IDC_SCENICON, CB_FINDSTRING,
                                                   (WPARAM) - 1,
                                                   (WPARAM) SceneryItems[CurrentScenery]->icon);
            SendDlgItemMessage(hwnd, IDC_SCENICON, CB_SETCURSEL, pos, 0);
            pos =
                SendDlgItemMessage(hwnd, IDC_SCENNAME, CB_FINDSTRING, (WPARAM) - 1,
                                   (WPARAM) SceneryItems[CurrentScenery]->name);
            SendDlgItemMessage(hwnd, IDC_SCENNAME, CB_SETCURSEL, pos, 0);
            if (SceneryItems[CurrentScenery]->wallicon != 0) {
                enabled = true;
                pos = SendDlgItemMessage(hwnd, IDC_WALLNAME, CB_FINDSTRING, (WPARAM) - 1, (WPARAM)
                                         SceneryItems[CurrentScenery]->wallname);
                SendDlgItemMessage(hwnd, IDC_WALLNAME, CB_SETCURSEL, pos, 0);
                pos = SendDlgItemMessage(hwnd, IDC_WALLICON, CB_FINDSTRING, (WPARAM) - 1, (WPARAM)
                                         SceneryItems[CurrentScenery]->wallicon);
                SendDlgItemMessage(hwnd, IDC_WALLICON, CB_SETCURSEL, pos, 0);
            } else {
                enabled = false;
                EnableWindow(GetDlgItem(hwnd, IDC_WALLICON), false);
                EnableWindow(GetDlgItem(hwnd, IDC_WALLNAME), false);
            }
            if (enabled == true) {
                CheckDlgButton(hwnd, IDC_DISABLE, false);
            } else {
                CheckDlgButton(hwnd, IDC_DISABLE, true);
            }
            char temp[100];
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->position.x);
            SetDlgItemText(hwnd, IDC_XPOS, temp);
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->position.y);
            SetDlgItemText(hwnd, IDC_YPOS, temp);
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->position.z);
            SetDlgItemText(hwnd, IDC_ZPOS, temp);
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->size.x);
            SetDlgItemText(hwnd, IDC_XSIZE, temp);
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->size.y);
            SetDlgItemText(hwnd, IDC_YSIZE, temp);
            sprintf(temp, "%f", SceneryItems[CurrentScenery]->size.z);
            SetDlgItemText(hwnd, IDC_ZSIZE, temp);
        } else {
            enabled = true;
            CheckDlgButton(hwnd, IDC_DISABLE, false);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_DISABLE:
            if (enabled == true) {
                enabled = false;
                EnableWindow(GetDlgItem(hwnd, IDC_WALLICON), false);
                EnableWindow(GetDlgItem(hwnd, IDC_WALLNAME), false);
            } else {
                enabled = true;
                EnableWindow(GetDlgItem(hwnd, IDC_WALLICON), true);
                EnableWindow(GetDlgItem(hwnd, IDC_WALLNAME), true);
            }
            break;
        case IDC_DDUNK4:
            if (IsDlgButtonChecked(hwnd, IDC_DDUNK4)) {
                EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), false);
            } else {
                EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), true);
            }
            break;
        case IDOK:
            unsigned long ovllen, loclen, sizex, sizey, wallicon, icon, wallname, name, type,
            unk4, supportstype, unk17;
            long cost, refund;
            unsigned long extra, sizeflag;
            unsigned long posxlen, posylen, poszlen, sizexlen, sizeylen, sizezlen;
            unsigned long unk31, unk32, unk33, dunk3, dunk5;
            long dunk2, dunk4;
            BOOL unk31fail, unk32fail, unk33fail, dunk2fail, dunk3fail, dunk4fail, dunk5fail;
            char *temp;
            BOOL costfail, refundfail, sizexfail, sizeyfail;
            ovllen = GetWindowTextLength(GetDlgItem(hwnd, IDC_OVL));
            loclen = GetWindowTextLength(GetDlgItem(hwnd, IDC_LOCATION));
            posxlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_XPOS));
            posylen = GetWindowTextLength(GetDlgItem(hwnd, IDC_YPOS));
            poszlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ZPOS));
            sizexlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_XSIZE));
            sizeylen = GetWindowTextLength(GetDlgItem(hwnd, IDC_YSIZE));
            sizezlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ZSIZE));
            sizex = GetDlgItemInt(hwnd, IDC_XSQUARES, &sizexfail, FALSE);
            sizey = GetDlgItemInt(hwnd, IDC_YSQUARES, &sizeyfail, FALSE);
            refund = GetDlgItemInt(hwnd, IDC_REFUND, &refundfail, TRUE);
            cost = GetDlgItemInt(hwnd, IDC_COST, &costfail, TRUE);
            unk31 = GetDlgItemInt(hwnd, IDC_UNK31, &unk31fail, FALSE);
            unk32 = GetDlgItemInt(hwnd, IDC_UNK32, &unk32fail, FALSE);
            unk33 = GetDlgItemInt(hwnd, IDC_UNK33, &unk33fail, FALSE);
            dunk2 = GetDlgItemInt(hwnd, IDC_DUNK2, &dunk2fail, TRUE);
            dunk3 = GetDlgItemInt(hwnd, IDC_DUNK3, &dunk3fail, FALSE);
            dunk4 = GetDlgItemInt(hwnd, IDC_DUNK4, &dunk4fail, TRUE);
            dunk5 = GetDlgItemInt(hwnd, IDC_DUNK5, &dunk5fail, FALSE);
            if (enabled == true) {
                wallicon = SendDlgItemMessage(hwnd, IDC_WALLICON, CB_GETCURSEL, 0, 0);
                wallname = SendDlgItemMessage(hwnd, IDC_WALLNAME, CB_GETCURSEL, 0, 0);
            } else {
                wallicon = 0;
                wallname = 0;
            }
            icon = SendDlgItemMessage(hwnd, IDC_SCENICON, CB_GETCURSEL, 0, 0);
            name = SendDlgItemMessage(hwnd, IDC_SCENNAME, CB_GETCURSEL, 0, 0);
            type = SendDlgItemMessage(hwnd, IDC_TYPE, CB_GETCURSEL, 0, 0);
            unk4 = SendDlgItemMessage(hwnd, IDC_UNK4, CB_GETCURSEL, 0, 0);
            supportstype = SendDlgItemMessage(hwnd, IDC_SUPPORTS, CB_GETCURSEL, 0, 0);
            unk17 = SendDlgItemMessage(hwnd, IDC_UNK17, CB_GETCURSEL, 0, 0);
            sizeflag = SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_GETCURSEL, 0, 0);
            extra = SendDlgItemMessage(hwnd, IDC_PARAMS, CB_GETCURSEL, 0, 0);
            if ((ovllen != 0) && (loclen != 0) && (posxlen != 0)
                    && (posylen != 0) && (poszlen != 0) && (sizexlen != 0)
                    && (sizeylen != 0) && (sizezlen != 0)
                    && (unk31fail == TRUE)
                    && (unk32fail == TRUE) && (unk33fail == TRUE)
                    && (dunk2fail == TRUE) && (dunk3fail == TRUE)
                    && (dunk4fail == TRUE) && (dunk5fail == TRUE)
                    && (sizexfail == TRUE) && (sizeyfail == TRUE)
                    && (refundfail == TRUE) && (costfail == TRUE)
                    && (icon != -1)
                    && (name != -1) && (type != -1) && (unk4 != -1)
                    && (supportstype != -1) && (unk17 != -1)
                    && (sizeflag != -1)
                    && (extra != -1)) {
                if ((enabled == false)
                        || ((enabled == true) && (wallicon != -1)
                            && (wallname != -1))) {
                    save = true;
                    if (CurrentScenery != -1) {
                        delete SceneryItems[CurrentScenery]->name;
                        delete SceneryItems[CurrentScenery]->ovl;
                        delete SceneryItems[CurrentScenery]->location;
                        delete SceneryItems[CurrentScenery]->icon;
                        if (SceneryItems[CurrentScenery]->wallicon != 0) {
                            delete SceneryItems[CurrentScenery]->wallicon;
                            delete SceneryItems[CurrentScenery]->wallname;
                        }
                    } else {
                        Scenery *t = new Scenery;
                        SceneryItems.push_back(t);
                        CurrentScenery = SceneryItems.size() - 1;
                    }
                    Scenery *s = SceneryItems[CurrentScenery];
                    s->ovl = new char[ovllen + 1];
                    GetDlgItemText(hwnd, IDC_OVL, s->ovl, ovllen + 1);
                    s->location = new char[loclen + 1];
                    GetDlgItemText(hwnd, IDC_LOCATION, s->location, loclen + 1);
                    temp = new char[posxlen + 1];
                    GetDlgItemText(hwnd, IDC_XPOS, temp, posxlen + 1);
                    s->position.x = (float) atof(temp);
                    delete temp;
                    temp = new char[posylen + 1];
                    GetDlgItemText(hwnd, IDC_YPOS, temp, posylen + 1);
                    s->position.y = (float) atof(temp);
                    delete temp;
                    temp = new char[poszlen + 1];
                    GetDlgItemText(hwnd, IDC_XPOS, temp, poszlen + 1);
                    s->position.z = (float) atof(temp);
                    delete temp;
                    temp = new char[sizexlen + 1];
                    GetDlgItemText(hwnd, IDC_XSIZE, temp, sizexlen + 1);
                    s->size.x = (float) atof(temp);
                    delete temp;
                    temp = new char[sizeylen + 1];
                    GetDlgItemText(hwnd, IDC_YSIZE, temp, sizeylen + 1);
                    s->size.y = (float) atof(temp);
                    delete temp;
                    temp = new char[sizezlen + 1];
                    GetDlgItemText(hwnd, IDC_ZSIZE, temp, sizezlen + 1);
                    s->size.z = (float) atof(temp);
                    delete temp;
                    s->cost = cost;
                    s->refund = refund;
                    s->sizex = sizex;
                    s->sizey = sizey;
                    s->unk31 = unk31;
                    s->unk32 = unk32;
                    s->unk33 = unk33;
                    s->dunk2 = dunk2;
                    s->dunk3 = dunk3;
                    if (dunk4 == 0xFFFFFFFF) {
                        dunk4 = 0;
                    }
                    if (IsDlgButtonChecked(hwnd, IDC_DDUNK4)) {
                        dunk4 = 0xFFFFFFFF;
                    }
                    s->dunk4 = dunk4;
                    s->dunk5 = dunk5;
                    s->flag1 = IsDlgButtonChecked(hwnd, IDC_FLAG1);
                    s->flag2 = IsDlgButtonChecked(hwnd, IDC_FLAG2);
                    s->flag3 = IsDlgButtonChecked(hwnd, IDC_FLAG3);
                    s->flag4 = IsDlgButtonChecked(hwnd, IDC_FLAG4);
                    s->flag5 = IsDlgButtonChecked(hwnd, IDC_FLAG5);
                    s->flag6 = IsDlgButtonChecked(hwnd, IDC_FLAG6);
                    s->flag7 = IsDlgButtonChecked(hwnd, IDC_FLAG7);
                    s->flag8 = IsDlgButtonChecked(hwnd, IDC_FLAG8);
                    s->flag9 = IsDlgButtonChecked(hwnd, IDC_FLAG9);
                    s->flag10 = IsDlgButtonChecked(hwnd, IDC_FLAG10);
                    s->flag11 = IsDlgButtonChecked(hwnd, IDC_FLAG11);
                    s->flag12 = IsDlgButtonChecked(hwnd, IDC_FLAG12);
                    s->flag13 = IsDlgButtonChecked(hwnd, IDC_FLAG13);
                    s->flag14 = IsDlgButtonChecked(hwnd, IDC_FLAG14);
                    s->flag15 = IsDlgButtonChecked(hwnd, IDC_FLAG15);
                    s->flag16 = IsDlgButtonChecked(hwnd, IDC_FLAG16);
                    s->flag17 = IsDlgButtonChecked(hwnd, IDC_FLAG17);
                    s->flag18 = IsDlgButtonChecked(hwnd, IDC_FLAG18);
                    s->flag19 = IsDlgButtonChecked(hwnd, IDC_FLAG19);
                    s->flag20 = IsDlgButtonChecked(hwnd, IDC_FLAG20);
                    s->flag21 = IsDlgButtonChecked(hwnd, IDC_FLAG21);
                    s->flag22 = IsDlgButtonChecked(hwnd, IDC_FLAG22);
                    s->flag23 = IsDlgButtonChecked(hwnd, IDC_FLAG23);
                    s->flag24 = IsDlgButtonChecked(hwnd, IDC_FLAG24);
                    s->flag25 = IsDlgButtonChecked(hwnd, IDC_FLAG25);
                    s->flag26 = IsDlgButtonChecked(hwnd, IDC_FLAG26);
                    s->flag27 = IsDlgButtonChecked(hwnd, IDC_FLAG27);
                    s->flag28 = IsDlgButtonChecked(hwnd, IDC_FLAG28);
                    s->flag29 = IsDlgButtonChecked(hwnd, IDC_FLAG29);
                    s->flag30 = IsDlgButtonChecked(hwnd, IDC_FLAG30);
                    s->flag31 = IsDlgButtonChecked(hwnd, IDC_FLAG31);
                    s->flag32 = IsDlgButtonChecked(hwnd, IDC_FLAG32);
                    s->flag33 = IsDlgButtonChecked(hwnd, IDC_FLAG33);
                    s->flag34 = IsDlgButtonChecked(hwnd, IDC_FLAG34);
                    s->flag35 = IsDlgButtonChecked(hwnd, IDC_FLAG35);
                    s->flag36 = IsDlgButtonChecked(hwnd, IDC_FLAG36);
                    s->flag37 = IsDlgButtonChecked(hwnd, IDC_FLAG37);
                    s->flag38 = IsDlgButtonChecked(hwnd, IDC_FLAG38);
                    s->flag39 = IsDlgButtonChecked(hwnd, IDC_FLAG39);
                    s->flag40 = IsDlgButtonChecked(hwnd, IDC_FLAG40);
                    s->flag41 = IsDlgButtonChecked(hwnd, IDC_FLAG41);
                    s->flag42 = IsDlgButtonChecked(hwnd, IDC_FLAG42);
                    s->flag43 = IsDlgButtonChecked(hwnd, IDC_FLAG43);
                    s->flag44 = IsDlgButtonChecked(hwnd, IDC_FLAG44);
                    s->flag45 = IsDlgButtonChecked(hwnd, IDC_FLAG45);
                    s->flag46 = IsDlgButtonChecked(hwnd, IDC_FLAG46);
                    s->flag47 = IsDlgButtonChecked(hwnd, IDC_FLAG47);
                    s->flag48 = IsDlgButtonChecked(hwnd, IDC_FLAG48);
                    s->flag49 = IsDlgButtonChecked(hwnd, IDC_FLAG49);
                    s->flag50 = IsDlgButtonChecked(hwnd, IDC_FLAG50);
                    s->flag51 = IsDlgButtonChecked(hwnd, IDC_FLAG51);
                    s->flag52 = IsDlgButtonChecked(hwnd, IDC_FLAG52);
                    s->flag53 = IsDlgButtonChecked(hwnd, IDC_FLAG53);
                    s->flag54 = IsDlgButtonChecked(hwnd, IDC_FLAG54);
                    s->flag55 = IsDlgButtonChecked(hwnd, IDC_FLAG55);
                    s->flag56 = IsDlgButtonChecked(hwnd, IDC_FLAG56);
                    s->flag57 = IsDlgButtonChecked(hwnd, IDC_FLAG57);
                    s->flag58 = IsDlgButtonChecked(hwnd, IDC_FLAG58);
                    s->flag59 = IsDlgButtonChecked(hwnd, IDC_FLAG59);
                    s->flag60 = IsDlgButtonChecked(hwnd, IDC_FLAG60);
                    s->flag61 = IsDlgButtonChecked(hwnd, IDC_FLAG61);
                    s->flag62 = IsDlgButtonChecked(hwnd, IDC_FLAG62);
                    s->flag63 = IsDlgButtonChecked(hwnd, IDC_FLAG63);
                    s->flag64 = IsDlgButtonChecked(hwnd, IDC_FLAG64);
                    s->type = type;
                    s->unk4 = unk4;
                    s->supportstype = supportstype;
                    s->unk17 = unk17;
                    s->sizeflag = sizeflag;
                    s->extra = extra;
                    s->icon = new char[strlen(Icons[icon]->name) + 1];
                    strcpy(s->icon, Icons[icon]->name);
                    s->name = new char[strlen(TextStrings[name]->name) + 1];
                    strcpy(s->name, TextStrings[name]->name);
                    if (enabled == true) {
                        s->wallicon = new char[strlen(Icons[wallicon]->name) + 1];
                        strcpy(s->wallicon, Icons[wallicon]->name);
                        s->wallname = new char[strlen(TextStrings[wallname]->name) + 1];
                        strcpy(s->wallname, TextStrings[wallname]->name);
                    } else {
                        s->wallicon = 0;
                        s->wallname = 0;
                    }
                    EndDialog(hwnd, 1);
                }
            }
            break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        case IDC_SAVESID:
            posxlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_XPOS));
            posylen = GetWindowTextLength(GetDlgItem(hwnd, IDC_YPOS));
            poszlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ZPOS));
            sizexlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_XSIZE));
            sizeylen = GetWindowTextLength(GetDlgItem(hwnd, IDC_YSIZE));
            sizezlen = GetWindowTextLength(GetDlgItem(hwnd, IDC_ZSIZE));
            sizex = GetDlgItemInt(hwnd, IDC_XSQUARES, &sizexfail, FALSE);
            sizey = GetDlgItemInt(hwnd, IDC_YSQUARES, &sizeyfail, FALSE);
            unk31 = GetDlgItemInt(hwnd, IDC_UNK31, &unk31fail, FALSE);
            unk32 = GetDlgItemInt(hwnd, IDC_UNK32, &unk32fail, FALSE);
            unk33 = GetDlgItemInt(hwnd, IDC_UNK33, &unk33fail, FALSE);
            dunk2 = GetDlgItemInt(hwnd, IDC_DUNK2, &dunk2fail, TRUE);
            dunk3 = GetDlgItemInt(hwnd, IDC_DUNK3, &dunk3fail, FALSE);
            dunk4 = GetDlgItemInt(hwnd, IDC_DUNK4, &dunk4fail, TRUE);
            dunk5 = GetDlgItemInt(hwnd, IDC_DUNK5, &dunk5fail, FALSE);
            type = SendDlgItemMessage(hwnd, IDC_TYPE, CB_GETCURSEL, 0, 0);
            unk4 = SendDlgItemMessage(hwnd, IDC_UNK4, CB_GETCURSEL, 0, 0);
            unk17 = SendDlgItemMessage(hwnd, IDC_UNK17, CB_GETCURSEL, 0, 0);
            sizeflag = SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_GETCURSEL, 0, 0);
            extra = SendDlgItemMessage(hwnd, IDC_PARAMS, CB_GETCURSEL, 0, 0);
            if ((posxlen != 0) && (posylen != 0) && (poszlen != 0)
                    && (sizexlen != 0) && (sizeylen != 0) && (sizezlen != 0)
                    && (unk31fail == TRUE) && (unk32fail == TRUE)
                    && (unk33fail == TRUE) && (dunk2fail == TRUE)
                    && (dunk3fail == TRUE) && (dunk4fail == TRUE)
                    && (dunk5fail == TRUE) && (sizexfail == TRUE)
                    && (sizeyfail == TRUE) && (type != -1) && (unk4 != -1)
                    && (unk17 != -1) && (sizeflag != -1) && (extra != -1)) {
                OPENFILENAME ofn;
                ZeroMemory(&ofn, sizeof(ofn));
                ofn.lStructSize = sizeof(ofn);
                ofn.hwndOwner = hwnd;
                ofn.lpstrFilter = "SID Files (*.sid)\0*.sid\0";
                ofn.lpstrFile = sfile;
                ofn.lpstrTitle = "Save SID File";
                ofn.lpstrDefExt = ".sid";
                ofn.nMaxFile = MAX_PATH;
                ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
                if (GetSaveFileName(&ofn)) {
                    SetDlgItemText(hwnd, IDC_SID_SHOW_PATH, sfile);
                    SIDData *s = new SIDData;
                    temp = new char[posxlen + 1];
                    GetDlgItemText(hwnd, IDC_XPOS, temp, posxlen + 1);
                    s->position.x = (float) atof(temp);
                    delete temp;
                    temp = new char[posylen + 1];
                    GetDlgItemText(hwnd, IDC_YPOS, temp, posylen + 1);
                    s->position.y = (float) atof(temp);
                    delete temp;
                    temp = new char[poszlen + 1];
                    GetDlgItemText(hwnd, IDC_ZPOS, temp, poszlen + 1);
                    s->position.z = (float) atof(temp);
                    delete temp;
                    temp = new char[sizexlen + 1];
                    GetDlgItemText(hwnd, IDC_XSIZE, temp, sizexlen + 1);
                    s->size.x = (float) atof(temp);
                    delete temp;
                    temp = new char[sizeylen + 1];
                    GetDlgItemText(hwnd, IDC_YSIZE, temp, sizeylen + 1);
                    s->size.y = (float) atof(temp);
                    delete temp;
                    temp = new char[sizezlen + 1];
                    GetDlgItemText(hwnd, IDC_ZSIZE, temp, sizezlen + 1);
                    s->size.z = (float) atof(temp);
                    delete temp;
                    s->sizex = sizex;
                    s->sizey = sizey;
                    s->unk31 = unk31;
                    s->unk32 = unk32;
                    s->unk33 = unk33;
                    s->dunk2 = dunk2;
                    s->dunk3 = dunk3;
                    if (dunk4 == 0xFFFFFFFF) {
                        dunk4 = 0;
                    }
                    if (IsDlgButtonChecked(hwnd, IDC_DDUNK4)) {
                        dunk4 = 0xFFFFFFFF;
                    }
                    s->dunk4 = dunk4;
                    s->dunk5 = dunk5;
                    s->flag1 = IsDlgButtonChecked(hwnd, IDC_FLAG1);
                    s->flag2 = IsDlgButtonChecked(hwnd, IDC_FLAG2);
                    s->flag3 = IsDlgButtonChecked(hwnd, IDC_FLAG3);
                    s->flag4 = IsDlgButtonChecked(hwnd, IDC_FLAG4);
                    s->flag5 = IsDlgButtonChecked(hwnd, IDC_FLAG5);
                    s->flag6 = IsDlgButtonChecked(hwnd, IDC_FLAG6);
                    s->flag7 = IsDlgButtonChecked(hwnd, IDC_FLAG7);
                    s->flag8 = IsDlgButtonChecked(hwnd, IDC_FLAG8);
                    s->flag9 = IsDlgButtonChecked(hwnd, IDC_FLAG9);
                    s->flag10 = IsDlgButtonChecked(hwnd, IDC_FLAG10);
                    s->flag11 = IsDlgButtonChecked(hwnd, IDC_FLAG11);
                    s->flag12 = IsDlgButtonChecked(hwnd, IDC_FLAG12);
                    s->flag13 = IsDlgButtonChecked(hwnd, IDC_FLAG13);
                    s->flag14 = IsDlgButtonChecked(hwnd, IDC_FLAG14);
                    s->flag15 = IsDlgButtonChecked(hwnd, IDC_FLAG15);
                    s->flag16 = IsDlgButtonChecked(hwnd, IDC_FLAG16);
                    s->flag17 = IsDlgButtonChecked(hwnd, IDC_FLAG17);
                    s->flag18 = IsDlgButtonChecked(hwnd, IDC_FLAG18);
                    s->flag19 = IsDlgButtonChecked(hwnd, IDC_FLAG19);
                    s->flag20 = IsDlgButtonChecked(hwnd, IDC_FLAG20);
                    s->flag21 = IsDlgButtonChecked(hwnd, IDC_FLAG21);
                    s->flag22 = IsDlgButtonChecked(hwnd, IDC_FLAG22);
                    s->flag23 = IsDlgButtonChecked(hwnd, IDC_FLAG23);
                    s->flag24 = IsDlgButtonChecked(hwnd, IDC_FLAG24);
                    s->flag25 = IsDlgButtonChecked(hwnd, IDC_FLAG25);
                    s->flag26 = IsDlgButtonChecked(hwnd, IDC_FLAG26);
                    s->flag27 = IsDlgButtonChecked(hwnd, IDC_FLAG27);
                    s->flag28 = IsDlgButtonChecked(hwnd, IDC_FLAG28);
                    s->flag29 = IsDlgButtonChecked(hwnd, IDC_FLAG29);
                    s->flag30 = IsDlgButtonChecked(hwnd, IDC_FLAG30);
                    s->flag31 = IsDlgButtonChecked(hwnd, IDC_FLAG31);
                    s->flag32 = IsDlgButtonChecked(hwnd, IDC_FLAG32);
                    s->flag33 = IsDlgButtonChecked(hwnd, IDC_FLAG33);
                    s->flag34 = IsDlgButtonChecked(hwnd, IDC_FLAG34);
                    s->flag35 = IsDlgButtonChecked(hwnd, IDC_FLAG35);
                    s->flag36 = IsDlgButtonChecked(hwnd, IDC_FLAG36);
                    s->flag37 = IsDlgButtonChecked(hwnd, IDC_FLAG37);
                    s->flag38 = IsDlgButtonChecked(hwnd, IDC_FLAG38);
                    s->flag39 = IsDlgButtonChecked(hwnd, IDC_FLAG39);
                    s->flag40 = IsDlgButtonChecked(hwnd, IDC_FLAG40);
                    s->flag41 = IsDlgButtonChecked(hwnd, IDC_FLAG41);
                    s->flag42 = IsDlgButtonChecked(hwnd, IDC_FLAG42);
                    s->flag43 = IsDlgButtonChecked(hwnd, IDC_FLAG43);
                    s->flag44 = IsDlgButtonChecked(hwnd, IDC_FLAG44);
                    s->flag45 = IsDlgButtonChecked(hwnd, IDC_FLAG45);
                    s->flag46 = IsDlgButtonChecked(hwnd, IDC_FLAG46);
                    s->flag47 = IsDlgButtonChecked(hwnd, IDC_FLAG47);
                    s->flag48 = IsDlgButtonChecked(hwnd, IDC_FLAG48);
                    s->flag49 = IsDlgButtonChecked(hwnd, IDC_FLAG49);
                    s->flag50 = IsDlgButtonChecked(hwnd, IDC_FLAG50);
                    s->flag51 = IsDlgButtonChecked(hwnd, IDC_FLAG51);
                    s->flag52 = IsDlgButtonChecked(hwnd, IDC_FLAG52);
                    s->flag53 = IsDlgButtonChecked(hwnd, IDC_FLAG53);
                    s->flag54 = IsDlgButtonChecked(hwnd, IDC_FLAG54);
                    s->flag55 = IsDlgButtonChecked(hwnd, IDC_FLAG55);
                    s->flag56 = IsDlgButtonChecked(hwnd, IDC_FLAG56);
                    s->flag57 = IsDlgButtonChecked(hwnd, IDC_FLAG57);
                    s->flag58 = IsDlgButtonChecked(hwnd, IDC_FLAG58);
                    s->flag59 = IsDlgButtonChecked(hwnd, IDC_FLAG59);
                    s->flag60 = IsDlgButtonChecked(hwnd, IDC_FLAG60);
                    s->flag61 = IsDlgButtonChecked(hwnd, IDC_FLAG61);
                    s->flag62 = IsDlgButtonChecked(hwnd, IDC_FLAG62);
                    s->flag63 = IsDlgButtonChecked(hwnd, IDC_FLAG63);
                    s->flag64 = IsDlgButtonChecked(hwnd, IDC_FLAG64);
                    s->type = type;
                    s->unk4 = unk4;
                    s->unk17 = unk17;
                    s->sizeflag = sizeflag;
                    s->extra = extra;
                    FILE *f = fopen(sfile, "wb");
                    fwrite(s, sizeof(SIDData), 1, f);
                    fclose(f);
                }
            }
            break;
        case IDC_OPENSID:
            OPENFILENAME ofn;
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "SID Files (*.sid)\0*.sid\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open SID File";
            ofn.lpstrDefExt = ".sid";
            ofn.nMaxFile = MAX_PATH;
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SIDData *s = new SIDData;
                FILE *f = fopen(sfile, "rb");
                SetDlgItemText(hwnd, IDC_SID_SHOW_PATH, sfile);
                fread(s, sizeof(SIDData), 1, f);
                fclose(f);
                SetDlgItemInt(hwnd, IDC_XSQUARES, s->sizex, FALSE);
                SetDlgItemInt(hwnd, IDC_YSQUARES, s->sizey, FALSE);
                SetDlgItemInt(hwnd, IDC_UNK31, s->unk31, FALSE);
                SetDlgItemInt(hwnd, IDC_UNK32, s->unk32, FALSE);
                SetDlgItemInt(hwnd, IDC_UNK33, s->unk33, FALSE);
                CheckDlgButton(hwnd, IDC_FLAG33, s->flag33);
                CheckDlgButton(hwnd, IDC_FLAG34, s->flag34);
                CheckDlgButton(hwnd, IDC_FLAG35, s->flag35);
                CheckDlgButton(hwnd, IDC_FLAG36, s->flag36);
                CheckDlgButton(hwnd, IDC_FLAG37, s->flag37);
                CheckDlgButton(hwnd, IDC_FLAG38, s->flag38);
                CheckDlgButton(hwnd, IDC_FLAG39, s->flag39);
                CheckDlgButton(hwnd, IDC_FLAG40, s->flag40);
                CheckDlgButton(hwnd, IDC_FLAG41, s->flag41);
                CheckDlgButton(hwnd, IDC_FLAG42, s->flag42);
                CheckDlgButton(hwnd, IDC_FLAG43, s->flag43);
                CheckDlgButton(hwnd, IDC_FLAG44, s->flag44);
                CheckDlgButton(hwnd, IDC_FLAG45, s->flag45);
                CheckDlgButton(hwnd, IDC_FLAG46, s->flag46);
                CheckDlgButton(hwnd, IDC_FLAG47, s->flag47);
                CheckDlgButton(hwnd, IDC_FLAG48, s->flag48);
                CheckDlgButton(hwnd, IDC_FLAG49, s->flag49);
                CheckDlgButton(hwnd, IDC_FLAG50, s->flag50);
                CheckDlgButton(hwnd, IDC_FLAG51, s->flag51);
                CheckDlgButton(hwnd, IDC_FLAG52, s->flag52);
                CheckDlgButton(hwnd, IDC_FLAG53, s->flag53);
                CheckDlgButton(hwnd, IDC_FLAG54, s->flag54);
                CheckDlgButton(hwnd, IDC_FLAG55, s->flag55);
                CheckDlgButton(hwnd, IDC_FLAG56, s->flag56);
                CheckDlgButton(hwnd, IDC_FLAG57, s->flag57);
                CheckDlgButton(hwnd, IDC_FLAG58, s->flag58);
                CheckDlgButton(hwnd, IDC_FLAG59, s->flag59);
                CheckDlgButton(hwnd, IDC_FLAG60, s->flag60);
                CheckDlgButton(hwnd, IDC_FLAG61, s->flag61);
                CheckDlgButton(hwnd, IDC_FLAG62, s->flag62);
                CheckDlgButton(hwnd, IDC_FLAG63, s->flag63);
                CheckDlgButton(hwnd, IDC_FLAG64, s->flag64);
                SetDlgItemInt(hwnd, IDC_DUNK2, s->dunk2, TRUE);
                SetDlgItemInt(hwnd, IDC_DUNK3, s->dunk3, FALSE);
                SetDlgItemInt(hwnd, IDC_DUNK4, s->dunk4, TRUE);
                if (s->dunk4 == 0xFFFFFFFF) {
                    CheckDlgButton(hwnd, IDC_DDUNK4, true);
                    SetDlgItemInt(hwnd, IDC_DUNK4, 0, true);
                    EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), false);
                } else {
                    CheckDlgButton(hwnd, IDC_DDUNK4, false);
                    EnableWindow(GetDlgItem(hwnd, IDC_DUNK4), true);
                }
                SetDlgItemInt(hwnd, IDC_DUNK5, s->dunk5, FALSE);
                CheckDlgButton(hwnd, IDC_FLAG1, s->flag1);
                CheckDlgButton(hwnd, IDC_FLAG2, s->flag2);
                CheckDlgButton(hwnd, IDC_FLAG3, s->flag3);
                CheckDlgButton(hwnd, IDC_FLAG4, s->flag4);
                CheckDlgButton(hwnd, IDC_FLAG5, s->flag5);
                CheckDlgButton(hwnd, IDC_FLAG6, s->flag6);
                CheckDlgButton(hwnd, IDC_FLAG7, s->flag7);
                CheckDlgButton(hwnd, IDC_FLAG8, s->flag8);
                CheckDlgButton(hwnd, IDC_FLAG9, s->flag9);
                CheckDlgButton(hwnd, IDC_FLAG10, s->flag10);
                CheckDlgButton(hwnd, IDC_FLAG11, s->flag11);
                CheckDlgButton(hwnd, IDC_FLAG12, s->flag12);
                CheckDlgButton(hwnd, IDC_FLAG13, s->flag13);
                CheckDlgButton(hwnd, IDC_FLAG14, s->flag14);
                CheckDlgButton(hwnd, IDC_FLAG15, s->flag15);
                CheckDlgButton(hwnd, IDC_FLAG16, s->flag16);
                CheckDlgButton(hwnd, IDC_FLAG17, s->flag17);
                CheckDlgButton(hwnd, IDC_FLAG18, s->flag18);
                CheckDlgButton(hwnd, IDC_FLAG19, s->flag19);
                CheckDlgButton(hwnd, IDC_FLAG20, s->flag20);
                CheckDlgButton(hwnd, IDC_FLAG21, s->flag21);
                CheckDlgButton(hwnd, IDC_FLAG22, s->flag22);
                CheckDlgButton(hwnd, IDC_FLAG23, s->flag23);
                CheckDlgButton(hwnd, IDC_FLAG24, s->flag24);
                CheckDlgButton(hwnd, IDC_FLAG25, s->flag25);
                CheckDlgButton(hwnd, IDC_FLAG26, s->flag26);
                CheckDlgButton(hwnd, IDC_FLAG27, s->flag27);
                CheckDlgButton(hwnd, IDC_FLAG28, s->flag28);
                CheckDlgButton(hwnd, IDC_FLAG29, s->flag29);
                CheckDlgButton(hwnd, IDC_FLAG30, s->flag30);
                CheckDlgButton(hwnd, IDC_FLAG31, s->flag31);
                CheckDlgButton(hwnd, IDC_FLAG32, s->flag32);
                SendDlgItemMessage(hwnd, IDC_TYPE, CB_SETCURSEL, s->type, 0);
                SendDlgItemMessage(hwnd, IDC_UNK4, CB_SETCURSEL, s->unk4, 0);
                SendDlgItemMessage(hwnd, IDC_UNK17, CB_SETCURSEL, s->unk17, 0);
                s->sizeflag &= 0x0000FFFF;
                SendDlgItemMessage(hwnd, IDC_SIZEFLAG, CB_SETCURSEL, s->sizeflag, 0);
                SendDlgItemMessage(hwnd, IDC_PARAMS, CB_SETCURSEL, s->extra, 0);
                char temp[100];
                sprintf(temp, "%f", s->position.x);
                SetDlgItemText(hwnd, IDC_XPOS, temp);
                sprintf(temp, "%f", s->position.y);
                SetDlgItemText(hwnd, IDC_YPOS, temp);
                sprintf(temp, "%f", s->position.z);
                SetDlgItemText(hwnd, IDC_ZPOS, temp);
                sprintf(temp, "%f", s->size.x);
                SetDlgItemText(hwnd, IDC_XSIZE, temp);
                sprintf(temp, "%f", s->size.y);
                SetDlgItemText(hwnd, IDC_YSIZE, temp);
                sprintf(temp, "%f", s->size.z);
                SetDlgItemText(hwnd, IDC_ZSIZE, temp);
            }
            break;
        case IDC_SELECT:
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Overlay Files (*.common.ovl)\0*.common.ovl\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open OVL File";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".common.ovl";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                SetDlgItemText(hwnd, IDC_OVL, sfile);
            }
            break;
        case ID_EDIT_CREATESCENERY:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_SCENERY), hwnd, SceneryDlgProc);
            /*int res = DialogBox(hInst, MAKEINTRESOURCE(IDD_CREATESCENERY), hwnd, CreateSceneryDlgProc);
               if (res==1) //ok is pressed
               {
               SetDlgItemText(hwnd,IDC_OVL,LastCreatedOVL);
               delete LastCreatedOVL;
               LastCreatedOVL=NULL;
               } */

            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK SIDManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
        for (SceneryItemsIterator = SceneryItems.begin();
                SceneryItemsIterator != SceneryItems.end(); SceneryItemsIterator++) {
            SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                               (LPARAM) (*SceneryItemsIterator)->name);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_ADD:
            CurrentScenery = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDEDIT), hwnd, SIDEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (SceneryItemsIterator = SceneryItems.begin();
                        SceneryItemsIterator != SceneryItems.end(); SceneryItemsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*SceneryItemsIterator)->name);
                }
            }
            break;
        case IDC_LIST:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDIT:
            CurrentScenery = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDEDIT), hwnd, SIDEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (SceneryItemsIterator = SceneryItems.begin();
                        SceneryItemsIterator != SceneryItems.end(); SceneryItemsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*SceneryItemsIterator)->name);
                }
            }
            break;
        case IDC_DEL:
            if (MessageBox
                    (hwnd, "Are you sure you want to delete?", "Are You Sure",
                     MB_YESNO | MB_ICONQUESTION) == IDYES) {
                save = true;
                CurrentScenery = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
                delete SceneryItems[CurrentScenery]->name;
                delete SceneryItems[CurrentScenery]->ovl;
                delete SceneryItems[CurrentScenery]->location;
                delete SceneryItems[CurrentScenery]->icon;
                if (SceneryItems[CurrentScenery]->wallicon != 0) {
                    delete SceneryItems[CurrentScenery]->wallicon;
                    delete SceneryItems[CurrentScenery]->wallname;
                }
                delete SceneryItems[CurrentScenery];
                SceneryItems.erase(SceneryItems.begin() + CurrentScenery);
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (SceneryItemsIterator = SceneryItems.begin();
                        SceneryItemsIterator != SceneryItems.end(); SceneryItemsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*SceneryItemsIterator)->name);
                }
            }
            break;
        case IDC_CLOSE:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK TextureManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
        for (IconTexturesIterator = IconTextures.begin();
                IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
            SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                               (LPARAM) (*IconTexturesIterator)->name);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_ADD:
            CurrentIconTexture = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREEDIT), hwnd, TextureEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconTexturesIterator = IconTextures.begin();
                        IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconTexturesIterator)->name);
                }
            }
            break;
        case IDC_LIST:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDIT:
            CurrentIconTexture = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREEDIT), hwnd, TextureEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconTexturesIterator = IconTextures.begin();
                        IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconTexturesIterator)->name);
                }
            }
            break;
        case IDC_DEL:
            if (MessageBox
                    (hwnd, "Are you sure you want to delete?", "Are You Sure",
                     MB_YESNO | MB_ICONQUESTION) == IDYES) {
                save = true;
                CurrentIconTexture = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
                delete IconTextures[CurrentIconTexture]->filename;
                delete IconTextures[CurrentIconTexture]->name;
                delete IconTextures[CurrentIconTexture];
                IconTextures.erase(IconTextures.begin() + CurrentIconTexture);
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconTexturesIterator = IconTextures.begin();
                        IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconTexturesIterator)->name);
                }
            }
            break;
        case IDC_CLOSE:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK GSIAutoDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_GSIAUTOLIST, LB_RESETCONTENT, 0, 0);
        for (IconTexturesIterator = IconTextures.begin();
                IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
            SendDlgItemMessage(hwnd, IDC_GSIAUTOLIST, LB_ADDSTRING, 0, (LPARAM) (*IconTexturesIterator)->name);
        }
        SendDlgItemMessage(hwnd, IDC_GSIAUTOLIST, LB_SETSEL, 1, -1);
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDOK: {
            char temp[512];
            int i = 0;
            for (IconTexturesIterator = IconTextures.begin();
                    IconTexturesIterator != IconTextures.end(); IconTexturesIterator++) {
                if (SendDlgItemMessage(hwnd, IDC_GSIAUTOLIST, LB_GETSEL, i, 0)) {
                    int n = (*IconTexturesIterator)->fh.Width / 64;
                    for (int y = 0; y < n; y++)
                        for (int x = 0; x < n; x++) {
                            snprintf(temp, 511, "%s Nr%03d x%02d/y%02d",
                                     (*IconTexturesIterator)->name, (x + (y * n)) + 1, x + 1,
                                     y + 1);

                            Icon *t = new Icon;
                            Icons.push_back(t);
                            CurrentIcon = Icons.size() - 1;
                            Icons[CurrentIcon]->name = new char[strlen(temp) + 1];
                            strcpy(Icons[CurrentIcon]->name, temp);
                            Icons[CurrentIcon]->top = (y * 64);
                            Icons[CurrentIcon]->left = (x * 64);
                            Icons[CurrentIcon]->bottom = ((y + 1) * 64) - 1;
                            Icons[CurrentIcon]->right = ((x + 1) * 64) - 1;
                            Icons[CurrentIcon]->texture =
                                new char[strlen((*IconTexturesIterator)->name) + 1];
                            strcpy(Icons[CurrentIcon]->texture, (*IconTexturesIterator)->name);
                        }
                }
                i++;
            }
            EndDialog(hwnd, 1);
        }
        break;
        case IDCANCEL:
            EndDialog(hwnd, 0);
            break;
        }
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK GSIManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
        for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
            SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0, (LPARAM) (*IconsIterator)->name);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_ADD:
            CurrentIcon = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIEDIT), hwnd, GSIEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconsIterator)->name);
                }
            }
            break;
        case IDC_AUTO_ICONS: {
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIAUTO), hwnd, GSIAutoDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconsIterator)->name);
                }
            }
        }
        break;
        case IDC_LIST:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDIT:
            CurrentIcon = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIEDIT), hwnd, GSIEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconsIterator)->name);
                }
            }
            break;
        case IDC_DEL:
            if (MessageBox
                    (hwnd, "Are you sure you want to delete?", "Are You Sure",
                     MB_YESNO | MB_ICONQUESTION) == IDYES) {
                save = true;
                CurrentIcon = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
                delete Icons[CurrentIcon]->name;
                delete Icons[CurrentIcon]->texture;
                delete Icons[CurrentIcon];
                Icons.erase(Icons.begin() + CurrentIcon);
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (IconsIterator = Icons.begin(); IconsIterator != Icons.end(); IconsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*IconsIterator)->name);
                }
            }
            break;
        case IDC_CLOSE:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK TextManDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    switch (Message) {
    case WM_INITDIALOG:
        SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
        for (TextStringsIterator = TextStrings.begin();
                TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
            SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                               (LPARAM) (*TextStringsIterator)->name);
        }
        break;
    case WM_CLOSE:
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDC_ADD:
            CurrentTextString = -1;
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTEDIT), hwnd, TextEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (TextStringsIterator = TextStrings.begin();
                        TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*TextStringsIterator)->name);
                }
            }
            break;
        case IDC_LIST:
            if (((int) HIWORD(wParam)) != LBN_DBLCLK)
                break;
            // fall through if double click
        case IDC_EDIT:
            CurrentTextString = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
            if (DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTEDIT), hwnd, TextEditDlgProc) == 1) {
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (TextStringsIterator = TextStrings.begin();
                        TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*TextStringsIterator)->name);
                }
            }
            break;
        case IDC_DEL:
            if (MessageBox
                    (hwnd, "Are you sure you want to delete?", "Are You Sure",
                     MB_YESNO | MB_ICONQUESTION) == IDYES) {
                save = true;
                CurrentTextString = SendDlgItemMessage(hwnd, IDC_LIST, LB_GETCURSEL, 0, 0);
                delete TextStrings[CurrentTextString]->name;
                delete TextStrings[CurrentTextString]->text;
                delete TextStrings[CurrentTextString];
                TextStrings.erase(TextStrings.begin() + CurrentTextString);
                SendDlgItemMessage(hwnd, IDC_LIST, LB_RESETCONTENT, 0, 0);
                for (TextStringsIterator = TextStrings.begin();
                        TextStringsIterator != TextStrings.end(); TextStringsIterator++) {
                    SendDlgItemMessage(hwnd, IDC_LIST, LB_ADDSTRING, 0,
                                       (LPARAM) (*TextStringsIterator)->name);
                }
            }
            break;
        case IDC_CLOSE:
            EndDialog(hwnd, 0);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

BOOL CALLBACK MainDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
    OPENFILENAME ofn;
    char sfile[MAX_PATH] = "";
    unsigned long i;
    switch (Message) {
    case WM_INITDIALOG:
        save = false;
        SetMenu(hwnd, LoadMenu(hInst, MAKEINTRESOURCE(IDR_MAIN)));
        SetWindowText(hwnd, TITLE_VERSION);
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "Generic");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "Western");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "Spooky");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "Sci-fi");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "Adventure");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "$Atlantis");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "$Paradise Island");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "#Safari");
        SendDlgItemMessage(hwnd, IDC_THEME, LB_ADDSTRING, 0, (LPARAM) "#Prehistoric");
        SetDlgItemText(hwnd, IDC_NOTES,
                       "Notes about RCT3 Soaked! and RCT3 Wild! expansion packs\r\nItems marked with a # will only work with the Wild! addon.\r\nItems marked with a $ will only work with the Soaked! addon.\r\nAll features assume the most recent patch is installed (either the\r\nnew Soaked! patch or the RCT3 Patch 3) as this adds features\r\n(such as billboards in Soaked! and normal RCT3 and tunnels and\r\nspeakers in normal RCT3) not otherwise available.");
        break;
    case WM_CLOSE:
        if (save == true) {
            if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO) == IDNO) {
                break;
            }
        }
        EndDialog(hwnd, 0);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case ID_FILE_EXIT:
            if (save == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            EndDialog(hwnd, 0);
            break;
        case ID_FILE_NEWTHEME:
            if (save == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            save = false;
            for (i = 0; i < SceneryItems.size(); i++) {
                delete SceneryItems[i]->name;
                delete SceneryItems[i]->ovl;
                delete SceneryItems[i]->location;
                delete SceneryItems[i]->icon;
                if (SceneryItems[i]->wallicon != 0) {
                    delete SceneryItems[i]->wallicon;
                    delete SceneryItems[i]->wallname;
                }
                delete SceneryItems[i];
            }
            for (i = 0; i < IconTextures.size(); i++) {
                delete IconTextures[i]->filename;
                delete IconTextures[i]->name;
                delete IconTextures[i];
            }
            for (i = 0; i < Icons.size(); i++) {
                delete Icons[i]->name;
                delete Icons[i]->texture;
                delete Icons[i];
            }
            for (i = 0; i < TextStrings.size(); i++) {
                delete TextStrings[i]->name;
                delete TextStrings[i]->text;
                delete TextStrings[i];
            }
            SetDlgItemText(hwnd, IDC_THEMENAME, "");
            SendDlgItemMessage(hwnd, IDC_THEME, LB_SETCURSEL, (WPARAM) - 1, 0);
            TextStrings.clear();
            Icons.clear();
            IconTextures.clear();
            SceneryItems.clear();
            break;
        case ID_FILE_SAVETHEME: {
            unsigned long style = SendDlgItemMessage(hwnd, IDC_THEME, LB_GETCURSEL, 0,
                                  0);
            unsigned long len = GetWindowTextLength(GetDlgItem(hwnd, IDC_THEMENAME));
            if ((SceneryItems.size() == 0) || (style == -1)
                    || (len == 0)) {
                if (SceneryItems.size() == 0)
                    MessageBox(hwnd,
                               "Unable to save because of missing data.\nNo scenery items.",
                               "Error", MB_OK);
                else if (style == -1)
                    MessageBox(hwnd,
                               "Unable to save because of missing data.\nNo theme type selected.",
                               "Error", MB_OK);
                else
                    MessageBox(hwnd,
                               "Unable to save because of missing data.\nTheme name missing.",
                               "Error", MB_OK);
                break;
            }
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Theme Files (*.thm)\0*.thm\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Save Theme file";
            ofn.lpstrDefExt = ".thm";
            ofn.nMaxFile = MAX_PATH;
            ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT;
            if (GetSaveFileName(&ofn)) {
                save = false;
                FILE *f = fopen(sfile, "wb");
                fwrite(&style, 4, 1, f);
                len++;
                char *temp = new char[len];
                GetDlgItemText(hwnd, IDC_THEMENAME, temp, len);
                fwrite(&len, 4, 1, f);
                fwrite(temp, len, 1, f);
                delete temp;
                len = TextStrings.size();
                fwrite(&len, 4, 1, f);
                for (i = 0; i < TextStrings.size(); i++) {
                    len = strlen(TextStrings[i]->name);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(TextStrings[i]->name, len, 1, f);
                    len = strlen(TextStrings[i]->text);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(TextStrings[i]->text, len, 1, f);
                }
                len = IconTextures.size();
                fwrite(&len, 4, 1, f);
                for (i = 0; i < IconTextures.size(); i++) {
                    len = strlen(IconTextures[i]->name);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(IconTextures[i]->name, len, 1, f);
                    len = strlen(IconTextures[i]->filename);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(IconTextures[i]->filename, len, 1, f);
                }
                len = Icons.size();
                fwrite(&len, 4, 1, f);
                for (i = 0; i < Icons.size(); i++) {
                    len = strlen(Icons[i]->name);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(Icons[i]->name, len, 1, f);
                    len = strlen(Icons[i]->texture);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(Icons[i]->texture, len, 1, f);
                    fwrite(&Icons[i]->top, 4, 1, f);
                    fwrite(&Icons[i]->left, 4, 1, f);
                    fwrite(&Icons[i]->bottom, 4, 1, f);
                    fwrite(&Icons[i]->right, 4, 1, f);
                }
                len = SceneryItems.size();
                fwrite(&len, 4, 1, f);
                for (unsigned long i = 0; i < SceneryItems.size(); i++) {
                    Scenery *s = SceneryItems[i];
                    len = strlen(s->ovl);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(s->ovl, len, 1, f);
                    if (s->wallicon != 0) {
                        len = strlen(s->wallicon);
                        len++;
                        fwrite(&len, 4, 1, f);
                        fwrite(s->wallicon, len, 1, f);
                    } else {
                        len = 0;
                        fwrite(&len, 4, 1, f);
                    }
                    len = strlen(s->icon);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(s->icon, len, 1, f);
                    len = strlen(s->name);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(s->name, len, 1, f);
                    if (s->wallname != 0) {
                        len = strlen(s->wallname);
                        len++;
                        fwrite(&len, 4, 1, f);
                        fwrite(s->wallname, len, 1, f);
                    } else {
                        len = 0;
                        fwrite(&len, 4, 1, f);
                    }
                    len = strlen(s->location);
                    len++;
                    fwrite(&len, 4, 1, f);
                    fwrite(s->location, len, 1, f);
                    fwrite(&s->cost, 4, 1, f);
                    fwrite(&s->refund, 4, 1, f);
                    fwrite(&s->type, 4, 1, f);
                    fwrite(&s->unk4, 4, 1, f);
                    fwrite(&s->supportstype, 4, 1, f);
                    fwrite(&s->unk17, 4, 1, f);
                    fwrite(&s->sizex, 4, 1, f);
                    fwrite(&s->sizey, 4, 1, f);
                    fwrite(&s->sizeflag, 4, 1, f);
                    fwrite(&s->extra, 4, 1, f);
                    fwrite(&s->size.x, 4, 1, f);
                    fwrite(&s->size.y, 4, 1, f);
                    fwrite(&s->size.z, 4, 1, f);
                    fwrite(&s->position.x, 4, 1, f);
                    fwrite(&s->position.y, 4, 1, f);
                    fwrite(&s->position.z, 4, 1, f);
                    fwrite(&s->flag1, 4, 1, f);
                    fwrite(&s->flag2, 4, 1, f);
                    fwrite(&s->flag3, 4, 1, f);
                    fwrite(&s->flag4, 4, 1, f);
                    fwrite(&s->flag5, 4, 1, f);
                    fwrite(&s->flag6, 4, 1, f);
                    fwrite(&s->flag7, 4, 1, f);
                    fwrite(&s->flag8, 4, 1, f);
                    fwrite(&s->flag9, 4, 1, f);
                    fwrite(&s->flag10, 4, 1, f);
                    fwrite(&s->flag11, 4, 1, f);
                    fwrite(&s->flag12, 4, 1, f);
                    fwrite(&s->flag13, 4, 1, f);
                    fwrite(&s->flag14, 4, 1, f);
                    fwrite(&s->flag15, 4, 1, f);
                    fwrite(&s->flag16, 4, 1, f);
                    fwrite(&s->flag17, 4, 1, f);
                    fwrite(&s->flag18, 4, 1, f);
                    fwrite(&s->flag19, 4, 1, f);
                    fwrite(&s->flag20, 4, 1, f);
                    fwrite(&s->flag21, 4, 1, f);
                    fwrite(&s->flag22, 4, 1, f);
                    fwrite(&s->flag23, 4, 1, f);
                    fwrite(&s->flag24, 4, 1, f);
                    fwrite(&s->flag25, 4, 1, f);
                    fwrite(&s->flag26, 4, 1, f);
                    fwrite(&s->flag27, 4, 1, f);
                    fwrite(&s->flag28, 4, 1, f);
                    fwrite(&s->flag29, 4, 1, f);
                    fwrite(&s->flag30, 4, 1, f);
                    fwrite(&s->flag31, 4, 1, f);
                    fwrite(&s->flag32, 4, 1, f);
                    fwrite(&s->flag33, 4, 1, f);
                    fwrite(&s->flag34, 4, 1, f);
                    fwrite(&s->flag35, 4, 1, f);
                    fwrite(&s->flag36, 4, 1, f);
                    fwrite(&s->flag37, 4, 1, f);
                    fwrite(&s->flag38, 4, 1, f);
                    fwrite(&s->flag39, 4, 1, f);
                    fwrite(&s->flag40, 4, 1, f);
                    fwrite(&s->flag41, 4, 1, f);
                    fwrite(&s->flag42, 4, 1, f);
                    fwrite(&s->flag43, 4, 1, f);
                    fwrite(&s->flag44, 4, 1, f);
                    fwrite(&s->flag45, 4, 1, f);
                    fwrite(&s->flag46, 4, 1, f);
                    fwrite(&s->flag47, 4, 1, f);
                    fwrite(&s->flag48, 4, 1, f);
                    fwrite(&s->flag49, 4, 1, f);
                    fwrite(&s->flag50, 4, 1, f);
                    fwrite(&s->flag51, 4, 1, f);
                    fwrite(&s->flag52, 4, 1, f);
                    fwrite(&s->flag53, 4, 1, f);
                    fwrite(&s->flag54, 4, 1, f);
                    fwrite(&s->flag55, 4, 1, f);
                    fwrite(&s->flag56, 4, 1, f);
                    fwrite(&s->flag57, 4, 1, f);
                    fwrite(&s->flag58, 4, 1, f);
                    fwrite(&s->flag59, 4, 1, f);
                    fwrite(&s->flag60, 4, 1, f);
                    fwrite(&s->flag61, 4, 1, f);
                    fwrite(&s->flag62, 4, 1, f);
                    fwrite(&s->flag63, 4, 1, f);
                    fwrite(&s->flag64, 4, 1, f);
                    fwrite(&s->unk31, 4, 1, f);
                    fwrite(&s->unk32, 4, 1, f);
                    fwrite(&s->unk33, 4, 1, f);
                    fwrite(&s->dunk2, 4, 1, f);
                    fwrite(&s->dunk3, 4, 1, f);
                    fwrite(&s->dunk4, 4, 1, f);
                    fwrite(&s->dunk5, 4, 1, f);
                }
                len = GetWindowTextLength(GetDlgItem(hwnd, IDC_THEMEPREFIX)) + 1;
                fwrite(&len, 4, 1, f);
                if (len > 0) {
                    temp = new char[len];
                    GetDlgItemText(hwnd, IDC_THEMEPREFIX, temp, len);
                    fwrite(temp, len, 1, f);
                    delete temp;
                }
                fclose(f);
            }
        }
        break;
        case ID_FILE_OPENTHEME:
            if (save == true) {
                if (MessageBox(hwnd, "Unsaved changes, continue?", "Unsaved changes", MB_YESNO)
                        == IDNO) {
                    break;
                }
            }
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Theme Files (*.thm)\0*.thm\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Theme File";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".thm";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                save = false;
                for (i = 0; i < SceneryItems.size(); i++) {
                    delete SceneryItems[i]->name;
                    if (SceneryItems[i]->ovl != 0) {
                        delete SceneryItems[i]->ovl;
                    }
                    delete SceneryItems[i]->location;
                    delete SceneryItems[i]->icon;
                    if (SceneryItems[i]->wallicon != 0) {
                        delete SceneryItems[i]->wallicon;
                        delete SceneryItems[i]->wallname;
                    }
                    delete SceneryItems[i];
                }
                for (i = 0; i < IconTextures.size(); i++) {
                    delete IconTextures[i]->filename;
                    delete IconTextures[i]->name;
                    delete IconTextures[i];
                }
                for (i = 0; i < Icons.size(); i++) {
                    delete Icons[i]->name;
                    delete Icons[i]->texture;
                    delete Icons[i];
                }
                for (i = 0; i < TextStrings.size(); i++) {
                    delete TextStrings[i]->name;
                    delete TextStrings[i]->text;
                    delete TextStrings[i];
                }
                TextStrings.clear();
                Icons.clear();
                IconTextures.clear();
                SceneryItems.clear();
                FILE *f = fopen(sfile, "rb");
                unsigned long style;
                unsigned long len;
                char *temp;
                fread(&style, 4, 1, f);
                fread(&len, 4, 1, f);
                temp = new char[len];
                fread(temp, len, 1, f);
                SendDlgItemMessage(hwnd, IDC_THEME, LB_SETCURSEL, style, 0);
                SetDlgItemText(hwnd, IDC_THEMENAME, temp);
                delete temp;
                unsigned long len2;
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Text *t = new Text;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->text = new char[len2];
                    fread(t->text, len2, 1, f);
                    TextStrings.push_back(t);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    IconTexture *t = new IconTexture;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->filename = new char[len2];
                    fread(t->filename, len2, 1, f);

                    ReadIconTexture(t);

                    IconTextures.push_back(t);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Icon *t = new Icon;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->texture = new char[len2];
                    fread(t->texture, len2, 1, f);
                    Icons.push_back(t);
                    fread(&t->top, 4, 1, f);
                    fread(&t->left, 4, 1, f);
                    fread(&t->bottom, 4, 1, f);
                    fread(&t->right, 4, 1, f);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Scenery *t = new Scenery;
                    fread(&len2, 4, 1, f);
                    t->ovl = new char[len2];
                    fread(t->ovl, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    if (len2 != 0) {
                        t->wallicon = new char[len2];
                        fread(t->wallicon, len2, 1, f);
                    } else {
                        t->wallicon = 0;
                    }
                    fread(&len2, 4, 1, f);
                    t->icon = new char[len2];
                    fread(t->icon, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    if (len2 != 0) {
                        t->wallname = new char[len2];
                        fread(t->wallname, len2, 1, f);
                    } else {
                        t->wallname = 0;
                    }
                    fread(&len2, 4, 1, f);
                    t->location = new char[len2];
                    fread(t->location, len2, 1, f);
                    fread(&t->cost, 4, 1, f);
                    fread(&t->refund, 4, 1, f);
                    fread(&t->type, 4, 1, f);
                    fread(&t->unk4, 4, 1, f);
                    fread(&t->supportstype, 4, 1, f);
                    fread(&t->unk17, 4, 1, f);
                    fread(&t->sizex, 4, 1, f);
                    fread(&t->sizey, 4, 1, f);
                    fread(&t->sizeflag, 4, 1, f);
                    fread(&t->extra, 4, 1, f);
                    fread(&t->size.x, 4, 1, f);
                    fread(&t->size.y, 4, 1, f);
                    fread(&t->size.z, 4, 1, f);
                    fread(&t->position.x, 4, 1, f);
                    fread(&t->position.y, 4, 1, f);
                    fread(&t->position.z, 4, 1, f);
                    fread(&t->flag1, 4, 1, f);
                    fread(&t->flag2, 4, 1, f);
                    fread(&t->flag3, 4, 1, f);
                    fread(&t->flag4, 4, 1, f);
                    fread(&t->flag5, 4, 1, f);
                    fread(&t->flag6, 4, 1, f);
                    fread(&t->flag7, 4, 1, f);
                    fread(&t->flag8, 4, 1, f);
                    fread(&t->flag9, 4, 1, f);
                    fread(&t->flag10, 4, 1, f);
                    fread(&t->flag11, 4, 1, f);
                    fread(&t->flag12, 4, 1, f);
                    fread(&t->flag13, 4, 1, f);
                    fread(&t->flag14, 4, 1, f);
                    fread(&t->flag15, 4, 1, f);
                    fread(&t->flag16, 4, 1, f);
                    fread(&t->flag17, 4, 1, f);
                    fread(&t->flag18, 4, 1, f);
                    fread(&t->flag19, 4, 1, f);
                    fread(&t->flag20, 4, 1, f);
                    fread(&t->flag21, 4, 1, f);
                    fread(&t->flag22, 4, 1, f);
                    fread(&t->flag23, 4, 1, f);
                    fread(&t->flag24, 4, 1, f);
                    fread(&t->flag25, 4, 1, f);
                    fread(&t->flag26, 4, 1, f);
                    fread(&t->flag27, 4, 1, f);
                    fread(&t->flag28, 4, 1, f);
                    fread(&t->flag29, 4, 1, f);
                    fread(&t->flag30, 4, 1, f);
                    fread(&t->flag31, 4, 1, f);
                    fread(&t->flag32, 4, 1, f);
                    fread(&t->flag33, 4, 1, f);
                    fread(&t->flag34, 4, 1, f);
                    fread(&t->flag35, 4, 1, f);
                    fread(&t->flag36, 4, 1, f);
                    fread(&t->flag37, 4, 1, f);
                    fread(&t->flag38, 4, 1, f);
                    fread(&t->flag39, 4, 1, f);
                    fread(&t->flag40, 4, 1, f);
                    fread(&t->flag41, 4, 1, f);
                    fread(&t->flag42, 4, 1, f);
                    fread(&t->flag43, 4, 1, f);
                    fread(&t->flag44, 4, 1, f);
                    fread(&t->flag45, 4, 1, f);
                    fread(&t->flag46, 4, 1, f);
                    fread(&t->flag47, 4, 1, f);
                    fread(&t->flag48, 4, 1, f);
                    fread(&t->flag49, 4, 1, f);
                    fread(&t->flag50, 4, 1, f);
                    fread(&t->flag51, 4, 1, f);
                    fread(&t->flag52, 4, 1, f);
                    fread(&t->flag53, 4, 1, f);
                    fread(&t->flag54, 4, 1, f);
                    fread(&t->flag55, 4, 1, f);
                    fread(&t->flag56, 4, 1, f);
                    fread(&t->flag57, 4, 1, f);
                    fread(&t->flag58, 4, 1, f);
                    fread(&t->flag59, 4, 1, f);
                    fread(&t->flag60, 4, 1, f);
                    fread(&t->flag61, 4, 1, f);
                    fread(&t->flag62, 4, 1, f);
                    fread(&t->flag63, 4, 1, f);
                    fread(&t->flag64, 4, 1, f);
                    fread(&t->unk31, 4, 1, f);
                    fread(&t->unk32, 4, 1, f);
                    fread(&t->unk33, 4, 1, f);
                    fread(&t->dunk2, 4, 1, f);
                    fread(&t->dunk3, 4, 1, f);
                    fread(&t->dunk4, 4, 1, f);
                    fread(&t->dunk5, 4, 1, f);
                    SceneryItems.push_back(t);
                }
                if (!feof(f)) {
                    len = 0;
                    if (!fread(&len, 4, 1, f))
                        len = 0;
                    if (len > 0) {
                        temp = new char[len];
                        fread(temp, len, 1, f);
                        SetDlgItemText(hwnd, IDC_THEMEPREFIX, temp);
                        delete temp;
                    }
                }
                fclose(f);
            }
            break;
        case ID_FILE_OPENTHEME2:
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = hwnd;
            ofn.lpstrFilter = "Theme Files (*.thm)\0*.thm\0All Files (*.*)\0*.*\0";
            ofn.lpstrFile = sfile;
            ofn.lpstrTitle = "Open Theme File";
            ofn.nMaxFile = MAX_PATH;
            ofn.lpstrDefExt = ".thm";
            ofn.Flags =
                OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_DONTADDTORECENT |
                OFN_NOCHANGEDIR;
            if (GetOpenFileName(&ofn)) {
                save = true;
                FILE *f = fopen(sfile, "rb");
                unsigned long style;
                unsigned long len;
                char *temp;
                fread(&style, 4, 1, f);
                fread(&len, 4, 1, f);
                temp = new char[len];
                fread(temp, len, 1, f);
                SendDlgItemMessage(hwnd, IDC_THEME, LB_SETCURSEL, style, 0);
                SetDlgItemText(hwnd, IDC_THEMENAME, temp);
                delete temp;
                unsigned long len2;
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Text *t = new Text;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->text = new char[len2];
                    fread(t->text, len2, 1, f);
                    TextStrings.push_back(t);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    IconTexture *t = new IconTexture;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->filename = new char[len2];
                    fread(t->filename, len2, 1, f);

                    ReadIconTexture(t);

                    IconTextures.push_back(t);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Icon *t = new Icon;
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->texture = new char[len2];
                    fread(t->texture, len2, 1, f);
                    Icons.push_back(t);
                    fread(&t->top, 4, 1, f);
                    fread(&t->left, 4, 1, f);
                    fread(&t->bottom, 4, 1, f);
                    fread(&t->right, 4, 1, f);
                }
                fread(&len, 4, 1, f);
                for (i = 0; i < len; i++) {
                    Scenery *t = new Scenery;
                    fread(&len2, 4, 1, f);
                    t->ovl = new char[len2];
                    fread(t->ovl, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    if (len2 != 0) {
                        t->wallicon = new char[len2];
                        fread(t->wallicon, len2, 1, f);
                    } else {
                        t->wallicon = 0;
                    }
                    fread(&len2, 4, 1, f);
                    t->icon = new char[len2];
                    fread(t->icon, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    t->name = new char[len2];
                    fread(t->name, len2, 1, f);
                    fread(&len2, 4, 1, f);
                    if (len2 != 0) {
                        t->wallname = new char[len2];
                        fread(t->wallname, len2, 1, f);
                    } else {
                        t->wallname = 0;
                    }
                    fread(&len2, 4, 1, f);
                    t->location = new char[len2];
                    fread(t->location, len2, 1, f);
                    fread(&t->cost, 4, 1, f);
                    fread(&t->refund, 4, 1, f);
                    fread(&t->type, 4, 1, f);
                    fread(&t->unk4, 4, 1, f);
                    fread(&t->supportstype, 4, 1, f);
                    fread(&t->unk17, 4, 1, f);
                    fread(&t->sizex, 4, 1, f);
                    fread(&t->sizey, 4, 1, f);
                    fread(&t->sizeflag, 4, 1, f);
                    fread(&t->extra, 4, 1, f);
                    fread(&t->size.x, 4, 1, f);
                    fread(&t->size.y, 4, 1, f);
                    fread(&t->size.z, 4, 1, f);
                    fread(&t->position.x, 4, 1, f);
                    fread(&t->position.y, 4, 1, f);
                    fread(&t->position.z, 4, 1, f);
                    fread(&t->flag1, 4, 1, f);
                    fread(&t->flag2, 4, 1, f);
                    fread(&t->flag3, 4, 1, f);
                    fread(&t->flag4, 4, 1, f);
                    fread(&t->flag5, 4, 1, f);
                    fread(&t->flag6, 4, 1, f);
                    fread(&t->flag7, 4, 1, f);
                    fread(&t->flag8, 4, 1, f);
                    fread(&t->flag9, 4, 1, f);
                    fread(&t->flag10, 4, 1, f);
                    fread(&t->flag11, 4, 1, f);
                    fread(&t->flag12, 4, 1, f);
                    fread(&t->flag13, 4, 1, f);
                    fread(&t->flag14, 4, 1, f);
                    fread(&t->flag15, 4, 1, f);
                    fread(&t->flag16, 4, 1, f);
                    fread(&t->flag17, 4, 1, f);
                    fread(&t->flag18, 4, 1, f);
                    fread(&t->flag19, 4, 1, f);
                    fread(&t->flag20, 4, 1, f);
                    fread(&t->flag21, 4, 1, f);
                    fread(&t->flag22, 4, 1, f);
                    fread(&t->flag23, 4, 1, f);
                    fread(&t->flag24, 4, 1, f);
                    fread(&t->flag25, 4, 1, f);
                    fread(&t->flag26, 4, 1, f);
                    fread(&t->flag27, 4, 1, f);
                    fread(&t->flag28, 4, 1, f);
                    fread(&t->flag29, 4, 1, f);
                    fread(&t->flag30, 4, 1, f);
                    fread(&t->flag31, 4, 1, f);
                    fread(&t->flag32, 4, 1, f);
                    fread(&t->flag33, 4, 1, f);
                    fread(&t->flag34, 4, 1, f);
                    fread(&t->flag35, 4, 1, f);
                    fread(&t->flag36, 4, 1, f);
                    fread(&t->flag37, 4, 1, f);
                    fread(&t->flag38, 4, 1, f);
                    fread(&t->flag39, 4, 1, f);
                    fread(&t->flag40, 4, 1, f);
                    fread(&t->flag41, 4, 1, f);
                    fread(&t->flag42, 4, 1, f);
                    fread(&t->flag43, 4, 1, f);
                    fread(&t->flag44, 4, 1, f);
                    fread(&t->flag45, 4, 1, f);
                    fread(&t->flag46, 4, 1, f);
                    fread(&t->flag47, 4, 1, f);
                    fread(&t->flag48, 4, 1, f);
                    fread(&t->flag49, 4, 1, f);
                    fread(&t->flag50, 4, 1, f);
                    fread(&t->flag51, 4, 1, f);
                    fread(&t->flag52, 4, 1, f);
                    fread(&t->flag53, 4, 1, f);
                    fread(&t->flag54, 4, 1, f);
                    fread(&t->flag55, 4, 1, f);
                    fread(&t->flag56, 4, 1, f);
                    fread(&t->flag57, 4, 1, f);
                    fread(&t->flag58, 4, 1, f);
                    fread(&t->flag59, 4, 1, f);
                    fread(&t->flag60, 4, 1, f);
                    fread(&t->flag61, 4, 1, f);
                    fread(&t->flag62, 4, 1, f);
                    fread(&t->flag63, 4, 1, f);
                    fread(&t->flag64, 4, 1, f);
                    fread(&t->unk31, 4, 1, f);
                    fread(&t->unk32, 4, 1, f);
                    fread(&t->unk33, 4, 1, f);
                    fread(&t->dunk2, 4, 1, f);
                    fread(&t->dunk3, 4, 1, f);
                    fread(&t->dunk4, 4, 1, f);
                    fread(&t->dunk5, 4, 1, f);
                    SceneryItems.push_back(t);
                }
                if (!feof(f)) {
                    len = 0;
                    if (!fread(&len, 4, 1, f))
                        len = 0;
                    if (len > 0) {
                        temp = new char[len];
                        fread(temp, len, 1, f);
                        SetDlgItemText(hwnd, IDC_THEMEPREFIX, temp);
                        delete temp;
                    }
                }
                fclose(f);
            }
            break;
        case ID_FILE_INSTALL:
            unsigned long styleval, len;
            styleval = SendDlgItemMessage(hwnd, IDC_THEME, LB_GETCURSEL, 0, 0);
            len = GetWindowTextLength(GetDlgItem(hwnd, IDC_THEMENAME));
            if ((SceneryItems.size() == 0) || (styleval == -1)
                    || (len == 0)) {
                if (SceneryItems.size() == 0)
                    MessageBox(hwnd, "Unable to save because of missing data.\nNo scenery items.",
                               "Error", MB_OK);
                else if (styleval == -1)
                    MessageBox(hwnd,
                               "Unable to save because of missing data.\nNo theme type selected.",
                               "Error", MB_OK);
                else
                    MessageBox(hwnd, "Unable to save because of missing data.\nTheme name missing.",
                               "Error", MB_OK);
            } else {
                if (InstallTheme(hwnd))
                    MessageBox(hwnd, "Theme successfully installed.", "Completed", MB_OK);	//added by KLN

            }
            break;
        case ID_EDIT_MANAGESCENERY:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_SIDMAN), hwnd, SIDManDlgProc);
            break;
        case ID_EDIT_MANAGETEXTURES:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTUREMAN), hwnd, TextureManDlgProc);
            break;
        case ID_EDIT_MANAGEREFERENCES:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_GSIMAN), hwnd, GSIManDlgProc);
            break;
        case ID_EDIT_MANAGESTRINGS:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_TEXTMAN), hwnd, TextManDlgProc);
            break;
        case ID_EDIT_CREATESCENERY:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_SCENERY), hwnd, SceneryDlgProc);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    ilInit();
    iluInit();
    hInst = hInstance;

    CoInitializeEx(NULL, COINIT_APARTMENTTHREADED); // We need this for directory selection

    // Load/Create Palettes
    // Blue/Magenta/Yellow Palette
    HRSRC hp = FindResource(hInst, MAKEINTRESOURCE(IDR_PALETTE_BMY), "PALETTE");
    HGLOBAL hp2 = LoadResource(hInst, hp);
    void *hp3 = LockResource(hp2);
    char tmprgb[3] = "\0\0";	// Last 0 is the 0-terminator =)
    ILuint tmp = ilGenImage();
    ilBindImage(tmp);
    ilTexImage(1, 1, 1, 3, IL_RGB, IL_UNSIGNED_BYTE, &tmprgb);
    ilConvertImage(IL_COLOUR_INDEX, IL_UNSIGNED_BYTE);
    ilConvertPal(IL_PAL_RGB24);
    memcpy(ilGetPalette(), hp3, 256 * 3);
    ilConvertPal(IL_PAL_BGR32);
    memcpy(&recolpalette_bmy, ilGetPalette(), 256 * 4);
    ilDeleteImage(tmp);
    // Red/Green/Blue Palette
    ZeroMemory(&recolpalette_rgb, sizeof(recolpalette_bmy));
    int i;
    for (i = 1; i <= 85; i++) {
        recolpalette_rgb[i].rgbRed = ((86 - i) * 255) / 85;
        recolpalette_rgb[i + 85].rgbGreen = ((86 - i) * 255) / 85;
        recolpalette_rgb[i + 170].rgbBlue = ((86 - i) * 255) / 85;
    }
    /*
       char tmptxt[256];
       snprintf(tmptxt, 256, "Palette Colours: %d %d %d\n", (int)recolpalette_rgb[20].rgbRed, (int)recolpalette_rgb[50].rgbRed, (int)recolpalette_rgb[70].rgbRed);
       MessageBox(NULL, tmptxt, "Test", MB_OK);
     */
    DialogBox(hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC) MainDlgProc);
}

bool InstallTheme(HWND hwnd) {
    unsigned long styleval, len;
    char InstallLocation[MAX_PATH] = "";
    char Location[MAX_PATH] = "";
    HKEY key;

    styleval = SendDlgItemMessage(hwnd, IDC_THEME, LB_GETCURSEL, 0, 0);
    len =
        GetWindowTextLength(GetDlgItem(hwnd, IDC_THEMENAME)) +
        GetWindowTextLength(GetDlgItem(hwnd, IDC_THEMEPREFIX)) + 1;
    char *theme = new char[len];
    GetDlgItemText(hwnd, IDC_THEMEPREFIX, theme, len);
    GetDlgItemText(hwnd, IDC_THEMENAME, InstallLocation, len);
    strcat(theme, InstallLocation);

    LONG res = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                            "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\{907B4640-266B-4A21-92FB-CD1A86CD0F63}",
                            0, KEY_QUERY_VALUE, &key);
    unsigned long length = MAX_PATH;
    unsigned long i;
    res = RegQueryValueEx(key, "InstallLocation", 0, NULL, (LPBYTE) InstallLocation, &length);
    if (res != ERROR_SUCCESS) {
        MessageBox(hwnd, "RCT3 install directory not found. Please select a direcory where the theme should be installed.\n\"\\Style\\Themed\" will be appended automatically.", "Error", MB_OK | MB_ICONERROR);
        BROWSEINFO bi;
        ZeroMemory(&bi, sizeof(bi));
        bi.hwndOwner = hwnd;
        bi.pszDisplayName = InstallLocation;
        bi.lpszTitle = "Select install directory for theme. Do not select links.";
        bi.ulFlags = 0;
        LPITEMIDLIST pids = SHBrowseForFolder(&bi);
        if (pids) {
            if (SHGetPathFromIDList(pids, InstallLocation)) {
                CoTaskMemFree(pids);
            } else {
                MessageBox(hwnd, "Invalid Folder selected.", "Error", MB_OK | MB_ICONERROR);
                res = RegCloseKey(key);
                return false;
            }
        } else {
            res = RegCloseKey(key);
            return false;
        }
    }
    res = RegCloseKey(key);
    unsigned long stringtablesize = 0;

    char FileName[MAX_PATH];
    sprintf(FileName, "%s\\%s%s\\", InstallLocation, stdstyle, theme);
    strcat(FileName, "style.common.ovl");

    CreateStyleOvl(FileName, styleval);
    for (i = 0; i < TextStrings.size(); i++) {
        Text *txt = TextStrings[i];
        AddStyleOVLInfo_TextString(txt);
    }
    for (i = 0; i < IconTextures.size(); i++) {
        IconTexture *icot = IconTextures[i];
        AddStyleOVLInfo_IconTexture(icot);
    }
    for (i = 0; i < Icons.size(); i++) {
        Icon *ico = Icons[i];
        AddStyleOVLInfo_Icon(ico);
    }
    for (unsigned long i = 0; i < SceneryItems.size(); i++) {
        char fname[_MAX_FNAME];
        Scenery *sc = SceneryItems[i];
        char *ovlfile = sc->ovl;
        _splitpath(sc->ovl, NULL, NULL, fname, NULL);
        strchr(fname, '.')[0] = 0;
        stringtablesize = 0;
        stringtablesize += strlen(fname) + 5;
        stringtablesize += strlen(fname) + 5;
        stringtablesize +=
            strlen(stdstyle) + strlen(theme) + 1 + strlen(SceneryItems[i]->location) + 1 +
            strlen(fname) + 1;


        sc->ovl2 = new char[stringtablesize];
        sc->ovlWithoutExt = new char[stringtablesize - 10];
        sc->InternalOVLName = new char[strlen(fname) + 1];
        strcpy(sc->InternalOVLName, fname);

        char *strings = sc->ovl2;
        char *fullstrings = strings;

        strcpy(strings, stdstyle);
        strings += strlen(stdstyle);
        strcpy(strings, theme);
        strings += strlen(theme);
        strcpy(strings, "\\");
        strings++;
        strcpy(strings, sc->location);
        strings += strlen(sc->location);
        sprintf(Location, "%s\\%s", InstallLocation, fullstrings);
        sprintf(sc->ovlWithoutExt, "%s\\%s", fullstrings, fname);
        SHCreateDirectoryEx(hwnd, Location, NULL);
        strcpy(strings, "\\");
        strings++;
        strcpy(strings, fname);
        strings += strlen(fname) + 1;
        strcat(Location, "\\");
        strcat(Location, fname);
        strcat(Location, ".common.ovl");
        CopyFile(ovlfile, Location, FALSE);
        strcpy(strstr(Location, ".common.ovl"), ".unique.ovl");
        strcpy(strstr(ovlfile, ".common.ovl"), ".unique.ovl");
        CopyFile(ovlfile, Location, FALSE);
        strcpy(strstr(ovlfile, ".unique.ovl"), ".common.ovl");
        sc->ovl3 = new char[strlen(Location) + 1];
        strcpy(sc->ovl3, Location);
        strcpy(strstr(sc->ovl3, ".unique.ovl"), ".common.ovl");

        AddStyleOVLInfo_SceneryItem(sc);
    }

    saveStyleOVL();

    return true;
}
unsigned long local_log2(unsigned long y) {
    unsigned long result = 0;
    while (y) {
        result++;
        y >>= 1;
    }
    return result - 1;
}

bool ReadIconTexture(IconTexture * t) {
    ILuint tex;
    ILenum Error;

    ilGenImages(1, &tex);
    ilBindImage(tex);

    if (!ilLoadImage(t->filename)) {
        MessageBox(SceneryDlgHwnd, "Can't open icon texture", "Error", MB_OK);
        while ((Error = ilGetError())) {
            MessageBox(SceneryDlgHwnd, iluErrorString(Error), "Error", MB_OK);
        }
        return false;
    }
    int width = ilGetInteger(IL_IMAGE_WIDTH);
    int height = ilGetInteger(IL_IMAGE_HEIGHT);
    if (width != height) {
        MessageBox(SceneryDlgHwnd, "Icon texture is not square", "Error", MB_OK);
        ilDeleteImages(1, &tex);
        return false;
    }
    if ((1 << local_log2(width)) != width) {
        MessageBox(SceneryDlgHwnd, "Icon texture's width/height is not a power of 2", "Error",
                   MB_OK);
        ilDeleteImages(1, &tex);
        return false;
    }
    if (!ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE)) {
        MessageBox(SceneryDlgHwnd, "Couldn't convert texture to RGBA", "Error", MB_OK);
        ilDeleteImages(1, &tex);
        return false;
    }
    if (ilGetInteger(IL_IMAGE_ORIGIN) == IL_ORIGIN_UPPER_LEFT)
        if (!iluFlipImage()) {
            MessageBox(SceneryDlgHwnd, "Couldn't flip icon texture", "Error", MB_OK);
            ilDeleteImages(1, &tex);
            return false;
        }
    int datasize = ilGetDXTCData(NULL, 0, IL_DXT3);
    data = new unsigned char[datasize];
    ilGetDXTCData(data, datasize, IL_DXT3);
    /*
       FILE *tex = fopen(t->filename,"rb");
       if (tex == 0)
       {
       MessageBox(SceneryDlgHwnd,"Can't open icon texture","Error",MB_OK);
       return false;
       }
       unsigned char header[8];
       fread(header, 1, 8, tex);
       bool is_png = !png_sig_cmp(header, 0, 8);
       if (!is_png)
       {
       MessageBox(SceneryDlgHwnd,"Icon Texture is not PNG file or is corrupt","Error",MB_OK);
       return false;
       }
       png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);
       if (!png_ptr)
       {
       MessageBox(SceneryDlgHwnd,"libpng error","Error",MB_OK);
       return false;
       }
       png_infop info_ptr = png_create_info_struct(png_ptr);
       if (!info_ptr)
       {
       png_destroy_read_struct(&png_ptr,(png_infopp)NULL,(png_infopp)NULL);
       MessageBox(SceneryDlgHwnd,"libpng error","Error",MB_OK);
       return false;
       }
       png_infop end_info = png_create_info_struct(png_ptr);
       if (!end_info)
       {
       png_destroy_read_struct(&png_ptr,&info_ptr,(png_infopp)NULL);
       MessageBox(SceneryDlgHwnd,"libpng error","Error",MB_OK);
       return false;
       }
       png_init_io(png_ptr,tex);
       png_set_sig_bytes(png_ptr, 8);
       png_read_info(png_ptr,info_ptr);
       unsigned long width;
       unsigned long height;
       int bit_depth;
       int color_type;
       png_get_IHDR(png_ptr,info_ptr,&width,&height,&bit_depth,&color_type,NULL,NULL,NULL);
       if ((width != height) || ((color_type != PNG_COLOR_TYPE_RGB) && (color_type != PNG_COLOR_TYPE_RGB_ALPHA)) || (width % 2))
       {
       MessageBox(SceneryDlgHwnd,"Icon Texture not square true color image with height/width a power of 2","Error",MB_OK);
       return false;
       }
       if (bit_depth == 16)
       {
       png_set_strip_16(png_ptr);
       }
       if (color_type == PNG_COLOR_TYPE_RGB)
       {
       png_set_add_alpha(png_ptr,0xFF,PNG_FILLER_AFTER);
       }
       png_set_bgr(png_ptr);
       png_read_update_info(png_ptr, info_ptr);
       unsigned long rowbytes = png_get_rowbytes(png_ptr,info_ptr);
       unsigned long size = height * rowbytes;
       data = new unsigned char[size];
       png_bytep *row_pointers = new png_bytep[height];
       for (unsigned long i=0;i<height;i++)
       {
       row_pointers[height-i-1] = data+(i*rowbytes);
       }
       png_read_image(png_ptr,row_pointers);
       png_read_end(png_ptr,end_info);
       png_destroy_read_struct(&png_ptr,&info_ptr,&end_info);
       fclose(tex);

       /*
       ATI_TC_Texture srcTexture;
       srcTexture.dwSize = sizeof(srcTexture);
       srcTexture.dwWidth = width;
       srcTexture.dwHeight = height;
       srcTexture.dwPitch = width * 4;
       srcTexture.format = ATI_TC_FORMAT_ARGB_8888;
       srcTexture.dwDataSize = ATI_TC_CalculateBufferSize(&srcTexture);
       if (srcTexture.dwDataSize != size)
       {
       MessageBox(SceneryDlgHwnd,"Texture Compression Error","Error",MB_OK);
       return false;
       }
       srcTexture.pData = data;
       ATI_TC_Texture destTexture;
       destTexture.dwSize = sizeof(destTexture);
       destTexture.dwWidth = width;
       destTexture.dwHeight = height;
       destTexture.dwPitch = 0;
       destTexture.format = ATI_TC_FORMAT_DXT3;
       unsigned long size2 = ATI_TC_CalculateBufferSize(&destTexture);
       destTexture.dwDataSize = size2;
       data2 = new unsigned char[destTexture.dwDataSize];
       destTexture.pData = data2;
       ATI_TC_ConvertTexture(&srcTexture, &destTexture, NULL, NULL, NULL, NULL);
     */

    /*
       // create the target data
       int bytesPerBlock = 16;
       int targetDataSize = bytesPerBlock*width*height/16;
       //Mem targetData( targetDataSize );
       int stride = 4;

       // loop over blocks and compress them
       u8* targetData = new u8[targetDataSize];
       u8* targetBlock = targetData;
       for( int y = 0; y < height; y += 4 )
       {
       // process a row of blocks
       for( int x = 0; x < width; x += 4 )
       {
       // get the block data
       u8 sourceRgba[16*4];
       for( int py = 0, i = 0; py < 4; ++py )
       {
       u8 const* row = (u8*)(data + y + py ) + x*stride;
       for( int px = 0; px < 4; ++px, ++i )
       {
       // get the pixel colour
       for( int j = 0; j < 3; ++j )
       sourceRgba[4*i + j] = *row++;

       sourceRgba[4*i + 3] = *row++;
       }
       }

       // compress this block
       Compress( sourceRgba, targetBlock, kDxt3 );

       // advance
       targetBlock += bytesPerBlock;
       }
       }
     */

    t->fh.Format = 0x13;
    t->fh.Height = height;
    t->fh.Width = width;
    t->fh.Mipcount = 0;
    t->fmh.Pitch = width * 4;
    t->fmh.Blocks = datasize / t->fmh.Pitch;
    t->fmh.MHeight = height;
    t->fmh.MWidth = width;
    t->data = data;

    ilDeleteImages(1, &tex);
    return true;
}
